@@ -100,11 +100,7 @@ class ProcManager(object):
 
     def _send_poison_pills(self):
         """flood the queue with poison pills to tell all workers to shut down"""
-        try:
-            async_coroutines = self.config.getint('performance', 'async_coroutines', 10)
-        except Exception as e:
-            async_coroutines = 1
-
+        async_coroutines = self.config.getint('performance', 'async_coroutines', fallback=1)
         try:
             for _ in range(min(len(self.workers)*async_coroutines, self.queuesize)):
                 # tasks queue is FIFO queue. As long as nothing is added to the queue
@@ -300,24 +296,12 @@ def fuglu_async_process_worker(queue, config, shared_state, child_to_server_mess
     logtools.client_configurer(logQueue)
     logging.basicConfig(level=logging.DEBUG)
 
-    try:
-        async_coroutines = config.getint('performance', 'async_coroutines', 10)
-    except Exception as e:
-        async_coroutines = 1
+    async_coroutines = config.getint('performance', 'async_coroutines', fallback=1)
 
-    try:
-        async_threadpoolworkers = config.getint('performance', 'async_threadpool', None)
-    except Exception as e:
-        async_threadpoolworkers = None
+    async_threadpoolworkers = config.getint('performance', 'async_threadpool', fallback=None)
     if not async_threadpoolworkers:
         async_threadpoolworkers = max(1, int(async_coroutines/2))
-
-    try:
-        async_prepare_timeout = config.getfloat('performance', 'async_prepare_timeout')
-    except Exception as e:
-        import traceback
-        traceback.print_exc()
-        async_prepare_timeout = None
+    async_prepare_timeout = config.getfloat('performance', 'async_prepare_timeout', fallback=None)
 
     workerstate = WorkerStateWrapper(shared_state,'loading configuration', async_coroutines=async_coroutines)
     logger = logging.getLogger('fuglu.process.%s(%u)' % (workerstate.process.name, workerstate.process.pid))
@@ -396,7 +380,11 @@ def fuglu_async_process_worker(queue, config, shared_state, child_to_server_mess
             # create worker coroutines for async loop
             asworkers = [fuglu_async_process_worker_main(queue, workerstate, logger, controller, i, logtimings, pool, async_prepare_timeout) for i in range(1, 1+async_coroutines)]
             logger.debug(f"We put {len(asworkers)} into the loop...")
-            loop.run_until_complete(asyncio.gather(*asworkers, loop=loop))
+            if sys.version_info > (3, 10):
+                # loop parameter has been removed for python-3.10
+                loop.run_until_complete(asyncio.gather(*asworkers, return_exceptions=True))
+            else:
+                loop.run_until_complete(asyncio.gather(*asworkers, loop=loop))
             logger.debug("Gathered result...")
     except KeyboardInterrupt:
         workerstate.workerstate = 'ended (keyboard interrupt)'
