@@ -23,7 +23,6 @@ import typing as tp
 import asyncio
 import traceback
 import time
-import datetime
 import functools
 import copy
 
@@ -32,7 +31,7 @@ from io import BytesIO
 
 from fuglu.protocolbase import ProtocolHandler
 from fuglu.stringencode import force_bString, force_uString
-from fuglu.shared import Suspect, DUNNO, REJECT, DELETE, DEFER, ACCEPT, string_to_actioncode
+from fuglu.shared import Suspect, DUNNO, REJECT, DELETE, DEFER, ACCEPT, string_to_actioncode, utcnow
 from fuglu.scansession import TrackTimings
 from configparser import ConfigParser
 from fuglu.logtools import createPIDinfo
@@ -262,7 +261,14 @@ class MilterHandler(TrackTimings):
 
         self.set_workerstate(message_prefix + u"Finishing message %s" % suspect)
 
-        return result, self.message
+        # enforce fugluid in return message
+        retmesg = "Rejected by content scanner"
+        if self.message is not None:
+            retmesg = self.message
+        if suspect.id not in retmesg:
+            # make sure fugluid is in return message
+            retmesg = f"{retmesg} ({suspect.id})"
+        return result, retmesg
 
     def run_plugins(self, suspect, pluglist):
         """Run scannerplugins on suspect"""
@@ -294,34 +300,34 @@ class MilterHandler(TrackTimings):
                 suspect.tags['decisions'].append((plugin.section, result))
 
                 if result == DUNNO:
-                    suspect.debug('Plugin makes no final decision')
-                    self.logger.debug(f'{suspect.id} Plugin makes no final decision')
+                    suspect.debug(f'Plugin {str(plugin)} makes no final decision')
+                    self.logger.debug(f'{suspect.id} Plugin {str(plugin)} makes no final decision')
                 elif result == ACCEPT:
-                    suspect.debug('Plugin accepts the message - skipping all further tests')
-                    self.logger.debug(f'{suspect.id} Plugin says: ACCEPT. Skipping all other tests')
+                    suspect.debug(f'Plugin {str(plugin)} accepts the message - skipping all further tests')
+                    self.logger.debug(f'{suspect.id} Plugin {str(plugin)} says: ACCEPT. Skipping all other tests')
                     self.action = ACCEPT
                     break
                 elif result == DELETE:
-                    suspect.debug('Plugin DELETES this message - no further tests')
-                    self.logger.debug(f'{suspect.id} Plugin says: DELETE. Skipping all other tests')
+                    suspect.debug(f'Plugin {str(plugin)} DELETES this message - no further tests')
+                    self.logger.debug(f'{suspect.id} Plugin {str(plugin)} says: DELETE. Skipping all other tests')
                     self.action = DELETE
                     self.message = message
                     self.trash(suspect, str(plugin))
                     break
                 elif result == REJECT:
-                    suspect.debug('Plugin REJECTS this message - no further tests')
-                    self.logger.debug(f'{suspect.id} Plugin says: REJECT. Skipping all other tests')
+                    suspect.debug(f'Plugin {str(plugin)} REJECTS this message - no further tests')
+                    self.logger.debug(f'{suspect.id} Plugin {str(plugin)} says: REJECT. Skipping all other tests')
                     self.action = REJECT
                     self.message = message
                     break
                 elif result == DEFER:
-                    suspect.debug('Plugin DEFERS this message - no further tests')
-                    self.logger.debug(f'{suspect.id} Plugin says: DEFER. Skipping all other tests')
+                    suspect.debug(f'Plugin {str(plugin)} DEFERS this message - no further tests')
+                    self.logger.debug(f'{suspect.id} Plugin {str(plugin)} says: DEFER. Skipping all other tests')
                     self.action = DEFER
                     self.message = message
                     break
                 else:
-                    self.logger.error(f'{suspect.id} Invalid Message action Code: %s. Using DUNNO' % result)
+                    self.logger.error(f'Plugin {str(plugin)} says invalid message action code: {result}. Using DUNNO')
 
             except Exception as e:
                 CrashStore.store_exception()
@@ -449,7 +455,7 @@ class MilterHandler(TrackTimings):
             try:
                 with open('%s/00-fuglutrash.log' % self.config.get('main', 'trashdir'), 'a') as handle:
                     # <date> <time> <from address> <to address> <plugin that said "DELETE"> <filename>
-                    now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+                    now = utcnow().isoformat()
                     handle.write("%s %s %s %s %s" % (
                         now, suspect.from_address, suspect.to_address, killerplugin, trashfilename))
                     handle.write("\n")
@@ -1278,8 +1284,6 @@ class MilterSession(lmp.ASYNCMilterProtocol):
                 self.logger.info(f"{self.id} {RCPT}-AddressCheck returns non-continue-result: {RETCODE2STR.get(res,f'unknown(orig:{res})')}, msg: {msg}")
                 return retcode
 
-        if recipient is not None:
-            self.recipients.append(recipient)
         plugins = self.mhandler.milterplugins.get(RCPT, [])
         self.log(f"{self.id} Plugins({RCPT}): {plugins}")
         self.mhandler.tracktime(f"Before-MPlugins({RCPT})")
@@ -1330,6 +1334,11 @@ class MilterSession(lmp.ASYNCMilterProtocol):
                     return lm.TEMPFAIL
                 return retcode
 
+        # add recipient here so in case of one recipient being rejected while others are accepted, the recipients
+        # array contains all the accepted recipients...
+        if recipient is not None:
+            self.recipients.append(recipient)
+
         return lm.CONTINUE
 
     async def header(self, key, val, command_dict):
@@ -1580,10 +1589,10 @@ class MilterSession(lmp.ASYNCMilterProtocol):
             if self.mhandler.pool:
                 # run in pool
                 loop = asyncio.get_event_loop()
-                self.logger.debug("Run prependers in Threadpool-executor")
+                self.logger.debug(f"{self.id} Run prependers in Threadpool-executor")
                 pluglist, applist = await loop.run_in_executor(self.mhandler.pool, self.mhandler.run_prependers, suspect)
                 # run plugins
-                self.logger.debug("Run Suspect plugins in Threadpool-executor")
+                self.logger.debug(f"{self.id} Run Suspect plugins in Threadpool-executor")
                 result, msg = await loop.run_in_executor(self.mhandler.pool,
                                                          functools.partial(self.mhandler.run_suspect_plugins,
                                                                            pluglist=pluglist,
@@ -1604,7 +1613,7 @@ class MilterSession(lmp.ASYNCMilterProtocol):
                 except Exception as e:
                     message_is_deferred = True
                     trb = traceback.format_exc()
-                    self.logger.error("Could not commit message. Error: %s" % trb)
+                    self.logger.error(f'{self.id} Could not commit message. Error: {trb}')
                     self.logger.exception(e)
                     await self._defer()
                     # reply with Deferred object (which does nothing) because we've already
@@ -1624,7 +1633,8 @@ class MilterSession(lmp.ASYNCMilterProtocol):
                 retmesg = "Rejected by content scanner"
                 if msg is not None:
                     retmesg = msg
-                retmesg = "%s (%s)" % (retmesg, suspect.id)
+                if suspect.id not in retmesg:
+                    retmesg = "%s (%s)" % (retmesg, suspect.id)
                 await self.reject(retmesg)
                 # reply with Deferred object (which does nothing) because we've already
                 # sent a reply
@@ -1636,8 +1646,7 @@ class MilterSession(lmp.ASYNCMilterProtocol):
                 # sent a reply
                 replycode = lm.Deferred()
             else:
-                self.logger.error(
-                    'Invalid Message action Code: %s. Using DEFER' % result)
+                self.logger.error(f'{self.id} Invalid Message action Code: {result}. Using DEFER')
                 message_is_deferred = True
                 await self._defer()
                 # reply with Deferred object (which does nothing) because we've already
@@ -1648,7 +1657,7 @@ class MilterSession(lmp.ASYNCMilterProtocol):
             if not message_is_deferred:
                 if self.mhandler.pool:
                     loop = asyncio.get_event_loop()
-                    self.logger.debug("Run Suspect appenders in Threadpool-executor")
+                    self.logger.debug(f"{self.id} Run Suspect appenders in Threadpool-executor")
                     await loop.run_in_executor(self.mhandler.pool,
                                                self.mhandler.run_appenders, suspect, result, applist)
                 else:
@@ -1934,8 +1943,9 @@ class MilterSession(lmp.ASYNCMilterProtocol):
             self.logger.warning(f"Add {len(msg)} headers from suspect mail")
             for key, val in iter(msg.items()):
                 self.logger.debug("Add header from msg-> %s: %s" % (key, val))
-                hdr = Header(val, header_name=key, continuation_ws=' ')
-                await self.addheader(key, hdr.encode())
+                if not isinstance(val, Header):
+                    val = Header(val, header_name=key, continuation_ws=' ')
+                await self.addheader(key, val.encode())
         # --
         # headers to add, same as for the other connectors
         # --
