@@ -118,7 +118,7 @@ class MilterHandler(ProtocolHandler):
             self.logger.debug("milter_mode: setting to default value: 'tags'")
             configstring = "tags"
 
-        if configstring not in ["auto", "readonly", "tags", "replace_demo", "manual"]:
+        if configstring not in ["auto", "readonly", "tags", "replace_demo", "manual", "autoheaders"]:
             self.logger.warning("milter_mode: '%s' not recognised, resetting to 'tags'" % configstring)
 
         self.enable_mode_manual = ("manual" == configstring)
@@ -137,9 +137,9 @@ class MilterHandler(ProtocolHandler):
 
         # options (can be combined into a string): "all" "body" "headers" "from" "to"
         try:
-            self.milter_mode_options = config.get('milter', 'milter_mode_options')
+            self.milter_mode_options = config.getlist('milter', 'milter_mode_options', lower=True)
         except Exception:
-            self.milter_mode_options = ""
+            self.milter_mode_options = []
 
         self.logger.debug("Milter config fixed replacements: all=%s, body=%s, headers=%s, from=%s, to=%s" %
                          ("all" in self.milter_mode_options, "body" in self.milter_mode_options,
@@ -998,8 +998,6 @@ class MilterSession(lm.MilterProtocol):
         self.log('RECIPIENT: %s, dict: %s' % (recipient, MilterSession.dict_unicode(command_dict)))
         self.store_info_from_dict(command_dict)
         recipient = self._clean_address(recipient)
-        if recipient is not None:
-            self.recipients.append(recipient)
         plugins = self.milterplugins.get(RCPT, [])
         for plug in plugins:
             if self.skip_plugin(plugin=plug):
@@ -1010,6 +1008,10 @@ class MilterSession(lm.MilterProtocol):
             if not res == lm.CONTINUE:
                 self.logger.info(f"{self.id} {RCPT}-Plugin {plug} returns non-continue-result: {RETCODE2STR.get(res,f'unknown(orig:{res})')}")
                 return retcode
+        # add recipient here so in case of one recipient being rejected while others are accepted, the recipients
+        # array contains all the accepted recipients...
+        if recipient is not None:
+            self.recipients.append(recipient)
 
         return lm.CONTINUE
 
