@@ -417,9 +417,7 @@ class SMTPSession(object):
         elif cmd == 'EHLO':
             self.state = SMTPSession.ST_HELO
             self.helo = data
-            helo = self.config.get('main', 'outgoinghelo')
-            if helo.strip() == '':
-                helo = socket.gethostname()
+            helo = get_outgoing_helo(self.config)
             if len(self.ehlo_options) > 0:
                 answer = [helo] + self.ehlo_options
                 rv = "250-"+"250-".join(a+"\n" for a in answer[:-1])+"250 %s" % answer[-1]
@@ -500,9 +498,10 @@ class SMTPSession(object):
             return None
     
     
+    _re_dots = re.compile(rb'(?m)^\.\.')
     def unquoteData(self, data):
         """two leading dots at the beginning of a line must be unquoted to a single dot"""
-        return re.sub(rb'(?m)^\.\.', b'.', force_bString(data))
+        return self._re_dots.sub(b'.', force_bString(data))
     
     
     def stripAddress(self, address):
