@@ -15,7 +15,6 @@
 #
 
 import code
-import datetime
 import inspect
 import logging
 import multiprocessing
@@ -37,7 +36,7 @@ from fuglu.connectors.ncconnector import NCServer
 from fuglu.connectors.smtpconnector import SMTPServer
 from fuglu.debug import ControlServer, CrashStore
 from fuglu.funkyconsole import FunkyConsole
-from fuglu.shared import HAVE_BEAUTIFULSOUP, Suspect, default_template_values
+from fuglu.shared import HAVE_BEAUTIFULSOUP, Suspect, default_template_values, utcnow
 from fuglu.mshared import BasicMilterPlugin
 from fuglu.stats import StatsThread
 from fuglu.stringencode import force_bString, force_uString
@@ -55,6 +54,8 @@ EXIT_FATAL = 66
 # other positive ints result from
 # counting errors in the setup
 
+
+_re_replycode = re.compile(r'^127\.0\.0\.(?P<replycode>\d{1,4})$')
 def check_version_status(lint=False):
     import pkg_resources
     """Check our version string in DNS for known issues and warn about them
@@ -130,7 +131,7 @@ def check_version_status(lint=False):
     if result is None:
         return
 
-    ret = re.match(r'^127\.0\.0\.(?P<replycode>\d{1,4})$', result)
+    ret = _re_replycode.match(result)
     if ret is not None:
         code = int(ret.groupdict()['replycode'])
         for bitmask, message in bitmaskmap.items():
@@ -306,6 +307,12 @@ class MainController(DefConfigMixin):
                 'description': "write suspect ID to every mail",
                 'default': "1",
             },
+            
+            'suspectidreturn': {
+                'section': 'main',
+                'description': "append suspect ID to return message (e.g. reject message)",
+                'default': "1",
+            },
 
             'mrtgdir': {
                 'section': 'main',
@@ -454,7 +461,13 @@ class MainController(DefConfigMixin):
             'dbpriority': {
                 'default': "sql",
                 'section': 'databaseconfig',
-                'description': """read runtime configuration values from database types specified in this order. currently supported db types: sql, rest""",
+                'description': """read runtime configuration values from database types specified in this order. currently supported db types: sql, rest, yaml""",
+            },
+            
+            'value_cache_ttl': {
+                'default': "30",
+                'section': 'databaseconfig',
+                'description': """cache values for given number of seconds. set to 0 to disable cache (not recommended)""",
             },
 
             'dbconnectstring': {
@@ -515,7 +528,7 @@ class MainController(DefConfigMixin):
             'restapi_headers': {
                 'default': "",
                 'section': 'databaseconfig',
-                'description': """list of additional headers, e.g. for API authentication""",
+                'description': """list of additional REST API headers, e.g. for authentication""",
                 'confidential': True,
             },
             
@@ -525,6 +538,18 @@ class MainController(DefConfigMixin):
                 'description': """verify REST API server's SSL certificate""",
             },
             
+            'restapi_timeout': {
+                'default': "10",
+                'section': 'databaseconfig',
+                'description': """REST API http connection timeout""",
+            },
+            
+            'restapi_cachettl': {
+                'default': "300",
+                'section': 'databaseconfig',
+                'description': """REST API caching timeout. Set to 0 to disable caching (not recommended)""",
+            },
+            
             # environment
             'boundarydistance': {
                 'default': "0",
@@ -698,16 +723,6 @@ class MainController(DefConfigMixin):
                 'section': 'PluginAlias',
             },
 
-            'm_ivmsg': {
-                'default': "fuglu.plugins.ivmsg.IVMSendgrid",
-                'section': 'PluginAlias',
-            },
-
-            'ivmfeed': {
-                'default': "fuglu.plugins.ivmsg.IVMFeed",
-                'section': 'PluginAlias',
-            },
-
             'knownsubject': {
                 'default': "fuglu.plugins.knownsubject.KnownSubject",
                 'section': 'PluginAlias',
@@ -889,7 +904,7 @@ class MainController(DefConfigMixin):
         self.procpool = None
         self.asyncprocpool = None
         self.controlserver = None
-        self.started = datetime.datetime.now()
+        self.started = utcnow()
         self.statsthread = None
         self.debugconsole = False
         self._logQueue = logQueue
@@ -992,16 +1007,11 @@ class MainController(DefConfigMixin):
 
     def _start_threadpool(self):
         self.logger.info("Init Threadpool")
-        try:
-            minthreads = self.config.getint('performance', 'minthreads')
-            maxthreads = self.config.getint('performance', 'maxthreads')
-            minfreethreads = self.config.getint('performance', 'minfreethreads')
-        except (configparser.NoSectionError, configparser.NoOptionError):
+        minthreads = self.config.getint('performance', 'minthreads', fallback=1)
+        maxthreads = self.config.getint('performance', 'maxthreads', fallback=3)
+        minfreethreads = self.config.getint('performance', 'minfreethreads', fallback=0)
+        if not self.config.has_section('performance'):
             self.logger.warning('Performance section not configured, using default thread numbers')
-            minthreads = 1
-            maxthreads = 3
-            minfreethreads = 0
-
         queuesize = maxthreads * 10
         return ThreadPool(self, minthreads=minthreads, maxthreads=maxthreads, queuesize=queuesize, freeworkers=minfreethreads)
 
@@ -1035,11 +1045,7 @@ class MainController(DefConfigMixin):
             numprocs = max(int(multiprocessing.cpu_count() * abs(numprocs)), 1)
             self.logger.info(f"Set number of worker processors for async procpool automatically ({abs(numprocs_in)}*#procs) to {numprocs}")
 
-        try:
-            async_coroutines = self.config.getint('performance', 'async_coroutines', 10)
-        except Exception as e:
-            async_coroutines = 1
-
+        async_coroutines = self.config.getint('performance', 'async_coroutines', fallback=1)
         queuesize = int(async_coroutines*numprocs*1.5)
         self.logger.info(f"Init async process pool with {numprocs} worker processes (queuesize={queuesize})")
         pool = fuglu.asyncprocpool.ProcManager(self._logQueue, numprocs=numprocs, config=self.config, queuesize=queuesize)
@@ -1578,31 +1584,31 @@ class MainController(DefConfigMixin):
 
         self.logger.debug('Module search path %s' % sys.path)
         self.logger.debug('Loading scanner plugins')
-        newplugins, loadok = self._load_all(self.config.getlist('main', 'plugins'))
+        newplugins, loadok = self._load_all(self.config.getlist('main', 'plugins', fallback=[]))
         if not loadok:
             allOK = False
 
-        newprependers, loadok = self._load_all(self.config.getlist('main', 'prependers'))
+        newprependers, loadok = self._load_all(self.config.getlist('main', 'prependers', fallback=[]))
         if not loadok:
             allOK = False
 
-        newappenders, loadok = self._load_all(self.config.getlist('main', 'appenders'))
+        newappenders, loadok = self._load_all(self.config.getlist('main', 'appenders', fallback=[]))
         if not loadok:
             allOK = False
 
-        try:
-            milterplugins, loadok = self._load_all(self.config.getlist('main', 'mplugins'))
-            if not loadok:
-                allOK = False
-        except (configparser.NoSectionError, configparser.NoOptionError):
-            milterplugins = []
+        milterplugins, loadok = self._load_all(self.config.getlist('main', 'mplugins', fallback=[]))
+        if not loadok:
+            allOK = False
 
         try:
             # create new dictionary with keys of all milter states
+            self.logger.info(f"MainController --> Setting up milter plugin states...")
             milterdict = dict((k, []) for k in BasicMilterPlugin.ALL_STATES.keys())
             for mplugin in milterplugins:
+                self.logger.info(f"Plugin: {str(mplugin)} has states: {mplugin.state}")
                 for s in mplugin.state:
                     milterdict[s].append(mplugin)
+
         except Exception as e:
             self.logger.error(f"ERROR: {e}", exc_info=e)
             milterdict = {}
@@ -1617,16 +1623,16 @@ class MainController(DefConfigMixin):
                 self._propagate_plugin_defaults_to_config()
 
         return allOK
-
+    
+    _re_config = re.compile(r"^(?P<structured_name>[a-zA-Z0-9._-]+)(?:\((?P<config_override>[a-zA-Z0-9._-]+)\))?$")
     def _load_all(self, plugins):
         """load all plugins from config string. returns tuple ([list of loaded instances],allOk)"""
         pluglist = []
-        config_re = re.compile(r"^(?P<structured_name>[a-zA-Z0-9._-]+)(?:\((?P<config_override>[a-zA-Z0-9._-]+)\))?$")
         allOK = True
         for plug in plugins:
             if plug == "":
                 continue
-            m = config_re.match(plug)
+            m = self._re_config.match(plug)
             if m is None:
                 self.logger.error('Invalid Plugin Syntax: %s' % plug)
                 allOK = False
