@@ -24,6 +24,7 @@ import string
 import os
 import weakref
 from fuglu.stringencode import force_bString, force_uString
+from fuglu.shared import utcnow
 
 try:
     import objgraph
@@ -242,7 +243,7 @@ class ControlSession(object):
 
     def uptime(self, args):
         start = self.controller.started
-        diff = datetime.datetime.now() - start
+        diff = utcnow() - start
         return "Fuglu was started on %s\nUptime: %s" % (start, diff)
 
     def exceptionlist(self, args):
@@ -260,7 +261,7 @@ class ControlSession(object):
 
     def stats(self, args):
         start = self.controller.started
-        runtime = datetime.datetime.now() - start
+        runtime = utcnow() - start
         stats = self.controller.statsthread.stats
         template = """Fuglu statistics
 ---------------
