@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*- #
-#   Copyright 2009-2021 Fumail Project
+#   Copyright 2009-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import zipfile
 import tarfile
 import re
 import os.path
+from unittest.mock import MagicMock
 
 STATUS = "available: zip, tar"
 ENABLED = True
@@ -39,12 +40,23 @@ except (ImportError, OSError):
 
 
 SEVENZIP_AVAILABLE = 0
+SEVENZIP_PACKAGE = None
 try:
-    import py7zlib # installed via pylzma library
+    #raise ImportError()
+    import py7zlib  # installed via pylzma library
     SEVENZIP_AVAILABLE = 1
+    SEVENZIP_PACKAGE = "py7zlib"
     STATUS += ", 7z"
 except (ImportError, OSError):
-    MISSING.append('7z')
+    py7zlib = MagicMock()
+    try:
+        import py7zr
+        SEVENZIP_AVAILABLE = 1
+        SEVENZIP_PACKAGE = "py7zr"
+        STATUS += ", 7z"
+    except (ImportError, OSError):
+        MISSING.append('7z')
+        py7zr = MagicMock()
     pass
 
 GZIP_AVAILABLE = 0
@@ -345,6 +357,85 @@ class Archive_7z(Archive_int):
         self._fdescriptor = None
 
 
+class Archive_7zr(Archive_int):
+    def __init__(self, filedescriptor, archivename=None):
+        super(Archive_7zr, self).__init__(filedescriptor, archivename)
+        self._fdescriptor = None
+        self._meta_protected = False
+        try:
+            self._handle = py7zr.SevenZipFile(filedescriptor)
+        except AttributeError:
+            self._fdescriptor = open(filedescriptor, 'rb')
+            self._handle = py7zr.SevenZipFile(self._fdescriptor)
+        except py7zr.exceptions.PasswordRequired:
+            self._meta_protected = True
+        except Exception as e:
+            # store setup exceptions like NoPasswordGivenError
+            raise Exception(str(e) if str(e).strip() else e.__class__.__name__)
+
+        if self._handle and self._archivename is None:
+            try:
+                self._archivename = os.path.basename(str(filedescriptor))
+            except Exception:
+                self._archivename = "generic.7z"
+
+    def protected_meta(self):
+        return self._meta_protected
+
+    def namelist(self):
+        """ Get archive file list
+
+        Returns:
+            (list) Returns a list of file paths within the archive
+        """
+
+        return self._handle.getnames()
+
+    def extract(self, path, archivecontentmaxsize):
+
+        if archivecontentmaxsize is not None and self.filesize(path) > archivecontentmaxsize:
+            return None
+        output = self._handle.read(path)
+        if path in output:
+            filecontent = output[path].getvalue()
+            return filecontent
+        return None
+
+    def filesize(self, path):
+        """get extracted file size
+
+        Args:
+            path (str): is the filename in the archive as returned by namelist
+        Returns:
+            (int) file size in bytes
+        """
+        filesize = 0
+        try:
+            list_in_archive = files_list
+        except Exception:
+            list_in_archive = []
+
+        for filecontent in self._handle.files:
+            try:
+                if filecontent.filename == path:
+                    filesize = filecontent.uncompressed
+            except KeyError:
+                pass
+        return filesize
+
+    def close(self):
+        """
+        Close handle
+        """
+        super(Archive_7zr, self).close()
+        if self._fdescriptor is not None:
+            try:
+                self._fdescriptor.close()
+            except Exception:
+                pass
+        self._fdescriptor = None
+
+
 class Archive_gz(Archive_int):
     def __init__(self, filedescriptor, archivename=None):
         super(Archive_gz, self).__init__(filedescriptor, archivename)
@@ -440,7 +531,7 @@ class Archive_gz(Archive_int):
             return 0
 
 class Archive_tgz():
-    """
+    r"""
     Archive combining tar and gzip extractor
 
     This archive combining gzip/tar extractor arises from a problem created when
@@ -533,7 +624,7 @@ class Archivehandle(object):
     archive_impl = {"zip": Archive_zip,
                     "rar": Archive_rar,
                     "tar": Archive_tar,
-                    "7z" : Archive_7z,
+                    "7z" : Archive_7z if SEVENZIP_PACKAGE == "py7zlib" else Archive_7zr,
                     "tgz": Archive_tgz,
                     "gz" : Archive_gz}
 
@@ -548,13 +639,13 @@ class Archivehandle(object):
 
     # key: regex matching content type as returned by file magic, value: archive type
     implemented_archive_ctypes = {
-        '^application\/zip': 'zip',
-        '^application\/x-tar': 'tar',
-        '^application\/x-gzip': 'tgz',
-        '^application\/x-bzip2': 'tar',
-        '^application\/gzip': 'tgz',
-        '^application\/x-rar': 'rar',         # available only if RARFILE_AVAILABLE > 0
-        '^application\/x-7z-compressed': '7z' # available only if SEVENZIP_AVAILABLE > 0
+        r'^application\/zip': 'zip',
+        r'^application\/x-tar': 'tar',
+        r'^application\/x-gzip': 'tgz',
+        r'^application\/x-bzip2': 'tar',
+        r'^application\/gzip': 'tgz',
+        r'^application\/x-rar': 'rar',         # available only if RARFILE_AVAILABLE > 0
+        r'^application\/x-7z-compressed': '7z' # available only if SEVENZIP_AVAILABLE > 0
     }
 
 
