@@ -23,6 +23,7 @@ import logging
 import traceback
 import weakref
 from fuglu.shared import default_template_values, FuConfigParser, Suspect, deprecated, get_default_cache
+from fuglu import get_version
 import typing as tp
 import time
 import os
@@ -49,6 +50,7 @@ ENABLED = SQL_EXTENSION_ENABLED # fuglu compatibility
 
 try:
     import requests
+    requests.packages.urllib3.disable_warnings()
     REQUESTS_AVAILABLE = True
 except ImportError:
     REQUESTS_AVAILABLE = False
@@ -139,9 +141,17 @@ class RESTAPIConfig(object):
     def __init__(self):
         self.logger = logging.getLogger('fuglu.sql.restconfig')
     
+    _re_title = re.compile('<title>(.{1,1000})</title>', re.I)
+    def _get_errormsg(self, reply):
+        errmsg = reply.text
+        rgxtitle =  self._re_title.search(errmsg)
+        if rgxtitle:
+            errmsg = rgxtitle.groups()[0]
+        return errmsg
     
-    def _http_request(self, uri, headers, suspectid, verify, retry=3):
-        reply = requests.get(uri, headers=headers, verify=verify, timeout=10)
+    
+    def _http_request(self, uri, headers, suspectid, verify, timeout, retry=3):
+        reply = requests.get(uri, headers=headers, verify=verify, timeout=timeout)
         if reply.status_code != 200:
             sleeptime = 0.1
             if reply.status_code == 503:
@@ -149,24 +159,21 @@ class RESTAPIConfig(object):
             if retry > 0:
                 time.sleep(sleeptime * (4-retry))
                 self.logger.debug(f'{suspectid} got unexpected status code {reply.status_code} from {uri} retry {retry}')
-                reply = self._http_request(uri, headers, suspectid, verify, retry=retry-1)
+                reply = self._http_request(uri, headers, suspectid, verify, timeout, retry=retry-1)
             else:
-                self.logger.error(f'{suspectid} got unexpected status code {reply.status_code} {reply.text} from {uri}')
+                self.logger.warning(f'{suspectid} got unexpected status code {reply.status_code} {self._get_errormsg(reply)} from {uri}')
         return reply
     
     
-    def _get(self, restapi_uri, headers, suspectid, restapi_verify):
+    def _get(self, restapi_uri, headers, suspectid, restapi_verify, timeout):
         try:
-            reply = self._http_request(restapi_uri, headers, suspectid, restapi_verify)
+            reply = self._http_request(restapi_uri, headers, suspectid, restapi_verify, timeout)
             if reply.status_code != 200:
                 try:
                     err = reply.json()
                     errmsg = err['errorMessage']
                 except (KeyError, json.JSONDecodeError):
-                    errmsg = reply.text
-                    rgxtitle =  re.search('<title>(.+)</title>', errmsg)
-                    if rgxtitle:
-                        errmsg = rgxtitle.groups()[0]
+                    errmsg = self._get_errormsg(reply)
                 raise RESTAPIError(errmsg)
             content = reply.json()
         except Exception as e:
@@ -174,7 +181,7 @@ class RESTAPIConfig(object):
         return content
     
     
-    def get(self, config:FuConfigParser, endpoint:str, suspectid:str):
+    def get(self, config:FuConfigParser, endpoint:str, suspectid:str, timeout:int = 0):
         if not REQUESTS_AVAILABLE:
             return []
         if not endpoint:
@@ -182,9 +189,13 @@ class RESTAPIConfig(object):
         
         restapi_uri = config.get('databaseconfig', 'restapi_uri', resolve_env=True)
         restapi_headers = config.getlist('databaseconfig', 'restapi_headers', resolve_env=True)
-        restapi_verify = config. getboolean('databaseconfig', 'restapi_verify')
+        restapi_verify = config.getboolean('databaseconfig', 'restapi_verify', fallback=False)
+        restapi_timeout = timeout or config.getfloat('databaseconfig', 'restapi_timeout', fallback=10)
         
-        headers = {'user-agent': 'fuglu', 'accept': 'application/json', }
+        headers = {
+            'User-Agent': f'Fuglu/{get_version().strip()}',
+            'accept': 'application/json',
+        }
         for item in restapi_headers:
             if not ':' in item:
                 self.logger.warning(f'{suspectid} invalid header definition {item}')
@@ -195,10 +206,8 @@ class RESTAPIConfig(object):
         if not restapi_uri.endswith('/'):
             restapi_uri = f'{restapi_uri}/'
         restapi_uri = f'{restapi_uri}{endpoint}'
-        if not restapi_uri.endswith('/'):
-            restapi_uri = f'{restapi_uri}/'
         
-        content = self._get(restapi_uri, headers, suspectid, restapi_verify)
+        content = self._get(restapi_uri, headers, suspectid, restapi_verify, restapi_timeout)
         return content
     
 
@@ -259,7 +268,8 @@ class DBConfig(FuConfigParser):
         del stringin
         # copy markers like %{confdir}
         self.markers = config.markers.copy()
-    
+        self._defaultdict = config._defaultdict
+
     
     def load_section(self, section: str) -> bool:
         """
@@ -267,57 +277,70 @@ class DBConfig(FuConfigParser):
         :param section: name of section to be loaded
         :return: True if loading was successful
         """
-        if SQL_EXTENSION_ENABLED and self.has_section('databaseconfig') and self.has_option('databaseconfig', 'dbconnectstring'):
-            connectstring = self.parentget('databaseconfig', 'dbconnectstring')
-            if connectstring.strip() == '':
-                return False
-            query = self.parentget('databaseconfig', 'sqlsection')
-            if query.strip() == '':
-                return False
-            
-            session = get_session(connectstring)
-            sqlvalues = {
-                'section': section,
-                'globalscope': self.parentget('databaseconfig', 'globalscope', fallback='$GLOBAL'),
-            }
-            default_template_values(self.suspect(), sqlvalues)
-            try:
-                result = session.execute(query, sqlvalues).fetchall()
-                self.sectioncache[section] = result
-            except Exception:
-                trb = traceback.format_exc()
-                self.logger.error("Error getting database config override section data: %s" % trb)
-                return False
+        loaded = False
+        if SQL_EXTENSION_ENABLED and self.has_section('databaseconfig') \
+                and self.has_option('databaseconfig', 'dbconnectstring') \
+                and self.has_option('databaseconfig', 'sqlsection'):
+            connectstring = self.parentget('databaseconfig', 'dbconnectstring', fallback='').strip()
+            query = self.parentget('databaseconfig', 'sqlsection').strip()
+            if connectstring and query:
+                session = get_session(connectstring)
+                sqlvalues = {
+                    'section': section,
+                    'globalscope': self.parentget('databaseconfig', 'globalscope', fallback='$GLOBAL'),
+                }
+                default_template_values(self.suspect(), sqlvalues)
+                try:
+                    result = session.execute(query, sqlvalues).fetchall()
+                    self.sectioncache[section] = result
+                except Exception:
+                    trb = traceback.format_exc()
+                    self.logger.error("Error getting database config override section data: %s" % trb)
+                loaded = True
 
-        if REQUESTS_AVAILABLE and  self.has_section('databaseconfig') and self.has_option('databaseconfig', 'restapi_uri') and self.has_option('databaseconfig', 'restapi_headers'):
+        if REQUESTS_AVAILABLE and self.has_section('databaseconfig') \
+                and self.has_option('databaseconfig', 'restapi_uri') \
+                and self.has_option('databaseconfig', 'restapi_headers') \
+                and self.parentget('databaseconfig', 'restapi_uri'):
             content = self._get_rest(section, None)[0]
             cachable_sectiondata = []
             for scope in content:
-                sectiondata = content[scope]
-                for option in sectiondata:
-                    cachable_sectiondata.append(SectionCacheRow(super(), scope, option, sectiondata[option]))
-                    try:
-                        self.sectioncache[section].extend(cachable_sectiondata)
-                    except KeyError:
-                        self.sectioncache[section] = cachable_sectiondata
+                scopedata = content[scope]
+                for sect in scopedata:
+                    if sect == section:
+                        sectiondata = scopedata[section]
+                        for option in sectiondata:
+                            cachable_sectiondata.append(SectionCacheRow(super(), scope, option, sectiondata[option]))
+                            try:
+                                self.sectioncache[section].extend(cachable_sectiondata)
+                            except KeyError:
+                                self.sectioncache[section] = cachable_sectiondata
+                    else:
+                        self.logger.debug(f'ignoring bogus section {section} in loaded rest data')
+            loaded = True
         
-        if YAML_AVAILABLE and self.has_section('databaseconfig') and not self.has_option('databaseconfig', 'yaml_filepath'):
+        if YAML_AVAILABLE and self.has_section('databaseconfig') \
+                and self.has_option('databaseconfig', 'yaml_filepath') \
+                and os.path.exists(self.parentget('databaseconfig', 'yaml_filepath')):
             content = self._get_yaml(section, None)[0]
             cachable_sectiondata = []
             for scope in content:
-                sectiondata = content[scope]
-                for option in sectiondata:
-                    cachable_sectiondata.append(SectionCacheRow(super(), scope, option, sectiondata[option]))
+                scopedata = content[scope]
+                for option in scopedata:
+                    cachable_sectiondata.append(SectionCacheRow(super(), scope, option, scopedata[option]))
                     try:
                         self.sectioncache[section].extend(cachable_sectiondata)
                     except KeyError:
                         self.sectioncache[section] = cachable_sectiondata
+            loaded = True
         
-        return True
+        return loaded
     
     
     def _get_cached(self, section: str, option: str):
         suspect = self.suspect()
+        if suspect is None:
+            return None
         todomain = f'%{suspect.to_domain}'
         
         optionfield = self.parentget('databaseconfig', 'option_field', fallback='option')
@@ -336,28 +359,31 @@ class DBConfig(FuConfigParser):
     
     
     def get_cached_options(self, section):
+        options = []
         suspect = self.suspect()
-        todomain = f'%{suspect.to_domain}'
+        if suspect is None:
+            return options
         
+        todomain = f'%{suspect.to_domain}'
         optionfield = self.parentget('databaseconfig', 'option_field', fallback='option')
         scopefield = self.parentget('databaseconfig', 'scope_field', fallback='scope')
         
-        options = []
         for row in self.sectioncache.get(section, []):
             scope = getattr(row, scopefield)
             dboption = getattr(row, optionfield)
             if scope in {suspect.to_address, todomain}:
                 options.append(dboption)
         return options
-    
-    
-    def get(self, section: str, option: str, **kwargs) -> tp.Any:
+
+
+    def get_override(self, section: str, option: str, **kwargs) -> tp.Any:
         result = None
         priorities = []
         if (self.has_section('databaseconfig')) and (self.has_option('databaseconfig', 'dbpriority')):
             priorities = self.parentgetlist('databaseconfig', 'dbpriority', lower=True)
-            if not 'cache' in priorities:
-                priorities.insert(0, 'cache')
+            #cache_ttl = self.parentgetint('databaseconfig', 'value_cache_ttl', fallback=30)
+            #if not 'cache' in priorities and cache_ttl > 0:
+            #    priorities.insert(0, 'cache')
         for priority in priorities:
             if priority in {'cache', 'sql', 'rest', 'yaml'}:
                 func = getattr(self, f'_get_{priority}')
@@ -366,14 +392,32 @@ class DBConfig(FuConfigParser):
                     break
                 if negcache is True:
                     break
-        #print(f'{section} {option} {result}')
-        if result is None:
-            result = self.parentget(section, option, **kwargs)
+        return result
+
+    def get(self, section: str, option: str, **kwargs) -> tp.Any:
+        override = self.get_override(section, option, **kwargs)
+        result = self.parentget(section, option, override=override, **kwargs)
+        return result
+
+    def getint(self, section: str, option: str, **kwargs):
+        override = self.get_override(section, option, **kwargs)
+        result = super().getint(section, option, override=override, **kwargs)
+        return result
+
+    def getboolean(self, section: str, option: str, **kwargs):
+        override = self.get_override(section, option, **kwargs)
+        result = self.parentgetboolean(section, option, override=override, **kwargs)
+        return result
+
+    def getlist(self, section: str, option: str, **kwargs) -> tp.Any:
+        override = self.get_override(section, option, **kwargs)
+        result = self.parentgetlist(section, option, override=override, **kwargs)
         return result
 
     
     def _get_sql(self, section: str, option: str, **kwargs) -> tp.Tuple[tp.Any, bool]:
-        if not SQL_EXTENSION_ENABLED or (not self.has_section('databaseconfig')) or (not self.has_option('databaseconfig', 'dbconnectstring')):
+        if not SQL_EXTENSION_ENABLED or (not self.has_section('databaseconfig')) \
+                or (not self.has_option('databaseconfig', 'dbconnectstring')):
             return None, False
         
         connectstring = self.parentget('databaseconfig', 'dbconnectstring')
@@ -407,36 +451,47 @@ class DBConfig(FuConfigParser):
             result = session.execute(query, sqlvalues).first()
         except Exception:
             trb = traceback.format_exc()
-            self.logger.error("Error getting database config override: %s" % trb)
-        
+            self.logger.error(f'{suspect.id} Error getting database config override: {trb}')
         session.remove()
+        
+        cache_ttl = self.parentgetint('databaseconfig', 'value_cache_ttl', fallback=30)
         if result is not None:
             value = result[0]
-            self.cache.put_cache(self._mkcachekey(True, suspect.to_address, section, option), value)
+            self.cache.put_cache(self._mkcachekey(True, suspect.to_address, section, option), value, cache_ttl)
             return value, False
         else:
-            self.cache.put_cache(self._mkcachekey(False, suspect.to_address, section, option), True)
-            self.cache.put_cache(self._mkcachekey(False, f'%{suspect.to_domain}', section, option), True)
-            self.cache.put_cache(self._mkcachekey(False, globalscope or '$GLOBAL', section, option), True)
+            self.cache.put_cache(self._mkcachekey(False, suspect.to_address, section, option), True, cache_ttl)
+            self.cache.put_cache(self._mkcachekey(False, f'%{suspect.to_domain}', section, option), True, cache_ttl)
+            self.cache.put_cache(self._mkcachekey(False, globalscope or '$GLOBAL', section, option), True, cache_ttl)
             return None, False
     
     
     def _get_rest(self, section: str, option: tp.Optional[str], **kwargs) -> tp.Tuple[tp.Any, bool]:
-        if (not REQUESTS_AVAILABLE) or (not self.has_section('databaseconfig')) or (not self.has_option('databaseconfig', 'restapi_uri')) or (not self.has_option('databaseconfig', 'restapi_headers')):
+        if (not REQUESTS_AVAILABLE) or (not self.has_section('databaseconfig')) \
+                or (not self.has_option('databaseconfig', 'restapi_uri')) \
+                or (not self.has_option('databaseconfig', 'restapi_headers')):
             try:
                 return self.parentget(section, option, **kwargs), False
             except (configparser.NoSectionError, configparser.NoOptionError):
                 return None, False
         
+        suspect = self.suspect()
         restapi_uri = self.parentget('databaseconfig', 'restapi_uri', resolve_env=True)
         restapi_endpoint = self.parentget('databaseconfig', 'restapi_endpoint')
+        if not restapi_uri or not restapi_endpoint:
+            return None, False
         restapi_headers = self.parentgetlist('databaseconfig', 'restapi_headers', resolve_env=True)
-        restapi_verify = self.parentgetboolean('databaseconfig', 'restapi_verify')
-        globalscope = self.parentget('databaseconfig', 'globalscope')
+        restapi_verify = self.parentgetboolean('databaseconfig', 'restapi_verify', fallback=True)
+        restapi_timeout = self.parentgetfloat('databaseconfig', 'restapi_timeout', fallback=10)
+        restapi_cachettl = self.parentgetfloat('databaseconfig', 'restapi_cachettl', fallback=300)
+        if (self.has_section('databaseconfig')) and (self.has_option('databaseconfig', 'globalscope')):
+            globalscope = self.parentget('databaseconfig', 'globalscope')
+        else:
+            globalscope = None
         headers = {'user-agent': 'fuglu', 'accept': 'application/json', }
         for item in restapi_headers:
             if not ':' in item:
-                self.logger.warning(f'invalid header definition {item}')
+                self.logger.warning(f'{suspect.id} invalid header definition {item}')
             else:
                 hdr, hdrval = item.split(':', 1)
                 headers[hdr] = hdrval
@@ -448,24 +503,29 @@ class DBConfig(FuConfigParser):
             restapi_uri = f'{restapi_uri}/'
         
         value = None
-        suspect = self.suspect()
         scopes = []
         if suspect.to_address and '@' in suspect.to_address:
-            scopes.append(suspect.to_address)
+            scopes.append(suspect.to_address.lower())
         if suspect.to_domain:
-            scopes.append(f'%{suspect.to_domain}')
+            scopes.append(f'%{suspect.to_domain.lower()}')
         if globalscope:
             scopes.append(globalscope)
-        
-        restapi = RESTAPIConfig()
+            
+        content = None
+        if restapi_cachettl > 0:
+            content = self.cache.get_cache(f'rest-{restapi_uri}')
+        if content is None:
+            restapi = RESTAPIConfig()
+            # returns data for all scopes from this endpoint
+            content = restapi._get(restapi_uri, headers, suspect.id, restapi_verify, restapi_timeout)
+            self.logger.debug(f'{suspect.id} fetched restapi data')
+            if restapi_cachettl > 0:
+                self.cache.put_cache(f'rest-{restapi_uri}', content, restapi_cachettl)
+        else:
+            self.logger.debug(f'{suspect.id} using cached restapi data')
         
         alldata = {}
         for scope in scopes:
-            restapi_uri_scope = f'{restapi_uri}{scope}'
-            content = self.cache.get_cache(f'rest-{restapi_uri_scope}')
-            if content is None:
-                content = restapi._get(restapi_uri, headers, suspect.id, restapi_verify)
-                self.cache.put_cache(f'rest-{restapi_uri_scope}', content)
             scopedata = content.get(scope, {})
             if option is None:
                 alldata[scope] = scopedata
@@ -473,10 +533,11 @@ class DBConfig(FuConfigParser):
             sectiondata = scopedata.get(section, {})
             value = sectiondata.get(option)
             #print(f'URI: {restapi_uri_scope} SECTION: {section} OPTION: {option} RESULT: {value}')
+            cache_ttl = self.parentgetint('databaseconfig', 'value_cache_ttl', fallback=30)
             if value is None:
-                self.cache.put_cache(self._mkcachekey(False, scope, section, option), True)
+                self.cache.put_cache(self._mkcachekey(False, scope, section, option), True, cache_ttl)
             else:
-                self.cache.put_cache(self._mkcachekey(True, scope, section, option), value)
+                self.cache.put_cache(self._mkcachekey(True, scope, section, option), value, cache_ttl)
                 break
         if option is None:
             return alldata, False
@@ -496,13 +557,17 @@ class DBConfig(FuConfigParser):
             user@example.com: false
         """
         
-        if (not YAML_AVAILABLE) or (not self.has_section('databaseconfig')) or (not self.has_option('databaseconfig', 'yaml_filepath')):
+        if (not YAML_AVAILABLE) or (not self.has_section('databaseconfig')) \
+                or (not self.has_option('databaseconfig', 'yaml_filepath')):
             try:
                 return self.parentget(section, option, **kwargs), False
             except (configparser.NoSectionError, configparser.NoOptionError):
                 return None, False
-        
-        globalscope = self.parentget('databaseconfig', 'globalscope')
+
+        if (self.has_section('databaseconfig')) and (self.has_option('databaseconfig', 'globalscope')):
+            globalscope = self.parentget('databaseconfig', 'globalscope')
+        else:
+            globalscope = None
         yaml_filepath = self.parentget('databaseconfig', 'yaml_filepath', resolve_env=True)
         suspect = self.suspect()
 
@@ -517,33 +582,35 @@ class DBConfig(FuConfigParser):
             with open(yaml_filepath, 'r') as fp:
                 self.yamldata[yaml_filepath] = yaml.load(fp, Loader=yaml.FullLoader)
             self.logger.debug(f'{suspect.id} loaded yaml file {yaml_filepath}')
+
+        if option is None:
+            alldata = {}
+            sectiondata = self.yamldata[yaml_filepath].get(section, {})
+            for o in sectiondata:
+                optiondata = sectiondata[o]
+                for s in optiondata:
+                    v = optiondata[s]
+                    try:
+                        alldata[s.lower()][o.lower()] = v
+                    except KeyError:
+                        alldata[s.lower()] = {o.lower: v}
+            return alldata, False
         
         scopes = []
         if suspect.to_address and '@' in suspect.to_address:
-            scopes.append(suspect.to_address)
+            scopes.append(suspect.to_address.lower())
         if suspect.to_domain:
-            scopes.append(suspect.to_domain)
+            scopes.append(suspect.to_domain.lower())
         if not globalscope:
             scopes.append(globalscope)
-        
-        alldata = {}
+            
         value = None
         for scope in scopes:
             sectiondata = self.yamldata[yaml_filepath].get(section, {})
-            if option is None:
-                for o in sectiondata:
-                    v = sectiondata[o].get(scope)
-                    try:
-                        alldata[scope][o] = v
-                    except KeyError:
-                        alldata[scope] = {o:v}
-            
             value = sectiondata.get(option, {}).get(scope, None)
             if value is not None:
                 value = str(value)
                 break
-        if option is None:
-            return alldata, False
         return value, False
     
     
@@ -559,10 +626,10 @@ class DBConfig(FuConfigParser):
         
         scopes = []
         if suspect.to_address and '@' in suspect.to_address:
-            scopes.append(suspect.to_address)
+            scopes.append(suspect.to_address.lower())
         if suspect.to_domain:
-            scopes.append(f'{suspect.to_domain}')
-            scopes.append(f'%{suspect.to_domain}')
+            scopes.append(f'{suspect.to_domain.lower()}')
+            scopes.append(f'%{suspect.to_domain.lower()}')
         globalscope = self.parentget('databaseconfig', 'globalscope', fallback='$GLOBAL')
         if globalscope:
             scopes.append(globalscope)
@@ -588,6 +655,12 @@ class DBConfig(FuConfigParser):
     def parentgetboolean(self, section: str, option: str, **kwargs) -> bool:
         return self._convert_to_boolean(super().get(section, option, **kwargs))
     
+    def parentgetfloat(self, section: str, option: str, **kwargs) -> float:
+        return float(super().get(section, option, **kwargs))
+    
+    def parentgetint(self, section: str, option: str, **kwargs) -> int:
+        return int(super().get(section, option, **kwargs))
+    
     def parentgetlist(self, section: str, option: str, **kwargs) -> tp.List[str]:
         return super().getlist(section, option, **kwargs)
 
