@@ -541,7 +541,7 @@ def verify(message, debuglog=None):
                     sig['x'], sig['t']), file=debuglog)
             return False
 
-    m = re.match("(\w+)(?:/(\w+))?$", sig['c'])
+    m = re.match(r"(\w+)(?:/(\w+))?$", sig['c'])
     if m is None:
         if debuglog is not None:
             print("c= value is not in format method/method (%s)" % sig[
