@@ -29,7 +29,7 @@ __author__ = 'Eric DeFriez'
 import binascii
 import logging
 import struct
-from fuglu.stringencode import force_bString, force_uString
+from fuglu.stringencode import force_uString
 
 
 MILTER_VERSION = 2  # Milter version we claim to speak (from pmilter)
@@ -192,7 +192,7 @@ class PpyMilterDispatcher(object):
         cmd, data = data[0], data[1:]
         try:
             if cmd not in COMMANDS:
-                logging.warn('Unknown command code: "%s" ("%s")', force_uString(cmd), force_uString(data))
+                logging.warning('Unknown command code: "%s" ("%s")', force_uString(cmd), force_uString(data))
                 return RESPONSE['CONTINUE']
             u_command = force_uString(COMMANDS[cmd])  # (unicode)
             parser_callback_name = '_Parse%s' % u_command
@@ -203,7 +203,7 @@ class PpyMilterDispatcher(object):
                 return RESPONSE['CONTINUE']
 
             if not hasattr(self.__milter, handler_callback_name):
-                logging.warn('Unimplemented command in milter %s: "%s" ("%s")' % (
+                logging.warning('Unimplemented command in milter %s: "%s" ("%s")' % (
                     self.__milter, u_command, data))
                 return RESPONSE['CONTINUE']
 
@@ -582,7 +582,7 @@ class PpyMilter(object):
         try:
             self.OnResetState()
         except AttributeError:
-            logging.warn('No OnResetState() callback is defined for this milter.')
+            logging.warning('No OnResetState() callback is defined for this milter.')
 
     # you probably should not be overriding this  :-p
     def OnOptNeg(self, cmd, ver, actions, protocol):
