@@ -846,12 +846,13 @@ class Mailattachment(Cachelimits):
             location += element_of + element_of.join(["{" + obj().filename + "}" for obj in self.parent_archives])
         return location
 
-
+    
+    _re_ws = re.compile(r'\s')
     @smart_cached_memberfunc(inputs=["filename"])
     def get_mangled_filename(self, prefix: str = '', maxchars: int = 15, keepending: bool = False):
         first=self.filename.split('.')[0]
         #replace spaces with underscore
-        first = re.sub(r'\s', '_', first)
+        first = self._re_ws.sub('_', first)
         #remove all non alnum chars
         first=''.join([x for x in first if x in string.ascii_letters or x in string.digits])
         if first.strip()=='' or first.strip()=='filename': #renattach renames to 'filename'
