@@ -13,8 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-__all__ = ['archive', 'attachment', 'decision', 'clamav', 'sa', 'p_debug', 'p_blwl', 'a_logging', 'bacn',
-           'p_skipper', 'p_fraction', 'vacation', 'antivirus', 'sssp', 'domainauth', 'script', 'a_statsd',
-           'fuzor', 'rspamd', 'mailcopy', 'uriextract', 'call_ahead', 'delay', 'ivmsg', 'domainauth',
-           'tlspolicy', 'originpolicy', 'outpolicy', 'restrictions', 'geoip', 'messagesize', 'knownsubject',
-           'bacn',]
+__all__ = [
+    'a_logging', 'a_statsd', 'antivirus', 'archive', 'attachment', 'bacn', 'bacn', 'call_ahead', 'clamav',
+    'dataprovider', 'decision', 'delay', 'dnsdata', 'domainauth', 'encryptedarchives', 'fuzor', 'geoip',
+    'knownsubject', 'mailcopy', 'messagesize', 'originpolicy', 'outpolicy', 'p_blwl', 'p_debug', 'p_fraction',
+    'p_skipper', 'restrictions', 'rspamd', 'sa', 'script', 'sssp', 'tlspolicy', 'uriextract', 'vacation',
+]
