@@ -40,19 +40,13 @@ except ImportError:
     HAVE_DOMAINMAGIC=False
 
 
-FIELDMAP = {
-    'id': 'fugluid',
-    'injectqueueid': 'queueid',
-    'Attachment.bounce.queueid': 'bounceqid'
-}
-
 LOG_MAPPING = {
     "properties": {
-        "headers": {"type": "nested"},
+        "headers": {"type": "nested", "properties": {"content": {"type": "text", "fields": {"keyword": {"type": "keyword", "ignore_above": 4096}}}}},
         "spamreport": {"type": "nested"},
         "virusreport": {"type": "nested"},
         "blockedreport": {"type": "nested"},
-        "uris": {"type": "nested"},
+        "uris": {"type": "nested", "properties": {"uri": {"type": "text", "fields": {"keyword": {"type": "keyword", "ignore_above": 512}}}}},
         "attachments": {"type": "nested"},
         "client_ip": {"type": "ip"},
         "size": {"type": "integer"},
@@ -60,6 +54,17 @@ LOG_MAPPING = {
 }
 
 
+class MappingFile(FileList):
+    def _parse_lines(self, lines):
+        content = '\n'.join(lines)
+        try:
+            jsoncontent = json.loads(content)
+        except Exception as e:
+            self.logger.error(f'failed to load json data: {e.__class__.__name__}: {str(e)}')
+            jsoncontent = None
+        return jsoncontent
+
+
 
 class URILogger(object):
     def __init__(self, config, section):
@@ -103,24 +108,26 @@ class URILogger(object):
                 u = urllib.parse.urlparse(uri)
             except Exception as e:
                 # log error and skip on logging this uri on error
-                self.logger.error(f"(uri from rcpt domain) {suspectid if suspectid else '<>'} msg: {str(e)} uri: {uri}")
+                self.logger.error(f"{suspectid} unparseable URL {uri} {e.__class__.__name__}: {str(e)}")
                 continue
         
             if u.hostname and (u.hostname == to_domain or u.hostname.endswith('.%s' % to_domain)):
+                self.logger.debug(f'{suspectid} skipping URL in recipient domain: {uri}')
                 continue
             elif not u.hostname:
-                self.logger.debug('%s not a parseable URL: %s' % (suspectid, uri))
+                self.logger.debug(f'{suspectid} not a parseable URL: {uri}')
             new_uris[uri] = uris.get(uri, 'unknown')
         return new_uris
     
 
-    def _remove_uri_fragments(self, uris):
+    def _remove_uri_fragments(self, uris, suspectid):
         new_uris = {}
         urikeys = list(uris.keys())
         for uri in urikeys:
             fragment = False
             for u in urikeys:
                 if uri != u and uri in u:
+                    self.logger.debug(f'{suspectid} discarding uri fragment {uri} in {u}')
                     fragment = True
                     break
             if not fragment:
@@ -164,8 +171,8 @@ class URILogger(object):
                 if uri not in uris and is_url_tldcheck(uri, exclude_fqdn=EXCLUDE_FQDN, exclude_domain=EXCLUDE_DOMAIN):
                     uris[uri] = tag
     
-        uris = self._remove_uris_in_rcpt_domain(uris, suspect.to_domain, suspect.id)
-        uris = self._remove_uri_fragments(uris)
+        uris = self._remove_uris_in_rcpt_domain(uris, suspect.to_domain, suspect.id or '<>')
+        uris = self._remove_uri_fragments(uris, suspect.id or '<>')
         for rawuri in list(uris.keys())[:maxitems]:
             uri = extractor.redirect_from_url(rawuri)
             fqdn, domain = self._get_domain_from_uri(uri, suspect=suspect)
@@ -278,14 +285,14 @@ class ElasticBackend(object):
     def __init__(self, config, section):
         self.config = config
         self.section = section
-        self.logger = logging.getLogger('logging.%s' % self.__class__.__name__)
+        self.logger = logging.getLogger('fuglu.logging.%s' % self.__class__.__name__)
         self.elastic_connection = {}
+        self.mapping_file = None
     
     
     def get_elastic_connection(self, reload=False) -> ElasticClient:
         es = None
-        elastic_uris = self.config.getlist(self.section,'elastic_uris')
-        verify_certs = self.config.getboolean(self.section, 'elastic_verify_certs')
+        elastic_uris = self.config.getlist(self.section,'elastic_uris', resolve_env=True)
         if elastic_uris:
             elastic_uri_string = ','.join(elastic_uris)
             try:
@@ -293,17 +300,38 @@ class ElasticBackend(object):
                     del self.elastic_connection[elastic_uri_string]
                 es = self.elastic_connection[elastic_uri_string]
             except KeyError:
-                es = ElasticClient(hosts=elastic_uris, verify_certs=verify_certs, ssl_show_warn=False)
+                verify_certs = self.config.getboolean(self.section, 'elastic_verify_certs')
+                timeout = self.config.getfloat(self.section, 'elastic_timeout')
+                es = ElasticClient(hosts=elastic_uris, verify_certs=verify_certs, ssl_show_warn=False, timeout=timeout)
                 self.elastic_connection[elastic_uri_string] = es
         return es
     
     
     def get_elastic_index(self, suspect:Suspect) -> str:
         indextmpl = self.config.get(self.section,'elastic_index')
+        tagindexlist = [item.rsplit(':',1) for item in self.config.getlist(self.section,'elastic_index_by_tags')]
+        for tag, itmpl in tagindexlist:
+            if suspect.get_tag(tag):
+                indextmpl = itmpl
         indexname = apply_template(indextmpl, suspect)
         return indexname
     
-
+    
+    def _get_mapping(self, mapping_file):
+        new_mapping = LOG_MAPPING
+        
+        if not self.mapping_file and mapping_file and os.path.exists(mapping_file):
+            self.mapping_file = MappingFile(filename=mapping_file)
+        if self.mapping_file:
+            loaded_mapping = self.mapping_file.get_list()
+            if not loaded_mapping:
+                self.logger.error(f'mapping file {mapping_file} could not be loaded')
+            else:
+                new_mapping = loaded_mapping
+        
+        return new_mapping
+    
+    
     @smart_cached_memberfunc(inputs=[])
     def _set_index_mapping(self, indexname:str) -> None:
         self.logger.info(f'checking mapping of index {indexname}')
@@ -311,12 +339,24 @@ class ElasticBackend(object):
         exists = es.indices.exists(index=indexname)
         if exists:
             need_put = False
-            mapping = es.indices.get_mapping(index=indexname)
-            properties = mapping.get(indexname, {}).get('mappings', {}).get('properties', {})
-            for key in LOG_MAPPING['properties']:
-                if not LOG_MAPPING['properties'][key]['type'] == properties.get(key, {}).get('type'):
-                    need_put = True
-                    break
+            current_mapping = es.indices.get_mapping(index=indexname)
+            properties = current_mapping.get(indexname, {}).get('mappings', {}).get('properties', {})
+            
+            mapping_file = self.config.get(self.section, 'elastic_mapping_file', fallback='')
+            new_mapping = self._get_mapping(mapping_file)
+            
+            try:
+                for key in new_mapping['properties']:
+                    if not new_mapping['properties'][key]['type'] == properties.get(key, {}).get('type'):
+                        need_put = True
+                        break
+            except KeyError as e:
+                need_put = False
+                if mapping_file:
+                    self.logger.error(f'failed to load mapping file {mapping_file}: {e.__class__.__name__}: {str(e)}')
+                else:
+                    self.logger.error(f'invalid mapping: {e.__class__.__name__}: {str(e)}')
+            
             if need_put:
                 try:
                     r = es.indices.put_mapping(body=LOG_MAPPING, index=indexname)
@@ -340,25 +380,26 @@ class ElasticBackend(object):
                 self.logger.info(f'error creating new elastic index {indexname} : {str(r)}')
     
     
-    def log_to_elastic(self, suspect:Suspect, logdata:tp.Dict, reload=False, retry:int=3) -> tp.Dict[str, str]:
+    def log_to_elastic(self, suspect:Suspect, logdata:tp.Dict, rcpt_id:int, reload=False, retry:int=3) -> tp.Dict[str, str]:
         result = {}
         try:
             logdata['timestamp'] = datetime.datetime.utcfromtimestamp(suspect.timestamp).isoformat()
             es = self.get_elastic_connection(reload)
             indexname = self.get_elastic_index(suspect)
             self._set_index_mapping(indexname)
-            r = es.index(index=indexname, id=suspect.id, body=logdata)
-            for key in ['_id']:
+            documentid = f'{suspect.id}_{rcpt_id}'
+            r = es.index(index=indexname, id=documentid, body=logdata)
+            for key in ['_id', 'result']:
                 try:
                     result[key] = r[key]
                 except KeyError:
-                    self.logger.error('%s key %s not found in result %s' % (suspect.id, key, r))
-            self.logger.info('%s indexed in elastic: %s' % (suspect.id, r))
+                    self.logger.warning('%s key %s not found in result %s' % (suspect.id, key, r))
+            self.logger.info('%s indexed in elastic %s: %s' % (suspect.id, indexname, r))
         except Exception as e:
             if retry>0:
                 self.logger.debug(f'{suspect.id} failed to index in elastic, retry={retry} reason={e.__class__.__name__}: {str(e)}')
                 time.sleep(0.2*(4-retry))
-                result = self.log_to_elastic(suspect, logdata, reload=True, retry=retry-1)
+                result = self.log_to_elastic(suspect, logdata, rcpt_id, reload=True, retry=retry-1)
             else:
                 raise
         return result
@@ -382,27 +423,54 @@ class ElasticLogger(AppenderPlugin):
                 'default': 'True',
                 'description': 'verify server\'s SSL certificates',
             },
+            'elastic_timeout': {
+                'default': '30',
+                'description': 'set elastic connection timeout to this value',
+            },
             'elastic_index':{
                 'default': 'fuglulog-${date}',
                 'description':'Name of ElasticSearch index in which document will be stored. Template vars (e.g. ${to_domain} or ${date}) can be used.',
             },
+            'elastic_mapping_file': {
+                'default': '',
+                'description': 'read elastic mapping from json file. leave empty to use default mapping.',
+            },
+            'elastic_index_by_tags':{
+                'default': '',
+                'description':'Comma separated list of tuples of SuspectTag:IndexName of ElasticSearch index in which document will be stored. Template vars (e.g. ${to_domain} or ${date}) can be used for index name. SuspectTag value must evaluate to True. First hit wins.',
+            },
             'log_headers':{
-                'default': 'from:addr,to:pseudo:md5,reply-to:addr,subject,subject:decode,subject:hash:md5,message-id',
+                'default': 'from:addr,to:pseudo:md5,reply-to:addr,subject,subject:decode,subject:hash:md5,message-id:low',
                 'description:': """
                     Name of message headers to log and extract.
                     Address headers (e.g. To, From, Reply-To) support tags such as :addr (address only), :pseudo:algo (address only, with hashed localpart), :name (name label only), :namedecode (name label only, decoded).
                     All header support tag hash:algo - algo must be one of hashlibs supported hash algorithms
                     All headers support tag decode - try decode encoded headers
-                    All header names will be prefixed hdr_ in elastic document, - and : will be converted to _"""
+                    All header support tag low - lowercase value
+                    All header names will be prefixed hdr_ in elastic document, - and : will be converted to _
+                    Currently only one tag per header is possible
+                    Name will be renamed according to log_field_map.
+                """
             },
             'log_values': {
-                'default': 'id,from_address,from_domain,from_localpart,queue_id,sasl_login,size,timestamp,to_address,to_domain,to_localpart,to_address:pseudo:md5,to_localpart:pseudo:md5',
+                'default': 'id,from_address,from_domain,from_localpart,queue_id,sasl_login,size,timestamp',
                 'description:': """
-                    Name of suspect attribute values to log and extract. Some values will be renamed.
-                    Email address like values support :pseudo:algo (hash localpart)"""
+                    Name of suspect attribute values to log and extract.
+                    Email address like values support :pseudo:algo (hash localpart)
+                    Name will be renamed according to log_field_map.
+                """
+            },
+            'log_field_map': {
+                'default': 'id:fugluid, injectqueueid:queueid, attachment_bounce_queueid:bounceqid',
+                'description': """
+                    Rename field names prior to logging.
+                    map is a comma separated list of key:value pairs.
+                    key is the name of the field to be renamed (as written to elastic document),
+                    value is the resulting name.
+                """
             },
             'log_tags':{
-                'default': 'injectqueueid,fuzor.digest,log.scanhost,log.decision,log.client_helo,log.client_ip,log.client_hostname,archived,Attachment.bounce.queueid,spf.result,dkim.result,arc.result,dmarc.result,log.real_from_address',
+                'default': 'to_address,to_domain,to_localpart,to_address:pseudo:md5,to_localpart:pseudo:md5,injectqueueid,fuzor.digest,log.scanhost,log.decision,log.client_helo,log.client_ip,log.client_hostname,archived,Attachment.bounce.queueid,spf.result,dkim.result,arc.result,dmarc.result,log.real_from_address,recipient_id',
                 'description:': 'Name of message tags to log and extract. Some tags will be renamed.'
             },
             'log_raw_headers': {
@@ -449,6 +517,7 @@ class ElasticLogger(AppenderPlugin):
         self.urilogger = None
         self.attlogger = None
         self.elasticbackend = None
+        self.fieldmap = None
         
         
     def _init_components(self) -> None:
@@ -461,13 +530,19 @@ class ElasticLogger(AppenderPlugin):
     
     
     def _normalise_fieldname(self, field_name:str) -> str:
+        if self.fieldmap is None:
+            self.fieldmap = {k:v for k,v in [x.strip().rsplit(':',1) for x in self.config.getlist(self.section, 'log_field_map')]}
+        
+        field_name = field_name.lower()
+        
         if field_name.startswith('log.'):
             field_name = field_name[4:]
-        else:
-            field_name = FIELDMAP.get(field_name, field_name)
-            badchars = '-.:'
-            for c in badchars:
-                field_name = field_name.replace(c, '_')
+        
+        badchars = '-.:'
+        for c in badchars:
+            field_name = field_name.replace(c, '_')
+        
+        field_name = self.fieldmap.get(field_name, field_name)
         return field_name
     
     
@@ -502,6 +577,7 @@ class ElasticLogger(AppenderPlugin):
     
     def _normalise_header(self, suspect:Suspect, header_name:str, header_tag:str, header_value:str) -> str:
         header_name = header_name.lower()
+        header_value = header_value.strip()
         if header_tag is not None:
             header_tag = header_tag.lower()
             if header_tag.startswith('hash'):
@@ -514,6 +590,8 @@ class ElasticLogger(AppenderPlugin):
                         header_value = None
                 else:
                     self.logger.info('%s unsupported hash algorithm %s in header %s' % (suspect.id, algo, header_name))
+            elif header_tag == 'low':
+                header_value = header_value.lower()
             elif header_tag == 'decode':
                 header_value = suspect.decode_msg_header(header_value)
             elif header_name in ['from', 'to', 'reply-to', 'sender']:
@@ -587,12 +665,16 @@ class ElasticLogger(AppenderPlugin):
                 value = domain_from_mail(value)
             elif value and attr == 'from_localpart':
                 value = split_mail(value)[0]
+            if value:
+                value = value.lower()
         elif attr in ['to_address', 'to_domain', 'to_localpart']:
             value = self._get_env_override(suspect, 'log_header_as_env_recipient')
             if value and attr == 'to_domain':
                 value = domain_from_mail(value)
             elif value and attr == 'to_localpart':
                 value = split_mail(value)[0]
+            if value:
+                value = value.lower()
         else:
             override = False
         
@@ -618,7 +700,7 @@ class ElasticLogger(AppenderPlugin):
             value = self._get_suspect_attr(suspect, real_field)
             if field_tag and field_tag.startswith('pseudo:') and value:
                 value = self._pseudonymise(value, field_tag)
-            key = self._normalise_fieldname(FIELDMAP.get(field, field))
+            key = self._normalise_fieldname(field)
             logdata[key] = value
         return logdata
     
@@ -627,7 +709,7 @@ class ElasticLogger(AppenderPlugin):
         logdata = {}
         status = suspect.get_status()
         for func_name in status:
-            key = self._normalise_fieldname(FIELDMAP.get(func_name, func_name))
+            key = self._normalise_fieldname(func_name)
             logdata[key] = status[func_name]
         return logdata
     
@@ -638,7 +720,7 @@ class ElasticLogger(AppenderPlugin):
         for tag in suspect_tags:
             value = suspect.get_tag(tag)
             if value is not None:
-                key = self._normalise_fieldname(FIELDMAP.get(tag, tag))
+                key = self._normalise_fieldname(tag)
                 logdata[key] = force_uString(value)
         return logdata
     
@@ -660,7 +742,7 @@ class ElasticLogger(AppenderPlugin):
             hdrdata['line'] = hdrline # current line number
             hdrdata['header'] = hdr_name
             hdrdata['iheader'] = hdr_name_low
-            hdrdata['content'] = force_uString(hdr_content)
+            hdrdata['content'] = force_uString(hdr_content).strip()
             headerdata.append(hdrdata)
         return headerdata
     
@@ -757,7 +839,7 @@ class ElasticLogger(AppenderPlugin):
         return logdata
     
     
-    def _log_to_file(self, suspect:Suspect, logdata:tp.Dict) -> None:
+    def _log_to_file(self, suspect:Suspect, logdata:tp.Dict, rcpt_id:int) -> None:
         dirpath = self.config.get(self.section, 'fallback_logdir')
         if dirpath:
             if not os.path.exists(dirpath):
@@ -765,7 +847,7 @@ class ElasticLogger(AppenderPlugin):
                 dirpath = '/tmp'
             
             indexname = self.elasticbackend.get_elastic_index(suspect)
-            filename = f'{suspect.id}.json'
+            filename = f'{suspect.id}_{rcpt_id}.json'
             indexpath = os.path.join(dirpath, indexname)
             filepath = os.path.join(indexpath, filename)
             try:
@@ -782,6 +864,7 @@ class ElasticLogger(AppenderPlugin):
     def _normalise_address(self, address:str) -> str:
         address = strip_batv(address)
         address = decode_srs(address)
+        address = address.lower()
         return address
     
     
@@ -820,13 +903,22 @@ class ElasticLogger(AppenderPlugin):
         if self.elasticbackend.get_elastic_connection() is None:
             return
         self._add_tags(suspect, decision)
-        logdata = self._get_log_coredata(suspect)
-        try:
-            result = self.elasticbackend.log_to_elastic(suspect, logdata)
-            self.logger.info('%s processed with result %s' % (suspect.id, result))
-        except Exception as e:
-            self.logger.error('%s failed to index in elastic due to %s' % (suspect.id, str(e)))
-            self._log_to_file(suspect, logdata)
+        rcpt_id = 0
+        for recipient in suspect.recipients:
+            recipient = recipient.lower()
+            suspect.set_tag('to_address', recipient)
+            suspect.set_tag('to_domain', domain_from_mail(recipient))
+            suspect.set_tag('to_localpart', split_mail(recipient)[0])
+            suspect.set_tag('recipient_id', rcpt_id)
+            suspect.set_tag('recipient_count', len(suspect.recipients))
+            logdata = self._get_log_coredata(suspect)
+            try:
+                result = self.elasticbackend.log_to_elastic(suspect, logdata, rcpt_id)
+                self.logger.info('%s processed with result %s' % (suspect.id, result))
+            except Exception as e:
+                self.logger.error('%s failed to index in elastic due to %s' % (suspect.id, str(e)))
+                self._log_to_file(suspect, logdata, rcpt_id)
+            rcpt_id += 1
     
     
     def lint(self):
@@ -843,7 +935,8 @@ class ElasticLogger(AppenderPlugin):
             print('WARNING: elastic_uris not defined, this plugin will do nothing')
             return False
         elif not es.ping():
-            print('ERROR: failed to connect to elasticsearch, connection info: %s' % str(es))
+            elastic_uris = self.config.get(self.section, 'elastic_uris', resolve_env=True)
+            print(f'ERROR: failed to connect to elasticsearch {", ".join(elastic_uris)}, connection info: {str(es)}')
             return False
         
         dirpath = self.config.get(self.section, 'fallback_logdir')
@@ -855,6 +948,18 @@ class ElasticLogger(AppenderPlugin):
         if invalid_hashes:
             print('ERROR: invalid hash algorithms in log_attachment_hashes: %s' % ','.join(invalid_hashes))
             return False
+        
+        # superficial lint of mapping...
+        mapping_file = self.config.get(self.section, 'elastic_mapping_file')
+        new_mapping = self.elasticbackend._get_mapping(mapping_file)
+        try:
+            for key in new_mapping['properties']:
+                if not new_mapping['properties'][key]['type']:
+                    print(f'empty value in key {key}')
+                    return False
+        except KeyError as e:
+            print(str(e))
+            return False
     
         return True
 
