@@ -185,7 +185,7 @@ Tags:
     
     
     def __init_socket__(self):
-        host = self.config.get(self.section, 'host')
+        host = self.config.get(self.section, 'host', resolve_env=True)
         port = self.config.getint(self.section, 'port')
         timeout = self.config.getint(self.section, 'timeout')
         try:
@@ -374,8 +374,12 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
     
     
     def __str__(self):
-        return "ICAP AV"
-    
+        classname = self.__class__.__name__
+        if self.section == classname:
+            return "ICAP AV"
+        else:
+            return f'ICAP AV - {self.section}'
+
     
     
     def scan_stream(self, content, suspectid='(NA)'):
@@ -392,27 +396,27 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
         s = self.__init_socket__()
         dr = {}
         
-        content = force_uString(content)
-        CRLF = "\r\n"
-        host = self.config.get(self.section, 'host')
+        content = force_bString(content)
+        CRLF = b"\r\n"
+        host = self.config.get(self.section, 'host', resolve_env=True)
         port = self.config.get(self.section, 'port')
         service = self.config.get(self.section, 'service')
         send_fakeheaders = self.config.getboolean(self.section, 'send_fakeheaders')
         buflen = len(content)
 
-        bodyparthexlen = hex(buflen)[2:]
+        bodyparthexlen = hex(buflen)[2:].encode()
         bodypart = bodyparthexlen + CRLF
         bodypart += content + CRLF
-        bodypart += "0" + CRLF
+        bodypart += b"0" + CRLF
         
         if send_fakeheaders:
-            fakerequestheader = "GET http://localhost/message.eml HTTP/1.1%s" + CRLF
-            fakerequestheader += "Host: localhost" + CRLF
+            fakerequestheader = b"GET http://localhost/message.eml HTTP/1.1%s" + CRLF
+            fakerequestheader += b"Host: localhost" + CRLF
             fakerequestheader += CRLF
             fakereqlen = len(fakerequestheader)
             
-            fakeresponseheader = "HTTP/1.1 200 OK" + CRLF
-            fakeresponseheader += "Content-Length: " + str(buflen) + CRLF
+            fakeresponseheader = b"HTTP/1.1 200 OK" + CRLF
+            fakeresponseheader += b"Content-Length: " + str(buflen).encode() + CRLF
             fakeresponseheader += CRLF
             fakeresplen = len(fakeresponseheader)
             
@@ -422,15 +426,15 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
         
         # now that we know the length of the fake request/response, we can
         # build the ICAP header
-        icapheader = ""
-        icapheader += "RESPMOD icap://%s:%s/%s ICAP/1.0%s" % (host, port, service, CRLF)
-        icapheader += "Host: " + host + CRLF
-        icapheader += "User-Agent: Fuglu/%s%s" % (get_version().strip(), CRLF)
-        icapheader += "Allow: 204" + CRLF
+        icapheader = b""
+        icapheader += b"RESPMOD icap://%s:%s/%s ICAP/1.0%s" % (host.encode(), port.encode(), service.encode(), CRLF)
+        icapheader += b"Host: " + host.encode() + CRLF
+        icapheader += b"User-Agent: Fuglu/%s%s" % (get_version().strip().encode(), CRLF)
+        icapheader += b"Allow: 204" + CRLF
         if send_fakeheaders:
-            icapheader += "Encapsulated: req-hdr=%s, res-hdr=%s, res-body=%s%s" % (hdrstart, responsestart, bodystart, CRLF)
+            icapheader += b"Encapsulated: req-hdr=%s, res-hdr=%s, res-body=%s%s" % (hdrstart, responsestart, bodystart, CRLF)
         else:
-            icapheader += "Encapsulated: res-body=0" + CRLF
+            icapheader += b"Encapsulated: res-body=0" + CRLF
         icapheader += CRLF
         
         if send_fakeheaders:
@@ -438,7 +442,7 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
         else:
             everything = icapheader + bodypart + CRLF
         #print(f"sending: {everything}")
-        s.sendall(force_bString(everything))
+        s.sendall(everything)
         result = force_uString(s.recv(20000))
         s.close()
         #print(f"result is: {result}")
@@ -479,7 +483,7 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
     
     def lint_options(self):
         CRLF = "\r\n"
-        host = self.config.get(self.section, 'host')
+        host = self.config.get(self.section, 'host', resolve_env=True)
         port = self.config.get(self.section, 'port')
         service = self.config.get(self.section, 'service')
         
@@ -691,7 +695,7 @@ Tags:
     
     
     def __init_socket__(self):
-        host = self.config.get(self.section, 'host')
+        host = self.config.get(self.section, 'host', resolve_env=True)
         port = self.config.getint(self.section, 'port')
         socktimeout = self.config.getint(self.section, 'timeout')
         try:
@@ -812,7 +816,7 @@ Prerequisites: Cyren Antivirus must be installed and running, not necessarily on
         result = force_uString(result.strip())
         self.logger.debug(f'{suspectid} cyren sez {result}')
         #daemon_status, scan_status, obj, message = re.split('\s', result, 3)
-        fields = re.split('\s', result)
+        fields = re.split(r'\s', result)
         if len(fields) == 3:
             daemon_status, scan_status, obj = fields
             message = None
