@@ -16,7 +16,8 @@
 #
 from fuglu.shared import ScannerPlugin, DUNNO, SuspectFilter, apply_template, get_outgoing_helo, Suspect
 from fuglu.stringencode import force_uString, force_bString
-from fuglu.extensions.elastic import ElasticClient, lint_elastic
+from fuglu.extensions.elastic import ElasticClient, lint_elastic, ElasticException
+from fuglu.caching import smart_cached_memberfunc
 import os
 import shutil
 import pwd
@@ -27,6 +28,7 @@ import smtplib
 import socket
 import datetime
 from collections import OrderedDict
+from base64 import b64encode
 
 try:
     from cassandra.cluster import Cluster
@@ -37,6 +39,16 @@ except ImportError:
     HAVE_CASSANDRA = False
 
 
+LOG_MAPPING = {
+    "properties": {
+        "blob": {
+            "type": "binary"
+        }
+    }
+}
+
+
+
 class AbstractBackend(object):
     requiredvars = {}
     
@@ -243,6 +255,10 @@ class ElasticBackend(AbstractBackend):
             'default': 'True',
             'description': 'verify server\'s SSL certificates',
         },
+        'elastic_timeout': {
+            'default': '30',
+            'description': 'set elastic connection timeout to this value',
+        },
         'elastic_index':{
             'default': 'fugluquar-${date}',
             'description':'Name of ElasticSearch index in which document will be stored. Template vars (e.g. ${to_domain} or ${date}) can be used.',
@@ -262,8 +278,7 @@ class ElasticBackend(AbstractBackend):
 
     def get_elastic_connection(self, reload=False):
         es = None
-        elastic_uris = self.config.getlist(self.section, 'elastic_uris')
-        verify_certs = self.config.getboolean(self.section, 'elastic_verify_certs')
+        elastic_uris = self.config.getlist(self.section, 'elastic_uris', resolve_env=True)
         if elastic_uris:
             elastic_uri_string = ','.join(elastic_uris)
             try:
@@ -271,7 +286,9 @@ class ElasticBackend(AbstractBackend):
                     del self.elastic_connection[elastic_uri_string]
                 es = self.elastic_connection[elastic_uri_string]
             except KeyError:
-                es = ElasticClient(hosts=elastic_uris, verify_certs=verify_certs, ssl_show_warn=False)
+                verify_certs = self.config.getboolean(self.section, 'elastic_verify_certs')
+                timeout = self.config.getfloat(self.section, 'elastic_timeout')
+                es = ElasticClient(hosts=elastic_uris, verify_certs=verify_certs, ssl_show_warn=False, timeout=timeout)
                 self.elastic_connection[elastic_uri_string] = es
         return es
     
@@ -282,18 +299,69 @@ class ElasticBackend(AbstractBackend):
         return indexname
     
     
+    @smart_cached_memberfunc(inputs=[])
+    def _set_index_mapping(self, indexname: str) -> None:
+        self.logger.info(f'checking mapping of index {indexname}')
+        es = self.get_elastic_connection()
+        exists = es.indices.exists(index=indexname)
+        if exists:
+            need_put = False
+            current_mapping = es.indices.get_mapping(index=indexname)
+            properties = current_mapping.get(indexname, {}).get('mappings', {}).get('properties', {})
+
+            new_mapping = LOG_MAPPING
+
+            try:
+                for key in new_mapping['properties']:
+                    if not new_mapping['properties'][key]['type'] == properties.get(key, {}).get('type'):
+                        need_put = True
+                        break
+            except KeyError as e:
+                need_put = False
+                self.logger.error(f'invalid mapping: {e.__class__.__name__}: {str(e)}')
+
+            if need_put:
+                try:
+                    r = es.indices.put_mapping(body=LOG_MAPPING, index=indexname)
+                except ElasticException as e:
+                    r = {'exc': e.__class__.__name__, 'msg': str(e)}
+                if r.get('acknowledged'):
+                    self.logger.info(f'put new mapping to elastic index {indexname}')
+                else:
+                    self.logger.info(f'error putting new mapping to elastic index {indexname} : {str(r)}')
+            else:
+                self.logger.debug(f'no need to update mapping of elastic index {indexname}')
+        else:
+            try:
+                log_mapping = {'mappings': LOG_MAPPING}
+                r = es.indices.create(indexname, body=log_mapping)
+            except ElasticException as e:
+                r = {'exc': e.__class__.__name__, 'msg': str(e)}
+            if r.get('acknowledged'):
+                self.logger.info(f'created new elastic index {indexname}')
+            else:
+                self.logger.info(f'error creating new elastic index {indexname} : {str(r)}')
+
     def archive(self, suspect, fugluid_override=None, reload=False, retry=3):
         result = {}
         try:
             original = self.config.getboolean(self.section, 'elastic_useoriginal')
             content = self._get_msg_source(suspect, original)
+
+            # store b64-encoded content if possible to account for malformed sources
+            try:
+                blob = b64encode(content).decode('ascii')
+            except Exception as e:
+                self.logger.warning(f"{suspect.id} Could not b64 encode blob due to: {str(e)}")
+                blob = None
             doc = {
                 'fugluid': fugluid_override or suspect.id,
                 'fugluid_original': suspect.id,
-                'content': force_uString(content),
+                'content': None if blob else force_uString(content),
                 'timestamp': datetime.datetime.utcfromtimestamp(suspect.timestamp),
                 'to_address': suspect.to_address,
                 'to_domain': suspect.to_domain,
+                'blob': blob,
             }
             
             extrafields = self.config.getlist(self.section, 'elastic_extrafields')
@@ -302,8 +370,9 @@ class ElasticBackend(AbstractBackend):
             
             es = self.get_elastic_connection(reload)
             indexname = self._get_elastic_index(suspect)
+            self._set_index_mapping(indexname)
             r = es.index(index=indexname, id=suspect.id, body=doc)
-            for key in ['_id']:
+            for key in ['_id', 'result']:
                 try:
                     result[key] = r[key]
                 except KeyError:
@@ -325,10 +394,11 @@ class ElasticBackend(AbstractBackend):
         
         es = self.get_elastic_connection()
         if es is None:
-            print('WARNING: elastic_uris not defined, this plugin will do nothing')
+            print('WARNING: elastic_uris not defined, this backend will do nothing')
             return False
         if not es.ping():
-            print('ERROR: failed to connect to elasticsearch, connection info: %s' % str(es))
+            elastic_uris = self.config.getlist(self.section, 'elastic_uris', resolve_env=True)
+            print(f'ERROR: failed to connect to elasticsearch {", ".join(elastic_uris)}, connection info: {str(es)}')
             return False
         
         return True
@@ -774,12 +844,14 @@ Note: The first rule to match in a message is the only rule that will be applied
             fugluid_override = None
         else:
             fugluid_override = suspect.get_message_rep().get(headername)
+            if fugluid_override is not None:
+                fugluid_override = fugluid_override.strip()
             skipmissing = self.config.getboolean(self.section, 'fugluid_headername_skipmissing')
             if not fugluid_override and skipmissing:
-                errors['preparation'] = 'could not find previous fugluid'
+                self.logger.info(f'{suspect.id} skip archiving: could not find previous fugluid in {headername}')
                 return result, errors
-            elif suspect.check_id(fugluid_override):
-                errors['preparation'] = 'previous fugluid is not valid: %s' % fugluid_override
+            elif suspect.check_id(fugluid_override) is None:
+                errors['preparation'] = f'previous fugluid is not valid: {fugluid_override}'
                 return result, errors
         
         self._load_backends()
@@ -793,8 +865,8 @@ Note: The first rule to match in a message is the only rule that will be applied
                 if not multibackend:
                     break
             except Exception as e:
-                self.logger.error('failed to archive using backend %s due to %s' % (str(backend), str(e)))
-                errors[backend_name] = str(e)
+                #self.logger.error(f'{suspect.id} failed to archive using backend {backend_name} due to {e.__class__.__name__}: {str(e)}')
+                errors[backend_name] = f'{e.__class__.__name__}: {str(e)}'
         return result, errors
     
     
@@ -834,11 +906,11 @@ Note: The first rule to match in a message is the only rule that will be applied
                     result, errors = self.do_archive(suspect)
                     if result:
                         suspect.set_tag('archived', True)
-                        self.logger.debug('%s archived successfully: %s' % (suspect.id, result))
+                        self.logger.info('%s archived successfully: %s' % (suspect.id, result))
                     if errors:
-                        self.logger.error('%s archiving errors in backend %s' % (suspect.id, ','.join(list(errors.keys()))))
+                        self.logger.warning('%s archiving errors in backend %s' % (suspect.id, ','.join(list(errors.keys()))))
                         for key in errors:
-                            self.logger.debug('%s archiving errors in backend %s: %s' % (suspect.id, key, errors[key]))
+                            self.logger.error('%s archiving errors in backend %s: %s' % (suspect.id, key, errors[key]))
                     suspect.set_tag('archived.result', result)
                     suspect.set_tag('archived.errors', errors)
                     
