@@ -16,10 +16,10 @@
 #
 import traceback
 
-from fuglu.shared import ScannerPlugin, DELETE, DUNNO, string_to_actioncode, FileList, get_outgoing_helo
+from fuglu.shared import ScannerPlugin, DELETE, DUNNO, string_to_actioncode, FileList, get_outgoing_helo, actioncode_to_string
 from fuglu.bounce import Bounce
 from fuglu.stringencode import force_uString, force_bString
-from fuglu.extensions.sql import SQL_EXTENSION_ENABLED, DBFile, DBConfig
+from fuglu.extensions.sql import SQL_EXTENSION_ENABLED, DBFile, DBConfig, RESTAPIError
 from fuglu.extensions.redisext import RedisPooledConn, redis, ENABLED as REDIS_ENABLED
 from fuglu.extensions.filearchives import Archivehandle
 from fuglu.extensions.filetype import filetype_handler
@@ -360,7 +360,7 @@ The other common template variables are available as well.
 """
 
     def __init__(self, config, section=None):
-        ScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.requiredvars = {
             'template_blockedfile': {
                 'default': '${confdir}/templates/blockedfile.tmpl',
@@ -441,7 +441,12 @@ The other common template variables are available as well.
             'enabledarchivetypes': {
                 'default': '',
                 'description': 'comma separated list of archive extensions. do only process archives of given types.',
-            }
+            },
+
+            'problemaction': {
+                'default': 'DEFER',
+                'description': "action if there is a problem (DUNNO, DEFER)",
+            },
 
         }
 
@@ -468,21 +473,25 @@ The other common template variables are available as well.
             self.rulescache = RulesCache(self.config.get(self.section, 'rulesdir'))
 
         self.blockedfiletemplate = self.config.get(self.section, 'template_blockedfile')
-
-        runtimeconfig = DBConfig(self.config, suspect)
-        self.checkarchivenames = runtimeconfig.getboolean(self.section, 'checkarchivenames')
-        self.checkarchivecontent = runtimeconfig.getboolean(self.section, 'checkarchivecontent')
-        self.sendbounce = runtimeconfig.getboolean(self.section, 'sendbounce')
-
-        enabledarchivetypes = runtimeconfig.getlist(self.section, 'enabledarchivetypes')
-        if enabledarchivetypes:
-            archtypes = list(self.active_archive_extensions.keys())
-            for archtype in archtypes:
-                if archtype not in enabledarchivetypes:
-                    del self.active_archive_extensions[archtype]
-
-        returnaction = self.walk(suspect)
-        return returnaction
+        
+        try:
+            runtimeconfig = DBConfig(self.config, suspect)
+            self.checkarchivenames = runtimeconfig.getboolean(self.section, 'checkarchivenames')
+            self.checkarchivecontent = runtimeconfig.getboolean(self.section, 'checkarchivecontent')
+            self.sendbounce = runtimeconfig.getboolean(self.section, 'sendbounce')
+            enabledarchivetypes = runtimeconfig.getlist(self.section, 'enabledarchivetypes')
+            if enabledarchivetypes:
+                archtypes = list(self.active_archive_extensions.keys())
+                for archtype in archtypes:
+                    if archtype not in enabledarchivetypes:
+                        del self.active_archive_extensions[archtype]
+            action, message = self.walk(suspect)
+        except RESTAPIError as e:
+            action = self._problemcode()
+            self.logger.warning(f'{suspect.id} {actioncode_to_string(action)} due to RESTAPIError: {str(e)}')
+            message = 'Internal Server Error'
+        
+        return action, message
     
     
     def asciionly(self, stri):
@@ -767,7 +776,7 @@ The other common template variables are available as well.
             res = self.matchMultipleSets([user_names, domain_names, default_names], att_name, suspect, att_name)
             if res == ATTACHMENT_SILENTDELETE:
                 self._debuginfo(suspect, "Attachment name=%s SILENT DELETE : blocked by name" % att_name)
-                return DELETE
+                return DELETE, None
             if res == ATTACHMENT_BLOCK:
                 self._debuginfo(suspect, "Attachment name=%s : blocked by name)" % att_name)
                 message = suspect.tags['FiletypePlugin.errormessage']
@@ -777,7 +786,7 @@ The other common template variables are available as well.
             res = self.matchMultipleSets([user_ctypes, domain_ctypes, default_ctypes], contenttype_mime, suspect, att_name)
             if res == ATTACHMENT_SILENTDELETE:
                 self._debuginfo(suspect, "Attachment name=%s content-type=%s SILENT DELETE: blocked by mime content type (message source)" % (att_name, contenttype_mime))
-                return DELETE
+                return DELETE, None
             if res == ATTACHMENT_BLOCK:
                 self._debuginfo(suspect, "Attachment name=%s content-type=%s : blocked by mime content type (message source)" % (att_name, contenttype_mime))
                 message = suspect.tags['FiletypePlugin.errormessage']
@@ -788,7 +797,7 @@ The other common template variables are available as well.
                 res = self.matchMultipleSets([user_ctypes, domain_ctypes, default_ctypes], contenttype_magic, suspect, att_name)
                 if res == ATTACHMENT_SILENTDELETE:
                     self._debuginfo(suspect, "Attachment name=%s content-type=%s SILENT DELETE: blocked by mime content type (magic)" % (att_name, contenttype_magic))
-                    return DELETE
+                    return DELETE, None
                 if res == ATTACHMENT_BLOCK:
                     self._debuginfo(suspect, "Attachment name=%s content-type=%s : blocked by mime content type (magic)" % (att_name, contenttype_magic))
                     message = suspect.tags['FiletypePlugin.errormessage']
@@ -838,7 +847,7 @@ The other common template variables are available as well.
                                 res = self.matchMultipleSets(ruleset, name, suspect, name)
                                 if res == ATTACHMENT_SILENTDELETE:
                                     self._debuginfo( suspect, "Blocked filename in archive %s SILENT DELETE" % att_name)
-                                    return DELETE
+                                    return DELETE, None
                                 if res == ATTACHMENT_BLOCK:
                                     self._debuginfo(suspect, "Blocked filename in archive %s" % att_name)
                                     message = suspect.tags['FiletypePlugin.errormessage']
@@ -867,7 +876,7 @@ The other common template variables are available as well.
                                         suspect, "Extracted file %s from archive %s content-type=%s "
                                                  "SILENT DELETE: blocked by mime content type (magic)"
                                                  % (safename, att_name, contenttype_magic))
-                                    return DELETE
+                                    return DELETE, None
                                 if res == ATTACHMENT_BLOCK:
                                     self._debuginfo(
                                         suspect, "Extracted file %s from archive %s content-type=%s : "
@@ -885,7 +894,7 @@ The other common template variables are available as well.
                     except Exception as e:
                         self.logger.debug(traceback.format_exc())
                         self.logger.error("%s archive scanning failed in attachment %s: %s" % (suspect.id, att_name, str(e)))
-        return DUNNO
+        return DUNNO, None
     
     def walk_all_parts(self, message):
         """Like email.message.Message's .walk() but also tries to find parts in the message's epilogue"""
@@ -967,15 +976,28 @@ The other common template variables are available as well.
 
 
 class AttHashAction(ScannerPlugin):
+    """
+    query attachment hashes against hashbl (e.g. spamhaus or abusix)
+    spamhaus: sha256 / b32
+    abusix: sha1 / hex
+    """
     def __init__(self, config, section=None):
         ScannerPlugin.__init__(self, config, section)
         self.logger = self._logger()
         self.rbllookup = None
         self.requiredvars = {
             'blocklistconfig':{
-                'default':'${confdir}/atthash-rbl.conf',
+                'default':'${confdir}/rblatthash.conf',
                 'description':'Domainmagic RBL lookup config file',
             },
+            'hashalgorithm': {
+                'default': 'sha256',
+                'description': 'what hash algorithm to use, any supported by hashlib',
+            },
+            'hashencoding': {
+                'default': 'b32',
+                'description': 'what hash encoding to use, e.g. hex, b64, b32',
+            },
             'enginename': {
                 'default': '',
                 'description': 'set custom engine name',
@@ -1006,10 +1028,29 @@ class AttHashAction(ScannerPlugin):
             print('ERROR: domainmagic not available - this plugin will do nothing')
             ok = False
         
+        hashenc = self.config.get(self.section, 'hashencoding').lower()
+        encoders = [Mailattachment.HASHENC_HEX, Mailattachment.HASHENC_B32, Mailattachment.HASHENC_B64]
+        if ok and hashenc not in encoders:
+            print(f'ERROR: invalid hashencoding {hashenc}, use one of {", ".join(encoders)}')
+            ok = False
+        
+        hashalgo = self.config.get(self.section, 'hashalgorithm').lower()
+        if hashalgo not in hashlib.algorithms_available:
+            print(f'ERROR: invalid hash algorithm {hashalgo}, use one of {", ".join(hashlib.algorithms_available)}')
+            ok = False
+        
         if ok:
-            result = self._check_hash(self.config.get(self.section, 'testhash'))
-            if not 'Malware' in list(result.keys()):
-                print('ERROR: Eicar not detected as Malware')
+            self._init_rbllookup()
+            if self.rbllookup is None:
+                blocklistconfig = self.config.get(self.section,'blocklistconfig')
+                print(f'ERROR: failed to load rbl config from file {blocklistconfig}')
+                ok = False
+        
+        if ok:
+            testhash = self.config.get(self.section, 'testhash')
+            result = self._check_hash(testhash)
+            if not result:
+                print(f'ERROR: Hash {testhash} not detected in rbl check')
                 ok = False
         
         return ok
@@ -1029,8 +1070,10 @@ class AttHashAction(ScannerPlugin):
             if not attobj.is_attachment:
                 self.logger.debug('Skipping inline part with filename: %s' % filename)
                 continue
-
-            myhash = attobj.get_checksum('sha256', Mailattachment.HASHENC_B32, strip=True)
+            
+            hashalgo = self.config.get(self.section, 'hashalgorithm').lower()
+            hashenc = self.config.get(self.section, 'hashencoding').lower()
+            myhash = attobj.get_checksum(hashalgo, hashenc, strip=True)
             
             result = self._check_hash(myhash)
             for blockname in result.keys():
@@ -1098,7 +1141,11 @@ class FileHashCheck(ScannerPlugin):
             'minfilesizebyext': {
                 'default': 'zip:40',
                 'description': 'comma separated list of file type specific min file size overrides. specifiy as ext:size',
-            }
+            },
+            'problemaction': {
+                'default': 'DEFER',
+                'description': "action if there is a problem (DUNNO, DEFER)",
+            },
             
         }
         
@@ -1178,25 +1225,31 @@ class FileHashCheck(ScannerPlugin):
             filecontent = attobj.buffer if attobj.buffer else b""
             
             if not attobj.is_attachment:
-                self.logger.debug('Skipping inline part with filename: %s' % filename)
+                self.logger.debug(f'{suspect.id} Skipping inline part with filename: {filename}')
                 continue
             
             if not self._check_filename(filename, len(filecontent)):
-                self.logger.debug('Skipping attachment size %s with filename: %s' % (len(filecontent), filename))
+                self.logger.debug(f'{suspect.id} Skipping attachment size {len(filecontent)} with filename: {filename}')
                 continue
             
             myhash = attobj.get_checksum(hashtype)
             if myhash in hashskiplist:
-                self.logger.debug('Skiplisted hash: %s' % myhash)
+                self.logger.debug(f'{suspect.id} Skiplisted hash: {myhash}')
                 continue
             
-            virusname = self._check_hash(myhash)
-            if virusname is not None:
-                blockinfo = {filename: f'{virusname} {myhash}'}
-                self._blockreport(suspect, blockinfo)
-                break
-            else:
-                self.logger.debug('no matching hash found for %s with hash %s' % (filename, myhash))
+            try:
+                virusname = self._check_hash(myhash)
+                if virusname is not None:
+                    blockinfo = {filename: f'{virusname} {myhash}'}
+                    self._blockreport(suspect, blockinfo)
+                    break
+                else:
+                    self.logger.debug(f'{suspect.id} no matching hash found for {filename} with hash {myhash}')
+            except Exception as e:
+                self.logger.error(f'{suspect.id} failed to retrieve hash due to {e.__class__.__name__}: {str(e)}')
+                action = self._problemcode()
+                message = 'Internal Server Error'
+                return action, message
         
         return DUNNO, None
     
@@ -1304,7 +1357,7 @@ class FileHashCheck(ScannerPlugin):
                         virusname = force_uString(result[0])
                     attempts = 0
                 except redis.exceptions.ConnectionError as e:
-                    msg = f"problem in {self.__class__.__name__} getting virusname for haxh {myhash}"
+                    msg = f"problem in {self.__class__.__name__} getting virusname for hash {myhash}"
                     self.logger.warning(msg) if attempts else self.logger.error(msg, exc_info=e)
                 except (socket.timeout, redis.exceptions.TimeoutError):
                     self.logger.info("Socket timeout in check_hash")
@@ -1381,14 +1434,16 @@ class FileHashRedis(object):
         try:
             redisconn = self.redis_pool.get_conn()
             result = redisconn.hmget(myhash, ['virusname', 'filesize'])
+            pipe = redisconn.pipeline()
             if result and result[1] == filesize:
                 virusname = result[0]
                 self.logger.debug('known hash %s has virus name %s' % (myhash, virusname))
             else:
-                redisconn.hmset(myhash, dict(filename=filename, filesize=filesize, virusname=virusname))
-            redisconn.expire(myhash, td)
+                redisconn.hset(myhash, mapping=dict(filename=filename, filesize=filesize, virusname=virusname))
+            pipe.expire(myhash, td)
+            pipe.execute()
         except redis.exceptions.ConnectionError as e:
-            self.logger.exception(e)
+            self.logger.warning(f'failed to insert hash in redis due to {e.__class__.__name__}: {str(e)}')
         except (socket.timeout, redis.exceptions.TimeoutError):
             self.logger.info("Socket timeout in insert_redis")
 
