@@ -189,18 +189,18 @@ Tags:
                 viruses = self.scan_stream(content, suspect.id)
                 success = True
                 break
-            except socket.error as e:
+            except (socket.error, socket.timeout, ConnectionRefusedError) as e:
                 self.__invalidate_socket()
 
                 # don't warn the first times if it's just a broken pipe which
                 # can happen with the new pipelining protocol
                 msg = f"{suspect.id} Problem encountered while contacting clamd " \
-                      f"(try {max_attempts - attempts}/{max_attempts}): {str(e)}"
+                      f"(try {max_attempts - attempts}/{max_attempts}): {e.__class__.__name__} {str(e)}"
                 if attempts:
                     if not e.errno == errno.EPIPE:
                         self.logger.warning(msg)
                 else:
-                    self.logger.error(msg, exc_info=e)
+                    self.logger.error(msg)
             except ClamavTimeout as e:
                 msg = f"Clamav timeout raised (try {max_attempts - attempts}/{max_attempts}): {str(e)}"
                 self.logger.warning(msg) if attempts else self.logger.error(msg)
@@ -316,7 +316,7 @@ Tags:
         self.logger.debug('%s: sent chunk %u/%u, last number of bytes sent was %u' % (suspectid, iChunk, numChunksToSend, chunklength))
         self.logger.debug('%s: All chunks sent, sending 0 - size to tell ClamAV the whole message has been sent' % suspectid)
         s.sendall(struct.pack(b'!L', 0))
-        self.logger.debug('%s: 0 has been sent, now wait for anser' % suspectid)
+        self.logger.debug('%s: 0 has been sent, now wait for answer' % suspectid)
         dr = {}
 
 
@@ -376,9 +376,9 @@ Tags:
         
         while True:
             try:
-                self.logger.debug(f"{suspectID}: try to receive chunk")
+                #self.logger.debug(f"{suspectID}: try to receive chunk")
                 chunk = sock.recv(4096)
-                self.logger.debug(f"{suspectID}: Got chunk of length {len(chunk)}")
+                #self.logger.debug(f"{suspectID}: Got chunk of length {len(chunk)}")
                 if len(chunk) == 0:
                     # Extra timeout condition here because we don't reach the else
                     # statement and won't do this extra check. This happened in a docker
@@ -438,7 +438,7 @@ Tags:
             existing_socket.settimeout(socktimeout)
             return existing_socket
 
-        clamd_HOST = self.config.get(self.section, 'host')
+        clamd_HOST = self.config.get(self.section, 'host', resolve_env=True)
         unixsocket = False
 
         try:
@@ -505,9 +505,10 @@ Tags:
             return False
         s.sendall(force_bString('PING'))
         result = s.recv(20000)
-        print("Got Pong: %s" % force_uString(result))
-        if result.strip() != b'PONG':
-            print("Invalid PONG: %s" % force_uString(result))
+        result = force_uString(result.strip())
+        print(f"Got Pong: {result}")
+        if result != 'PONG':
+            print(f"Invalid PONG: {result}")
         return True
     
     
@@ -518,5 +519,5 @@ Tags:
             return False
         s.sendall(b'VERSION')
         result = s.recv(20000)
-        print("Got Version: %s" % force_uString(result))
+        print("Got Version: %s" % force_uString(result.strip()))
         return True
