@@ -14,8 +14,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-from fuglu.shared import ScannerPlugin, DUNNO, FileList
-from fuglu.stringencode import force_uString, force_bString
+from fuglu.shared import ScannerPlugin, DUNNO, FileList, apply_template, SuspectFilter
+from fuglu.stringencode import force_bString
 import socket
 import logging
 import time
@@ -23,8 +23,8 @@ import string
 import re
 import os
 import fcntl
+import json
 from urllib.parse import urlparse
-from typing import Optional
 
 
 
@@ -230,10 +230,42 @@ class URIUDPSend(ScannerPlugin):
                 'default': 'false',
                 'description': 'log only without sending'
             },
+            
+            'filter_regex': {
+                'default': '',
+                'description': 'do not send URIs that match this regex',
+            },
+            
+            'filter_file': {
+                'default': '',
+                'description': 'suspectfilter rule file: do not send URIs if suspect conditions are met',
+            },
+            
+            'format': {
+                'default': 'text:${uri}\n${source}\n${id}',
+                'description': 'format in which to send data. supports json:field1:field2:... or text:${field1}+sep+${field2}...',
+            },
         }
         
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-    
+        self.filter = None
+        
+        
+    def _initfilter(self):
+        if self.filter is not None:
+            return True
+
+        filename = self.config.get(self.section, 'filter_file')
+        if filename is None or filename == "":
+            self.logger.debug('Filter regex file not defined')
+            return True
+
+        if not os.path.exists(filename):
+            self.logger.error(f'Filter regex file not found: {filename}')
+            return False
+
+        self.filter = SuspectFilter(filename)
+        return True
     
     
     def _get_source(self, recipient):
@@ -242,37 +274,71 @@ class URIUDPSend(ScannerPlugin):
             source = self.config.get(section, recipient).strip()
         else:
             source = self.config.get(section, 'default').strip()
-        return force_bString(source)
+        return source
     
     
+    def _format_data(self, suspect, uri, source):
+        data = ''
+        fmt = self.config.get(self.section, 'format')
+        if fmt.startswith('json:'):
+            fields = fmt.split(':')[1:]
+            values = {}
+            for field in fields:
+                if field == 'uri':
+                    values[field] = uri
+                elif field == 'source':
+                    values[field] = source
+                elif field.startwith('@'):
+                    field = field[1:]
+                    val = suspect.get_tag(field)
+                    if val is not None:
+                        values[field] = val
+                elif hasattr(suspect, field):
+                    values[field] = getattr(suspect, field)
+            data = json.dumps(values)
+        elif fmt.startswith('text:'):
+            fmt = fmt.split(':',1)[-1]
+            data = apply_template(fmt, suspect, dict(uri=uri, source=source))
+        else:
+            self.logger.error(f'{suspect.id} invalid format {fmt}')
+        return data
+    
     
     def _send_udp(self, suspect, uris):
         logonly = self.config.getboolean(self.section, 'log_only')
-        sid = force_bString(suspect.id)
         targetlist = self.config.getlist(self.section, 'targets')
         targets, errors = parse_targets(targetlist)
         for error in errors:
-            self.logger.error('%s %s in target %s' % (suspect.id, errors[error], error))
+            self.logger.error(f'{suspect.id} {errors[error]} in target {error}')
         for host, port in targets:
             for uri, source in uris:
+                data = self._format_data(suspect, uri, source)
                 if logonly:
-                    self.logger.info(f"{suspect.id} LOGONLY: uri={force_uString(uri)} src={force_uString(source)} sid={force_uString(sid)}")
+                    self.logger.info(f"{suspect.id} LOGONLY: data={data}")
                 else:
-                    content = b'%s\n%s\n%s' % (force_bString(uri.strip()), source, sid)
-                    self.sock.sendto(content, (host, port))
-                    self.logger.debug(f"{suspect.id} SENT: uri={force_uString(uri)} src={force_uString(source)} sid={force_uString(sid)} to {host}:{port}")
+                    self.sock.sendto(force_bString(data), (host, port))
+                    self.logger.debug(f"{suspect.id} SENT: data={data} to {host}:{port}")
     
     
     def _get_uris(self, suspect, tags):
         uris = []
         for tag in tags:
             uris.extend(suspect.get_tag(tag, []))
+            
+        exclude_re = self.config.get(self.section, 'filter_regex').strip()
+        if exclude_re:
+            exre = re.compile(exclude_re)
+            filtered_uris = []
+            for uri in uris:
+                if not exre.search(uri):
+                    filtered_uris.append(uri)
+            uris = filtered_uris
         return uris
 
 
-
     def examine(self, suspect):
-        return self._run(suspect)
+        self._run(suspect)
+        return DUNNO
 
 
     def process(self, suspect, decision):
@@ -281,8 +347,16 @@ class URIUDPSend(ScannerPlugin):
 
     def _run(self, suspect):
         source = self._get_source(suspect.to_address)
-        if source == force_bString(self.config.get(self.section, 'skip_source')):
-            return DUNNO
+        if source == self.config.get(self.section, 'skip_source'):
+            return
+        
+        if not self._initfilter():
+            return
+        if self.filter is not None:
+            match, arg = self.filter.matches(suspect)
+            if match:
+                self.logger.info(f'{suspect.id} not sending URIs due to filter match {arg}')
+                return
         
         body_tags = self.config.getlist(self.section, 'body_tags', separators=' ')
         uris = self._get_uris(suspect, body_tags)
@@ -293,13 +367,13 @@ class URIUDPSend(ScannerPlugin):
         
         att_tags = self.config.getlist(self.section, 'att_tags', separators=' ')
         uris = self._get_uris(suspect, att_tags)
-        attsrc = force_bString(self.config.get(self.section, 'att_source'))
+        attsrc = self.config.get(self.section, 'att_source')
         for uri in uris:
             urilist.append((uri, attsrc))
         
         self._send_udp(suspect, urilist)
-        self.logger.info('%s sent %s URIs' % (suspect.id, len(urilist)))
-    
+        self.logger.info(f'{suspect.id} sent {len(urilist)} URIs')
+        return
     
     
     def lint(self):
@@ -323,15 +397,18 @@ class URIUDPSend(ScannerPlugin):
             print('INFO: no targets defined, this plugin will do nothing')
         if errors:
             for error in errors:
-                print('ERROR: %s in target %s' % (errors[error], error))
+                print(f'ERROR: {errors[error]} in target {error}')
+            return False
+        
+        filterinit = self._initfilter()
+        if not filterinit:
+            filter_file = self.config.get(self.section, 'filter_file')
+            print(f'failed to init suspect filter with filter file {filter_file}')
             return False
         
         return True
 
 
-
-
-
 class URLTypeMap(FileList):
     def __init__(self, *args, **kwargs):
         FileList.__init__(self, *args, **kwargs)
@@ -347,7 +424,7 @@ class URLTypeMap(FileList):
             if not line:
                 continue
             if not ':' in line:
-                self.logger.debug('ignoring invalid line: %s' % line)
+                self.logger.debug(f'ignoring invalid line: {line}')
                 # print('ignoring invalid line: %s' % line)
                 continue
             
@@ -365,7 +442,6 @@ class URLTypeMap(FileList):
         return newcontent
     
     
-    
     def get(self, item):
         """Returns the current list. If the file has been changed since the last call, it will rebuild the list automatically."""
         if self.filename is not None:
