@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 # -*- coding: UTF-8 -*-
 #   Copyright 2009-2022 Oli Schacher, Fumail Project
 #
@@ -21,7 +21,7 @@ import logging
 import socket
 import os
 from email import message_from_bytes
-from fuglu.shared import ScannerPlugin, AppenderPlugin, SuspectFilter, DUNNO, FileList, get_outgoing_helo
+from fuglu.shared import ScannerPlugin, AppenderPlugin, SuspectFilter, DUNNO, ACCEPT, ALLCODES, FileList, get_outgoing_helo
 from fuglu.extensions.redisext import RedisPooledConn, redis, ENABLED as REDIS_ENABLED
 from fuglu.lib.patchedemail import PatchedMessage
 from fuglu.stringencode import force_bString
@@ -237,9 +237,8 @@ class FuzorMixin(object):
         
         try:
             self.logger.debug(
-                "suspect %s to=%s hash %s usable_body=%s predigest=%s subject=%s" %
-                (suspect.id, suspect.to_address, fuhash.digest, fuhash.bodytext_size, fuhash.predigest[:50],
-                 msg.get('Subject')))
+                "suspect %s to=%s hash=%s usable_body=%s predigest=%s subject=%s" %
+                (suspect.id, suspect.to_address, fuhash.digest, fuhash.bodytext_size, fuhash.predigest[:50], msg.get('Subject')))
         except Exception:
             pass
         
@@ -251,13 +250,14 @@ class FuzorMixin(object):
             except BackendError as e:
                 self.logger.error('failed to connect to backend: %s' % str(e))
                 have_backend = False
-
+            
+            amount = len(suspect.recipients)
             attempts = 2
             while attempts:
                 attempts -= 1
                 try:
                     self._init_backend()
-                    count = self.backend.increase(fuhash.digest)
+                    count = self.backend.increase(fuhash.digest, amount)
                     self.logger.info("suspect %s hash %s seen %s times before" % (suspect.id, fuhash.digest, count - 1))
                     attempts = 0
                 except redis.exceptions.TimeoutError as e:
@@ -322,13 +322,19 @@ class FuzorReportAppender(AppenderPlugin, FuzorMixin):
 
     def process(self, suspect, decision):
         if not REDIS_ENABLED and not KAFKA_AVAILABLE:
-            return DUNNO
+            return
+        
+        if decision not in [DUNNO, ACCEPT]:
+            self.logger.debug(f'{suspect.id} not increasing count decision {ALLCODES.get(decision)}')
+            return
         
         digest, count = self.report(suspect)
         if digest is not None and suspect.get_tag('FuZor') is None:
             suspect.set_tag('FuZor', (digest, count))
             suspect.set_tag('fuzor.digest', digest)
             suspect.set_tag('fuzor.count', count)
+            self.logger.debug(f'{suspect.id} increased fuzor count for digest {digest} by {count}')
+    
     
     
     def lint(self):
@@ -349,12 +355,10 @@ class FuzorCheck(ScannerPlugin, FuzorMixin):
                 'default': 'X-FuZor',
                 'description': 'header name',
             },
-            
             'ignorelist':{
                 'default':'',
                 'description':'path to file containing fuzor sums which should not be considered. one hash per line',
-            }
-            
+            },
         })
         self.ignorelist = None
         
@@ -499,9 +503,7 @@ class FuzorPrint(ScannerPlugin):
             self.logger.info("Predigest: %s" % fuhash.predigest)
             self.logger.info('%s: hash %s' % (suspect.id, fuhash.digest))
         else:
-            self.logger.info(
-                '%s does not produce enough data for a unique hash' %
-                suspect.id)
+            self.logger.info('%s does not produce enough data for a unique hash' % suspect.id)
         
         return DUNNO
 
@@ -624,7 +626,7 @@ class BackendError(Exception):
 
 
 class FuzorBackend(object):
-    def increase(self, digest):
+    def increase(self, digest, amount=1):
         raise NotImplementedError
     def get(self, digest):
         raise NotImplementedError
@@ -637,10 +639,10 @@ class RedisBackend(FuzorBackend):
         self.ttl = 7 * 24 * 3600
         self.logger = logging.getLogger("fuglu.fuzor.RedisBackend")
     
-    def increase(self, digest):
+    def increase(self, digest, amount=1):
         redisconn = self.redis_pool.get_conn()
         pipe = redisconn.pipeline()
-        pipe.incr(digest)
+        pipe.incr(digest, amount)
         pipe.expire(digest, self.ttl)
         try:
             result = pipe.execute()
@@ -671,10 +673,11 @@ class KafkaBackend(FuzorBackend):
         )
         self.topic = topic
     
-    def increase(self, digest):
+    def increase(self, digest, amount=1):
         messagebytes = force_bString(digest)
-        self.producer.send(self.topic, value=messagebytes, key=messagebytes)
-        return 1 # stay compatible to redis backend, return 1 as in "counter increased by 1"
+        for x in range(0, amount):
+            self.producer.send(self.topic, value=messagebytes, key=messagebytes)
+        return amount # stay compatible to redis backend, return 1 as in "counter increased by 1"
     
     def get(self, digest):
         raise BackendError('Kafka backend can only be used for reporting')
