@@ -614,7 +614,7 @@ class MailFeed(AppenderPlugin):
     
     def _send_mail(self, target_host: str, target_args: dict, suspect: Suspect):
         exceptions = target_args.get('exc', [])
-        if suspect.to_domain in exceptions:
+        if suspect.to_domain.lower() in exceptions:
             self.logger.debug(f'{suspect.id} skipping sending to {target_host}')
             return
         
@@ -632,8 +632,9 @@ class MailFeed(AppenderPlugin):
         else:
             to_address = apply_template(to_template, suspect, {'target':target_host})
         to_user, to_domain = split_mail(to_address)
-        if to_domain == get_outgoing_helo(self.config):
+        if to_domain.lower() == get_outgoing_helo(self.config):
             to_address = f'{to_user}@{target_host}'
+        to_address = to_address.lower()
         
         retry = target_args.get('retry', 3)
         disable_aiosmtp = self.config.getboolean('performance', 'disable_aiosmtp')
