@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2012-2021 Fumail Project
+#   Copyright 2012-2022 Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,14 +35,17 @@ class MessageSize(BMPEOBMixin, BasicMilterPlugin):
         super().__init__(config, section)
         self.logger = self._logger()
         self.requiredvars={
+            'datasource': {
+                'default': '',
+                'description': """
+                    sql:SELECT max_size from domain where domain_name=:domain - get from sql database :domain will be replaced with the actual domain name. must return one integer field containing maximum size in bytes
+                    tag:filtersettings:max_message_size - get from p_blwl FilterSettings backend tag
+                    """,
+            },
             'dbconnection':{
                 'default':"mysql://root@localhost/config?charset=utf8",
                 'description':'SQLAlchemy Connection string. Leave empty to disable SQL lookups',
             },
-            'domain_sql_query':{
-                'default':"SELECT max_size from domain where domain_name=:domain",
-                'description':'get from sql database :domain will be replaced with the actual domain name. must return field max_size',
-            },
             'messagetemplate':{
                 'default':'message size ${msg_size} exceeds size limit ${max_size} of recipient domain ${to_domain}',
                 'description':'reject message template for policy violators'
@@ -53,21 +56,29 @@ class MessageSize(BMPEOBMixin, BasicMilterPlugin):
                                f'applied ({",".join(BasicMilterPlugin.ALL_STATES.keys())})'
             }
         }
-
-    def _get_domain_limit(self, to_domain):
+    
+    def _get_domain_limit(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], to_domain:str) -> tp.Union[int, None]:
         max_size = None
         dbconnection = self.config.get(self.section, 'dbconnection').strip()
-        sqlquery = self.config.get(self.section, 'domain_sql_query')
-
-        # use DBConfig instead of get_domain_setting
-        if dbconnection!='' and SQL_EXTENSION_ENABLED:
+        datasource = self.config.get(self.section, 'datasource')
+        
+        if SQL_EXTENSION_ENABLED and datasource.startswith('sql:') and dbconnection != '':
             cache = get_default_cache()
+            sqlquery = datasource[4:]
+            # use DBConfig instead of get_domain_setting
             max_size = get_domain_setting(to_domain, dbconnection, sqlquery, cache, self.section, False, self.logger)
         
+        elif datasource.startswith('tag:'):
+            keylist = datasource.split(':')[1:]
+            value = sess.tags
+            for key in keylist:
+                value = value.get(key, {})
+                if isinstance(value, (int, float)):
+                    max_size = int(value)
+        
         return max_size
-
-
-
+    
+    
     def examine_eob(self, sess: tp.Union[sm.MilterSession, asm.MilterSession]) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         if not SQL_EXTENSION_ENABLED:
             return sm.CONTINUE
@@ -94,7 +105,7 @@ class MessageSize(BMPEOBMixin, BasicMilterPlugin):
         max_size_reduced = None
 
         for to_domain in to_domains:
-            max_size = self._get_domain_limit(to_domain)
+            max_size = self._get_domain_limit(sess, to_domain)
 
             if max_size:
                 if max_size_reduced is None:
@@ -134,8 +145,9 @@ class MessageSize(BMPEOBMixin, BasicMilterPlugin):
         
         try:
             dbconnection = self.config.get(self.section, 'dbconnection')
-            conn=get_session(dbconnection)
-            conn.execute("SELECT 1")
+            if dbconnection:
+                conn=get_session(dbconnection)
+                conn.execute("SELECT 1")
         except Exception as e:
             print("Failed to connect to SQL database: %s" % str(e))
             return False
