@@ -23,15 +23,15 @@ import typing as tp
 from string import Template
 import random
 import re
-from email import message_from_bytes
+from email import message_from_bytes, utils
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
-from fuglu.shared import strip_address, extract_domain, FileList, ScannerPlugin, DUNNO, apply_template, \
-    string_to_actioncode, actioncode_to_string, get_outgoing_helo
+from fuglu.shared import strip_address, extract_domain, FileList, ScannerPlugin, DUNNO, REJECT, apply_template, \
+    string_to_actioncode, actioncode_to_string, get_outgoing_helo, utcnow
 from fuglu.bounce import Bounce
-from fuglu.stringencode import force_uString
+from fuglu.stringencode import force_uString, force_bString
 from fuglu.extensions.sql import get_session, text, SQL_EXTENSION_ENABLED
-from fuglu.extensions.redisext import redis, ENABLED as REDIS_ENABLED
+from fuglu.extensions.redisext import RedisPooledConn, redis, ENABLED as REDIS_ENABLED
 from fuglu.mshared import BMPRCPTMixin, BasicMilterPlugin
 from fuglu.lib.patchedemail import PatchedMessage
 import fuglu.connectors.asyncmilterconnector as asm
@@ -40,11 +40,13 @@ from fuglu.logtools import createPIDinfo
 from .fuzor import FuzorMixin, FuzorDigest
 
 try:
-    from domainmagic.mailaddr import strip_batv, decode_srs, email_normalise_ebl
+    from domainmagic.mailaddr import strip_batv, decode_srs, email_normalise_ebl, domain_from_mail
     DOMAINMAGIC_AVAILABLE = True
 except ImportError:
     def email_normalise_ebl(address, logger=None):
         return address.lower()
+    def domain_from_mail(value, **kwargs):
+        return value.rsplit('@',1)[-1]
     DOMAINMAGIC_AVAILABLE=False
     
 try:
@@ -367,6 +369,7 @@ class SenderDomainRules(BMPRCPTMixin, BasicMilterPlugin):
         return sm.CONTINUE
 
 
+
 class NoBounce(BMPRCPTMixin, BasicMilterPlugin):
     """
     do not send bounces to certain recipient domains (e.g. to prevent listing on backscatter rbls)
@@ -376,16 +379,25 @@ class NoBounce(BMPRCPTMixin, BasicMilterPlugin):
         super().__init__(config, section=section)
         self.logger = self._logger()
         self.nobounce = None
+        self.nobounce_mx = None
         
         self.requiredvars = {
             'nobouncefile': {
                 'default': '${confdir}/nobounce.txt',
                 'description': 'list of domains to which bounces will be disallowed'
             },
+            'nobounce_mx_file': {
+                'default': '${confdir}/nobounce_mx.txt',
+                'description': 'list of mx hosts to which bounces will be disallowed (requires DNSData mx lookup on recipient)'
+            },
             'rejectmessage': {
                 'default': '${to_domain} does not accept bounces',
                 'description': 'reject message template for policy violators'
             },
+            'rejectmessage_mx': {
+                'default': '${to_domain}\'s MX {mx} does not accept bounces',
+                'description': 'reject message template for policy violators due to uncooperative mx'
+            },
             'state': {
                 'default': asm.RCPT,
                 'description': f'comma/space separated list states this plugin should be '
@@ -396,7 +408,24 @@ class NoBounce(BMPRCPTMixin, BasicMilterPlugin):
     
     def _init_lists(self):
         if self.nobounce is None:
-            self.nobounce = FileList(self.config.get(self.section, 'nobouncefile'))
+            nobouncefile = self.config.get(self.section, 'nobouncefile')
+            if nobouncefile and os.path.exists(nobouncefile):
+                self.nobounce = FileList(nobouncefile)
+        
+        if self.nobounce_mx is None:
+            nobounce_mx_file = self.config.get(self.section, 'nobounce_mx_file')
+            if nobounce_mx_file and os.path.exists(nobounce_mx_file):
+                self.nobounce_mx = FileList(nobounce_mx_file)
+    
+    
+    def _check_nobounce_mx(self, sess, rcpt_domain):
+        if self.nobounce_mx is not None:
+            mxrecs = sess.tags.get(f'dnsdata.recipient.{rcpt_domain}', {}).get('MX', [])
+            trap_mx = set(self.nobounce_mx.get_list())
+            for mxrec in mxrecs:
+                if mxrec in trap_mx:
+                    return mxrec
+        return None
     
     
     def examine_rcpt(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], recipient: bytes) \
@@ -410,13 +439,23 @@ class NoBounce(BMPRCPTMixin, BasicMilterPlugin):
             to_address = strip_address(to_address)
             to_domain = extract_domain(to_address)
             
-            nobounce = self.nobounce.get_list()
-            if to_domain in nobounce:
-                rejstring = self.config.get(self.section, 'rejectmessage')
+            if self.nobounce is not None:
+                nobounce = set(self.nobounce.get_list())
+                if to_domain in nobounce:
+                    rejstring = self.config.get(self.section, 'rejectmessage')
+                    tmpl = Template(rejstring)
+                    fields = sess.get_templ_dict()
+                    fields['to_domain'] = to_domain
+                    rejectmessage = tmpl.safe_substitute(fields)
+                    return sm.REJECT, rejectmessage
+            
+            nobounce_mx = self._check_nobounce_mx(sess, to_domain)
+            if nobounce_mx is not None:
+                rejstring = self.config.get(self.section, 'rejectmessage_mx')
                 tmpl = Template(rejstring)
-                
                 fields = sess.get_templ_dict()
-                
+                fields['mx'] = nobounce_mx
+                fields['to_domain'] = to_domain
                 rejectmessage = tmpl.safe_substitute(fields)
                 return sm.REJECT, rejectmessage
         
@@ -436,17 +475,25 @@ class NoBounce(BMPRCPTMixin, BasicMilterPlugin):
             print(fc.strcolor("ERROR - config check", "red"))
             return False
         
-        filepath = self.config.get(self.section, 'nobouncefile')
-        if not os.path.exists(filepath):
-            print(fc.strcolor('ERROR: ', "red"), f'nobouncefile {filepath} does not exist')
+        nobouncefile = self.config.get(self.section, 'nobouncefile')
+        if nobouncefile and not os.path.exists(nobouncefile):
+            print(fc.strcolor('ERROR: ', "red"), f'nobouncefile {nobouncefile} does not exist')
+            return False
+
+        nobouncefile_mx = self.config.get(self.section, 'nobounce_mx_file')
+        if nobouncefile_mx and not os.path.exists(nobouncefile_mx):
+            print(fc.strcolor('ERROR: ', "red"), f'nobounce_mx_file {nobouncefile_mx} does not exist')
             return False
         
         self._init_lists()
-        if self.nobounce is None:
-            print('ERROR: failed to initialise no bounce list')
+        if nobouncefile and self.nobounce is None:
             print(fc.strcolor('ERROR: ', "red"), 'failed to initialise no bounce list')
             return False
         
+        if nobouncefile_mx and self.nobounce_mx is None:
+            print(fc.strcolor('ERROR: ', "red"), 'failed to initialise no bounce MX list')
+            return False
+        
         return True
     
     
@@ -496,6 +543,16 @@ class TrapIntercept(ScannerPlugin):
                 'description': 'regex to match traps by pattern'
             },
             
+            'traps_mx_file': {
+                'default': '${confdir}/traps_mx.txt',
+                'description': 'file with known traps MX (requires DNSData MX lookup on recipient)'
+            },
+            
+            'traps_mxa_file': {
+                'default': '${confdir}/traps_mxa.txt',
+                'description': 'file with IPs of known traps MX (requires DNSData MXA lookup on recipient)'
+            },
+            
             'sender_exceptions_file': {
                 'default': '${confdir}/trap_sender_exceptions.txt',
                 'description': 'file with whitelisted senders'
@@ -525,6 +582,16 @@ class TrapIntercept(ScannerPlugin):
                 'description': 'sql query to enable block'
             },
             
+            'redis_conn':{
+                'default':'',
+                'description':'redis backend database connection: redis://host:port/dbid',
+            },
+            
+            'redis_timeout':{
+                'default':'2',
+                'description':'redis backend timeout in seconds',
+            },
+            
             'headername_sasluser': {
                 'default': 'X-SASL-Auth-User',
                 'description': 'Name of header to store sasl login user name',
@@ -558,23 +625,34 @@ class TrapIntercept(ScannerPlugin):
         }
         
         self.traps = None
+        self.traps_mx = None
+        self.traps_mxa = None
         self.exceptions = None
-        
+        self.redis_pool = {}
         
     
     def _init_lists(self):
         if self.traps is None:
             traps_file = self.config.get(self.section, 'traps_file')
-            if traps_file:
+            if traps_file and os.path.exists(traps_file):
                 self.traps = FileList(traps_file, additional_filters=email_normalise_ebl)
-                
+        
+        if self.traps_mx is None:
+            traps_mx_file = self.config.get(self.section, 'traps_mx_file')
+            if traps_mx_file and os.path.exists(traps_mx_file):
+                self.traps_mx = FileList(traps_mx_file)
+        
+        if self.traps_mxa is None:
+            traps_mxa_file = self.config.get(self.section, 'traps_mxa_file')
+            if traps_mxa_file and os.path.exists(traps_mxa_file):
+                self.traps_mxa = FileList(traps_mxa_file)
+        
         if self.exceptions is None:
             sender_exceptions_file = self.config.get(self.section, 'sender_exceptions_file')
-            if sender_exceptions_file:
+            if sender_exceptions_file and os.path.exists(sender_exceptions_file):
                 self.exceptions = FileList(sender_exceptions_file, additional_filters=email_normalise_ebl)
     
     
-    
     def _static_traps(self, rcpt):
         is_trap = False
         rgx = self.config.get(self.section, 'trap_regex')
@@ -583,66 +661,143 @@ class TrapIntercept(ScannerPlugin):
         return is_trap
     
     
+    def _check_trap_mx(self, suspect, rcpt_domain):
+        if self.traps_mx is not None:
+            mxrecs = suspect.get_tag(f'dnsdata.recipient.{rcpt_domain}', {}).get('MX', [])
+            trap_mx = set(self.traps_mx.get_list())
+            for mxrec in mxrecs:
+                if mxrec in trap_mx:
+                    return True
+        if self.traps_mxa is not None:
+            mxarecs = suspect.get_tag(f'dnsdata.recipient.{rcpt_domain}', {}).get('MXA', [])
+            trap_mxa = set(self.traps_mxa.get_list())
+            for mxarec in mxarecs:
+                if mxarec in trap_mxa:
+                    return True
+        return False
+    
     
     def examine(self, suspect):
+        message = self._reject_sender(suspect)
+        if message is not None:
+            return REJECT, message
+        
         self._init_lists()
         exceptions = self.exceptions.get_list()
         if email_normalise_ebl(suspect.from_address) in exceptions or suspect.from_domain in exceptions:
             self.logger.debug('%s sender %s is on exception list' % (suspect.id, suspect.from_address))
             return DUNNO
         
-        traps = set(self.traps.get_list())
-        rcpt = email_normalise_ebl(suspect.to_address)
-        if self._static_traps(rcpt) or rcpt in traps or suspect.to_domain in traps:
-            if SQL_EXTENSION_ENABLED and suspect.from_address:
-                self._block_sender(suspect)
-            self._send_mail(suspect)
-            actioncode = string_to_actioncode(self.config.get(self.section, 'actioncode'), self.config)
-            message = apply_template(self.config.get(self.section, 'rejectmessage'), suspect, {})
-            return actioncode, message
+        if self.traps is not None:
+            traps = set(self.traps.get_list())
+        else:
+            traps = {}
+        
+        for recipient in suspect.recipients:
+            rcpt = email_normalise_ebl(recipient)
+            rcpt_domain = domain_from_mail(recipient)
+            if self._static_traps(rcpt) or self._check_trap_mx(suspect, rcpt_domain) or rcpt in traps or rcpt_domain in traps:
+                if suspect.from_address and (SQL_EXTENSION_ENABLED or REDIS_ENABLED):
+                    try:
+                        self._block_sender(suspect)
+                    except Exception as e:
+                        self.logger.error(f'{suspect.id} failed to block sender {suspect.from_address} due to {e.__class__.__name__}: {str(e)}')
+                try:
+                    self._send_mail(suspect)
+                except Exception as e:
+                    reportto = self.config.get(self.section, 'report_recipient')
+                    self.logger.error(f'{suspect.id} failed to send trapintercept mail to {reportto} due to {e.__class__.__name__}: {str(e)}')
+                actioncode = string_to_actioncode(self.config.get(self.section, 'actioncode'), self.config)
+                message = apply_template(self.config.get(self.section, 'rejectmessage'), suspect, {})
+                return actioncode, message
         return DUNNO
     
     
-    
     def lint(self):
         if not self.check_config():
             print('ERROR: config error')
             return False
         
         traps_file = self.config.get(self.section, 'traps_file')
-        if not os.path.exists(traps_file):
+        if traps_file and not os.path.exists(traps_file):
             print('ERROR: cannot find traps_file %s' % traps_file)
             return False
 
-        self._init_lists()
-        if self.traps is None:
-            print('ERROR: failed to initialise traps')
+        traps_mx_file = self.config.get(self.section, 'traps_mx_file')
+        if traps_mx_file and not os.path.exists(traps_mx_file):
+            print('ERROR: cannot find traps_mx_file %s' % traps_mx_file)
             return False
-        if self.exceptions is None:
-            print('ERROR: failed to initialise exceptions list')
+
+        traps_mxa_file = self.config.get(self.section, 'traps_mxa_file')
+        if traps_mxa_file and not os.path.exists(traps_mxa_file):
+            print('ERROR: cannot find traps_mxa_file %s' % traps_mxa_file)
+            return False
+        
+        exceptions_file = self.config.get(self.section, 'sender_exceptions_file')
+        if exceptions_file and not os.path.exists(exceptions_file):
+            print('ERROR: cannot find sender_exceptions_file %s' % exceptions_file)
+            return False
+
+        self._init_lists()
+        if  traps_file and self.traps is None:
+            print(f'ERROR: failed to initialise traps from file {traps_file}')
+            return False
+        
+        if  traps_mx_file and self.traps_mx is None:
+            print(f'ERROR: failed to initialise traps MX from file {traps_mx_file}')
+            return False
+        
+        if  traps_mxa_file and self.traps_mxa is None:
+            print(f'ERROR: failed to initialise traps MXA from file {traps_mxa_file}')
+            return False
+        
+        if exceptions_file and self.exceptions is None:
+            print(f'ERROR: failed to initialise exceptions list from file {exceptions_file}')
             return False
         
         dbconnectstring = self.config.get(self.section, 'dbconnection')
         if not dbconnectstring:
             print('INFO: not using SQL backend')
-            return True
-        
-        if not SQL_EXTENSION_ENABLED:
-            print('WARNING: SQL extension not enabled, not using SQL database')
-            return False
-        
-        try:
-            dbsession = get_session(dbconnectstring)
-            dbsession.execute('SELECT 1')
-        except Exception as e:
-            print('ERROR: failed to connect to SQL database: %s' % str(e))
-            return False
+        else:
+            if not SQL_EXTENSION_ENABLED:
+                print('WARNING: SQL extension not enabled, not using SQL database')
+                return False
+            
+            try:
+                dbsession = get_session(dbconnectstring)
+                dbsession.execute('SELECT 1')
+            except Exception as e:
+                print('ERROR: failed to connect to SQL database: %s' % str(e))
+                return False
+            
+        redis_url = self.config.get(self.section, 'redis_conn')
+        if not redis_url:
+            print('INFO: not using Redis backend')
+        else:
+            redisconn = self._get_redis_conn(redis_url)
+            if not redisconn.ping():
+                print('ERROR cannot ping redis server')
+                return False
         return True
     
     
-    
     def _block_sender(self, suspect):
+        blocked = None
         dbconnectstring = self.config.get(self.section, 'dbconnection')
+        if dbconnectstring:
+            blocked = self._block_sender_sql(suspect, dbconnectstring)
+        
+        redis_url = self.config.get(self.section, 'redis_conn')
+        if redis_url:
+            blocked = self._block_sender_redis(suspect, redis_url)
+        
+        if blocked is None:
+            self.logger.warning(f'{suspect.id} no valid user block mechanism found')
+            blocked = False
+        return blocked
+    
+    
+    def _block_sender_sql(self, suspect, dbconnectstring):
         dbsession = get_session(dbconnectstring)
         
         sql_stmt_block = self.config.get(self.section, 'sql_stmt_block')
@@ -651,17 +806,80 @@ class TrapIntercept(ScannerPlugin):
         valdict = {
             'sender': suspect.from_address,
             'action': self.config.get(self.section, 'actioncode'),
-            'message': self.config.get(self.section, 'rejectmessage'),
+            'message': apply_template(self.config.get(self.section, 'rejectmessage'), suspect, {}),
             'comment': '%s to %s' % (suspect.id, suspect.to_address),
             'relay_account': get_login_from_suspect(suspect, sasl_hdr),
         }
         
+        rowcount = 0
         try:
-            dbsession.execute(sql_stmt_block, valdict)
+            result = dbsession.execute(sql_stmt_block, valdict)
+            rowcount = result.rowcount
         except Exception as e:
-            self.logger.error('%s failed to block sender: %s' % (suspect.id, str(e)))
+            self.logger.error(f'{suspect.id} failed to block sender {suspect.from_address} due to sql error: {e.__class__.__name__}: {str(e)}')
+        
+        if rowcount == 1:
+            self.logger.debug(f'{suspect.id} blocked {suspect.from_address} in sql db')
+            return True
+        elif rowcount == 0:
+            self.logger.warning(f'{suspect.id} tried to block {suspect.from_address} but nothing was updated in sql db')
+            return False
+        else:
+            self.logger.warning(f'{suspect.id} tried to block {suspect.from_address} but more than one record was updated in sql db')
+            return False
     
     
+    def _get_redis_conn(self, redis_url):
+        if self.redis_pool.get(redis_url) is None:
+            timeout = self.config.getint(self.section, 'redis_timeout', fallback=3)
+            self.redis_pool[redis_url] = RedisPooledConn(redis_url, timeout=timeout)
+        redis_pool = self.redis_pool[redis_url]
+        redisconn = redis_pool.get_conn()
+        return redisconn
+    
+    
+    def _block_sender_redis(self, suspect, redis_url):
+        """
+        call-ahead style redis backend
+        """
+        name = 'outblock-%s' % suspect.from_address
+        values = {
+            'address': suspect.from_address,
+            'domain': suspect.from_domain,
+            'positive': 'False',
+            'message': apply_template(self.config.get(self.section, 'rejectmessage'), suspect, {}),
+            'check_ts': utcnow().isoformat(sep=' ', timespec='seconds'),
+        }
+        try:
+            redisconn = self._get_redis_conn(redis_url)
+            redisconn.hset(name, mapping=values)
+            self.logger.debug(f'{suspect.id} blocked {suspect.from_address} in redis')
+            return True
+        except Exception as e:
+            self.logger.warning(f'{suspect.id} failed to block {suspect.from_address} in redis due to {e.__class__.__name__}: {str(e)}')
+            return False
+        
+    
+    def _reject_sender(self, suspect):
+        message = None
+        redis_url = self.config.get(self.section, 'redis_conn')
+        if redis_url:
+            message = self._reject_sender_redis(suspect, redis_url)
+        return message
+    
+    
+    def _reject_sender_redis(self, suspect, redis_url):
+        message = None
+        name = 'outblock-%s' % suspect.from_address
+        try:
+            redisconn = self._get_redis_conn(redis_url)
+            entry = redisconn.hmget(name, ['message'])
+            if entry is not None and entry[0] is not None:
+                message = force_uString(entry[0])
+        except Exception as e:
+            self.logger.warning(f'{suspect.id} failed to get {suspect.from_address} info from redis due to {e.__class__.__name__}: {str(e)}')
+        return message
+        
     
     def _send_mail(self, suspect):
         reportto = self.config.get(self.section, 'report_recipient')
@@ -687,6 +905,10 @@ class TrapIntercept(ScannerPlugin):
         msg['From'] = reporter
         msg['To'] = reportto
         msg['Subject'] = apply_template(self.config.get(self.section, 'subject_template'), suspect, tmpldata)
+        msg['Date'] = utils.formatdate(localtime=True)
+        msg['Message-ID'] = utils.make_msgid()
+        msg['Auto-Submitted'] = 'auto-generated'
+        msg['X-Auto-Response-Suppress'] = 'DR, RN, NRN, OOF, AutoReply'
         
         body  = 'Sender: %s\n' % suspect.from_address
         body += 'Trap Recipient: %s\n' % suspect.to_address
@@ -706,7 +928,7 @@ class TrapIntercept(ScannerPlugin):
 
 
 
-class FuzorRateLimit(FuzorMixin, ScannerPlugin):
+class FuzorRateLimit(ScannerPlugin, FuzorMixin):
     """
     This plugin checks fuzor checksum of mail against redis database.
     if threshold is exceeded, a copy of the mail will be bounced to
@@ -771,6 +993,14 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
                 'default': '',
                 'description': 'template of URI to log search results',
             },
+            'rate_limit_none_digest': {
+                'default': 'False',
+                'description': 'apply rate limits even if no fuzor digest could be calculated',
+            },
+            'problemaction': {
+                'default': 'DEFER',
+                'description': "action if there is a problem (DUNNO, DEFER)",
+            },
         })
         
         self.sender_exceptions = None
@@ -844,34 +1074,35 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
         digest, count = suspect.get_tag('FuZor', (None, 0))
         
         if digest is None:
-            self.logger.debug("digest is none... -> calculate")
+            self.logger.debug(f'{suspect.id} digest is none... -> calculate')
             maxsize = self.config.getint(self.section, 'maxsize')
             if suspect.size > maxsize:
                 stripoversize = self.config.getboolean(self.section, 'stripoversize')
                 if stripoversize:
-                    self.logger.debug('Fuzor: message too big (%u), stripping down to %u' % (suspect.size, maxsize))
+                    self.logger.debug('%s Fuzor: message too big (%u), stripping down to %u' % (suspect.id, suspect.size, maxsize))
                     msg = message_from_bytes(
                         suspect.source_stripped_attachments(maxsize=maxsize), _class=PatchedMessage
                     )
                 else:
-                    self.logger.debug('Fuzor: message too big (%u > %u), skipping' % (suspect.size, maxsize))
+                    self.logger.debug('%s Fuzor: message too big (%u > %u), skipping' % (suspect.id, suspect.size, maxsize))
                     return DUNNO
             else:
                 msg = suspect.get_message_rep()
             
             digest = FuzorDigest(msg).digest
-            self.logger.debug("digest is %s" % digest if digest else "<none>")
+            if digest is None and self.config.getboolean(self.section, 'rate_limit_none_digest'):
+                digest = '00000000000000000000000000000000'
+            self.logger.debug(f'{suspect.id} digest is {digest if digest else "<none>"}')
             
             if digest is not None:
-                suspect.debug('Fuzor digest = %s' % digest)
                 try:
                     self._init_backend()
                 except redis.exceptions.ConnectionError as e:
-                    self.logger.error('failed to connect to redis server: %s' % str(e))
+                    self.logger.error(f'{suspect.id} failed to connect to redis server: {str(e)}')
                     return DUNNO
                 
                 if self.backend is None:
-                    self.logger.warning('failed to initialise fuzor backend')
+                    self.logger.warning(f'{suspect.id} failed to initialise fuzor backend')
                     return DUNNO
                 
                 attempts = 2
@@ -879,7 +1110,7 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
                     attempts -= 1
                     try:
                         count = self.backend.get(digest)
-                        self.logger.debug("count is %u" % count)
+                        self.logger.debug(f'{suspect.id} count is {count}')
                         attempts = 0
                     except redis.exceptions.TimeoutError as e:
                         msg = f'{suspect.id} failed getting count due to {str(e)}'
@@ -887,7 +1118,7 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
                             self.logger.warning(msg)
                         else:
                             self.logger.error(msg)
-                            return DUNNO
+                            return self._problemcode()
                     except redis.exceptions.ConnectionError as e:
                         msg = f'{suspect.id} failed getting count due to {str(e)}, resetting connection'
                         if attempts:
@@ -897,7 +1128,7 @@ class FuzorRateLimit(FuzorMixin, ScannerPlugin):
                         else:
                             self.logger.error(msg)
                             self.backend = None
-                            return DUNNO
+                            return self._problemcode()
         
         threshold = self.config.getint(self.section, 'threshold')
         if count >= threshold:
@@ -1078,7 +1309,7 @@ class PFQDRateLimit(BMPRCPTMixin, BasicMilterPlugin):
     def _get_queue(self, from_domain, from_address, active_queue_factor):
         host_regex = self.config.get(self.section, 'host_regex')
         if host_regex:
-            rgx = re.compile(host_regex)
+            rgx = re.compile(force_bString(host_regex))
         else:
             rgx = None
         relay_deferred = self.qstore.get_summary(QStore.QUEUE_DEFERRED, rgx)
