@@ -17,7 +17,8 @@
 #
 import traceback
 import typing as tp
-from fuglu.shared import PrependerPlugin, AppenderPlugin, SuspectFilter, get_default_cache, Suspect, apply_template, DUNNO, REJECT, ACCEPT, DEFER
+from fuglu.shared import PrependerPlugin, AppenderPlugin, SuspectFilter, get_default_cache, Suspect, apply_template, \
+    DUNNO, REJECT, DEFER, string_to_actioncode, utcnow
 from fuglu.extensions.sql import SQL_EXTENSION_ENABLED, get_session, DBConfig, DeclarativeBase, RESTAPIError, RESTAPIConfig
 from fuglu.extensions.redisext import ENABLED as REDIS_EXTENSION_ENABLED, RedisPooledConn, ExpiringCounter, redis
 from fuglu.stringencode import force_uString
@@ -30,7 +31,6 @@ from operator import attrgetter
 import logging
 import fnmatch
 import ipaddress
-import datetime
 import configparser
 import os
 
@@ -85,7 +85,7 @@ class BlockWelcomeEntry(object):
                 setattr(self, key, kw[key])
     
     def __str__(self):
-        return f'<BlockWelcomeEntry type={self.list_type} scope={self.scope} addr={self.sender_addr} host={self.sender_host}/{self.netmask}>'
+        return f'<BlockWelcomeEntry id={self.list_id} type={self.list_type} scope={self.scope} addr={self.sender_addr} host={self.sender_host}/{self.netmask}>'
     
     def __repr__(self):
         return self.__str__()
@@ -219,9 +219,9 @@ class AbstractBackend(object):
         for item in items:
             if item == 'global' and have_global:
                 evalorder.append(GLOBALSCOPE)
-            elif item == 'domain':
+            elif item == 'domain' and suspect.to_domain:
                 evalorder.append(f'%{suspect.to_domain.lower()}')
-            elif item == 'user':
+            elif item == 'user' and suspect.to_address:
                 evalorder.append(suspect.to_address.lower())
         return evalorder
 
@@ -389,15 +389,15 @@ class FugluBlockWelcome(AbstractBackend):
         listings = {}
         if usecache:
             listings = self.cache.get_cache(key) or {}
-            self.logger.debug(f"{fugluid} Get {len(listings)} listings from cache as {listings}")
+            self.logger.debug(f"{fugluid} Got {len(listings)} listings from cache")
         if not listings:
             self.logger.debug(f"{fugluid} No cached listings, get")
             if listings is None:
                 listings = {}
             for func in [self._get_sql, self._get_rest]:
-                self.logger.debug(f"{fugluid} No cached listings, get using function {func}")
+                self.logger.debug(f"{fugluid} No cached listings, get using function {func.__name__}")
                 items = func(fugluid)
-                self.logger.debug(f"{fugluid} Got items: {items}")
+                #self.logger.debug(f"{fugluid} Got items: {items}")
                 for scope in items:
                     try:
                         listings[scope].extend(items[scope])
@@ -455,13 +455,9 @@ class FugluBlockWelcome(AbstractBackend):
         self.logger.debug(f"{fugluid} sender_domain_pct={sender_domain_pct}, sender_domain_dot={sender_domain_dot}, sender_hostip")
 
         for listing in check_listings:
-            print(f"Check listing: "
-                  f"list_scope:{listing.list_scope}, "
-                  f"sender_addr:{listing.sender_addr}, "
-                  f"sender_host:{listing.sender_host}, "
-                  f"sender_netmask:{listing.netmask}")
+            #print(f"Check listing: list_scope:{listing.list_scope}, sender_addr:{listing.sender_addr}, sender_host:{listing.sender_host}, sender_netmask:{listing.netmask}")
             if not listing.list_scope in (BlockWelcomeEntry.SCOPE_ANY, listscope):
-                self.logger.debug(f"{fugluid} checking listing: {listing} -> skip due to not in scope")
+                self.logger.debug(f"{fugluid} checking listing: {listing} -> skip due to listscope {listing.list_scope} not in scopes {BlockWelcomeEntry.SCOPE_ANY}, {listscope}")
                 continue
             if listing.sender_addr is None and listing.sender_host is None: # would allow any sender from any host
                 self.logger.debug(f"{fugluid} checking listing: {listing} "
@@ -515,7 +511,10 @@ class FugluBlockWelcome(AbstractBackend):
         return block_before_welcome
     
     
-    def _sort_listings(self, suspect, all_listings, scopes):
+    def _sort_listings(self,
+                       suspect: Suspect,
+                       all_listings: tp.Dict[str, tp.List[tp.Union[BlockWelcomeEntry, BlockWelcomeTable]]],
+                       scopes: tp.List[str]) -> tp.List[tp.Union[BlockWelcomeEntry, BlockWelcomeTable]]:
         block_before_welcome = self._block_before_welcome(suspect)
         check_listings = []
         for scope in scopes:
@@ -526,7 +525,10 @@ class FugluBlockWelcome(AbstractBackend):
         return check_listings
     
     
-    def _check_user_listings(self, suspect, all_listings):
+    def _check_user_listings(self,
+                             suspect: Suspect,
+                             all_listings: tp.Dict[str, tp.List[tp.Union[BlockWelcomeEntry, BlockWelcomeTable]]]
+                             ) -> tp.Tuple[tp.Union[BlockWelcomeEntry, BlockWelcomeTable], str]:
         eval_order = self._get_eval_order(suspect, self.config.getlist(self.section, 'fublwl_eval_order', lower=True), have_global=True)
         check_listings = self._sort_listings(suspect, all_listings, eval_order)
         
@@ -539,16 +541,22 @@ class FugluBlockWelcome(AbstractBackend):
         sender_address = suspect.from_address
         list_scope = BlockWelcomeEntry.SCOPE_ENV
 
-        self.logger.debug(f"{suspect.id} Check listings with: sender={sender_address}, clienthostname={clienthostname}, clientip={clientip}, list_scope={list_scope}, listings={check_listings}")
-        listing = self._check_listings(check_listings, sender_address, clienthostname, clientip, list_scope)
-        self.logger.debug(f"{suspect.id} Check listings returned: {listing}")
+        #self.logger.debug(f"{suspect.id} Check listings with: sender={sender_address}, clienthostname={clienthostname}, clientip={clientip}, list_scope={list_scope}, listings={check_listings}")
+        listing = self._check_listings(check_listings, sender_address, clienthostname, clientip, list_scope, suspect.id)
+        self.logger.debug(f"{suspect.id} Check listings returned: {listing} for list_scope {list_scope}")
         if not listing:
             list_scope = BlockWelcomeEntry.SCOPE_HDR
             headers = self.config.getlist(self.section, 'fublwl_header_checks')
+            header = None
             for header in headers:
                 hdr_addresses = [item[1] for item in suspect.parse_from_type_header(header=header, validate_mail=True)]
                 for sender_address in hdr_addresses:
-                    listing = self._check_listings(check_listings, sender_address, clienthostname, clientip, list_scope)
+                    listing = self._check_listings(check_listings, sender_address, clienthostname, clientip, list_scope, suspect.id)
+                    if listing:
+                        break
+                if listing:
+                    break
+            self.logger.debug(f"{suspect.id} Check listings returned: {listing} for list_scope {list_scope} in header {header}")
         return listing, list_scope
     
     
@@ -563,7 +571,7 @@ class FugluBlockWelcome(AbstractBackend):
                     self.logger.warning('%s Cannot update listing with id %s - not found' % (suspect.id, list_id))
                     return
                 listing.hitcount+=1
-                listing.lasthit=datetime.datetime.now()
+                listing.lasthit=utcnow()
                 session.flush()
                 session.remove()
         except Exception as e:
@@ -636,7 +644,7 @@ class FugluBlockWelcome(AbstractBackend):
             try:
                 errors = self._test_listings(all_listings)
                 for listing, err in errors:
-                    print('%s %s' % (err, listing))
+                    print(f'{err} {str(listing)}')
                 if errors:
                     return False
             except Exception:
@@ -649,19 +657,20 @@ class FugluBlockWelcome(AbstractBackend):
     
     def evaluate(self, suspect:Suspect):
         if not SQL_EXTENSION_ENABLED and not self.config.get(self.section, 'fublwl_restapi_uri'):
-            self.logger.debug(f"{suspect.id} Skip because SQL_EXTENSION is not enabled and fublwl_restapi is false")
+            self.logger.debug(f"{suspect.id} Skip because SQL_EXTENSION is not enabled and fublwl_restapi is not set")
             return
-
+        
         self.logger.debug(f"{suspect.id} Get listings")
         try:
             listings = self._get_listings(fugluid=suspect.id)
             self.logger.debug(f"{suspect.id} Start with {len(listings)} listings")
-        except RESTAPIError:
-            suspect.set_tag('restapi.error', True)
-            listings = {}
+        except RESTAPIError as e:
+            suspect.set_tag('restapi.error', e)
+            return
+        
         listing, list_scope = self._check_user_listings(suspect, listings)
-        self.logger.debug(f"{suspect.id} after applying user -> listing: {listing}, list_scpoe: {list_scope}")
-
+        self.logger.debug(f"{suspect.id} after applying user -> listing: {listing}, list_scope: {list_scope}")
+        
         if listing and listing.list_type == BlockWelcomeEntry.TYPE_BLOCK:
             self._add_tag(suspect, BLOCK)
         elif listing and listing.list_type == BlockWelcomeEntry.TYPE_WELCOME:
@@ -741,7 +750,6 @@ class FilterSettingsBackend(AbstractBackend):
     
     def __init__(self, config, section):
         super().__init__(config, section)
-        self.cache = get_default_cache()
         self.engine = 'FilterSettings'
         
         
@@ -758,7 +766,9 @@ class FilterSettingsBackend(AbstractBackend):
         dbsection = self.config.get(self.section, 'fs_section_name')
         
         runtimeconfig = DBConfig(self.config, suspect)
-        runtimeconfig.load_section(self.section)
+        loaded = runtimeconfig.load_section(self.section) # maybe need to catch restapierror here
+        if not loaded:
+            self.logger.warning(f'{suspect.id} failed to load runtimeconfig, will proceed without cached data')
         
         welcome_options = self.config.getlist(self.section, 'fs_welcome_options', lower=True)
         for option in welcome_options:
@@ -769,9 +779,9 @@ class FilterSettingsBackend(AbstractBackend):
                     self._add_tag(suspect, WELCOME)
             except (configparser.NoSectionError, configparser.NoOptionError) as e:
                 errors.add(str(e))
-                self.logger.debug('%s %s %s' % (suspect.id, e.__class__.__name__, str(e)))
-            except RESTAPIError:
-                suspect.set_tag('restapi.error', True)
+                self.logger.debug(f'{suspect.id} fs_welcome_options {str(e)}')
+            except RESTAPIError as e:
+                suspect.set_tag('restapi.error', e)
 
         yesno_options = self.config.getlist(self.section, 'fs_yesno_options', lower=True)
         for option in yesno_options:
@@ -781,9 +791,9 @@ class FilterSettingsBackend(AbstractBackend):
                     self._add_filtersetting_tag(suspect, option, value)
             except (configparser.NoSectionError, configparser.NoOptionError) as e:
                 errors.add(str(e))
-                self.logger.debug('%s %s %s' % (suspect.id, e.__class__.__name__, str(e)))
-            except RESTAPIError:
-                suspect.set_tag('restapi.error', True)
+                self.logger.debug(f'{suspect.id} fs_yesno_options {str(e)}')
+            except RESTAPIError as e:
+                suspect.set_tag('restapi.error', e)
 
         level_options = self.config.getlist(self.section, 'fs_level_options', lower=True)
         for option in level_options:
@@ -793,9 +803,9 @@ class FilterSettingsBackend(AbstractBackend):
                     self._add_filtersetting_tag(suspect, option, value)
             except (configparser.NoSectionError, configparser.NoOptionError) as e:
                 errors.add(str(e))
-                self.logger.debug('%s %s %s' % (suspect.id, e.__class__.__name__, str(e)))
-            except RESTAPIError:
-                suspect.set_tag('restapi.error', True)
+                self.logger.debug(f'{suspect.id} fs_level_options {str(e)}')
+            except RESTAPIError as e:
+                suspect.set_tag('restapi.error', e)
 
         value_options = self.config.getlist(self.section, 'fs_value_options', lower=True)
         for option in value_options:
@@ -806,9 +816,9 @@ class FilterSettingsBackend(AbstractBackend):
                     self._add_filtersetting_tag(suspect, option, value)
             except (configparser.NoSectionError, configparser.NoOptionError) as e:
                 errors.add(str(e))
-                self.logger.info('%s %s %s' % (suspect.id, e.__class__.__name__, str(e)))
-            except RESTAPIError:
-                suspect.set_tag('restapi.error', True)
+                self.logger.debug(f'{suspect.id} fs_value_options {str(e)}')
+            except RESTAPIError as e:
+                suspect.set_tag('restapi.error', e)
         
         return errors
     
@@ -1007,6 +1017,10 @@ class SAUserPrefBackend(AbstractBackend):
 
 class StaticBackend(AbstractBackend):
     requiredvars = {
+        'static_recipients_welcome': {
+            'default': '',
+            'description': "list of recipients that to which all mail is supposed to be welcome",
+        },
         'static_senders_welcome': {
             'default': '',
             'description': "list of senders that are always welcomelisted (domains or full addresses)",
@@ -1018,14 +1032,31 @@ class StaticBackend(AbstractBackend):
     }
     
     def evaluate(self, suspect:Suspect):
+        recipients_welcome = self.config.getlist(self.section, 'static_recipients_welcome')
+        for recipient in suspect.recipients:
+            if recipient in recipients_welcome or recipient.rsplit('@',1)[-1] in recipients_welcome:
+                self.logger.info(f'{suspect.id} is welcome (rcpt={recipient})')
+                self._add_tag(suspect, WELCOME)
+                self._set_global_tag(suspect)
+                break
+        else:
+            if recipients_welcome:
+                self.logger.debug(f'{suspect.id} is not in recipients welcome list {",".join(recipients_welcome)}')
+        
         senders_welcome = self.config.getlist(self.section, 'static_senders_welcome')
         if suspect.from_address in senders_welcome or suspect.from_domain in senders_welcome:
+            self.logger.info(f'{suspect.id} is welcome (sender={suspect.from_address})')
             self._add_tag(suspect, WELCOME)
             self._set_global_tag(suspect)
+        elif senders_welcome:
+            self.logger.debug(f'{suspect.id} is not in senders welcome list {",".join(senders_welcome)}')
             
         senders_blocked = self.config.getlist(self.section, 'static_senders_blocked')
         if suspect.from_address in senders_blocked or suspect.from_domain in senders_blocked:
+            self.logger.info(f'{suspect.id} is blocked')
             self._add_tag(suspect, BLOCK)
+        elif senders_blocked:
+            self.logger.debug(f'{suspect.id} is not in senders blocked list {",".join(senders_welcome)}')
 
     def lint(self):
         return True
@@ -1038,7 +1069,7 @@ class AutoListMixin(object):
     requiredvars = {
         'redis_conn': {
             'default': 'redis://127.0.0.1:6379/1',
-            'description': 'the redis database connection: host:port:dbid',
+            'description': 'the redis database connection URI: redis://host:port/dbid',
         },
         'redis_ttl': {
             'default': str(7 * 24 * 3600),
@@ -1109,14 +1140,14 @@ class AutoListMixin(object):
             self.backend_redis = ExpiringCounter(redis_pool, ttl, maxcount=maxcount)
     
     
-    def _address_normalise(self, address:str):
+    def _address_normalise(self, address:str) -> str:
         if DOMAINMAGIC_AVAILABLE or not address:
             return force_uString(strip_batv(email_normalise_ebl(address)))
         else:
             return address
     
     
-    def _gen_key(self, sender:str, recipient:str):
+    def _gen_key(self, sender:str, recipient:str) -> str:
         # rcpt first to allow more efficient search by rcpt
         return '%s\n%s' % (recipient, sender)
     
@@ -1173,6 +1204,8 @@ class AutoListBackend(AbstractBackend, AutoListMixin):
     def evaluate(self, suspect:Suspect):
         if not REDIS_EXTENSION_ENABLED:
             return
+        if suspect.is_welcomelisted() or suspect.is_blocklisted():
+            return
         envelope_sender = self._address_normalise(suspect.from_address)
         envelope_recipient = self._address_normalise(suspect.to_address)
         if self._envelope_skip(envelope_sender, envelope_recipient):
@@ -1193,7 +1226,7 @@ class AutoListBackend(AbstractBackend, AutoListMixin):
                 self._init_backend_redis()
                 if self.backend_redis:
                     count = self.backend_redis.get_count(rediskey)
-            except redis.exceptions.ConnectionError as e:
+            except (redis.exceptions.ConnectionError, redis.exceptions.TimeoutError) as e:
                 msg = f'{suspect.id} failed to connect to redis server (retry={bool(attempts)}): {str(e)}'
                 if attempts:
                     self.logger.warning(msg)
@@ -1308,6 +1341,8 @@ class BlockWelcomeList(PrependerPlugin):
             backend = self.backends[backend_name]
             try:
                 backend.evaluate(suspect)
+            except RESTAPIError as e:
+                suspect.set_tag('restapi.error', e)
             except Exception as e:
                 self.logger.debug(traceback.format_exc())
                 self.logger.error('%s backend %s failed to evaluate due to %s: %s' % (suspect.id, backend_name, e.__class__.__name__, str(e)))
@@ -1339,6 +1374,11 @@ class BlockWelcomeMilter(BMPRCPTMixin, BasicMilterPlugin, BlockWelcomeList):
         self.logger = self._logger()
         
         requiredvars = {
+            'welcomeaction': {
+                'default': 'DUNNO',
+                'description': "default action on welcome list hit (usually DUNNO or ACCEPT - ACCEPT will end milter processing)",
+            },
+            
             'rejectmessage': {
                 'default': 'message identified as spam',
                 'description': "reject message template if running in milter mode",
@@ -1389,7 +1429,7 @@ class BlockWelcomeMilter(BMPRCPTMixin, BasicMilterPlugin, BlockWelcomeList):
                 stageaction = REJECT
                 message = apply_template(self.config.get(self.section, 'rejectmessage'), suspect)
             elif suspect.is_welcomelisted():
-                stageaction = ACCEPT
+                stageaction = string_to_actioncode(self.config.get(self.section, 'welcomeaction'), self.config)
                 
                 wltag = self.config.get(self.section, 'wltagvalue')
                 wlname = self.config.get(self.section, 'wltagname')
@@ -1455,14 +1495,7 @@ class AutoListAppender(AppenderPlugin, AutoListMixin):
         if not suspect.is_ham():
             self.logger.debug(f'{suspect.id} Skipped non ham message')
             return
-        
-        envelope_sender = self._address_normalise(suspect.from_address)
-        envelope_recipient = self._address_normalise(suspect.to_address)
-        
-        if self._envelope_skip(envelope_sender, envelope_recipient):
-            self.logger.debug(f'{suspect.id} Skipped due to envelope sender={envelope_sender} recipient={envelope_recipient}')
-            return
-        
+            
         header = self._header_skip(suspect)
         if header:
             self.logger.debug(f'{suspect.id} Skipped due to header {header}')
@@ -1470,39 +1503,47 @@ class AutoListAppender(AppenderPlugin, AutoListMixin):
         
         sascore = self._get_spamscore(suspect)
         if sascore > self.config.getfloat(self.section, 'max_sa_score'):
-            self.logger.debug("%s skipping mail from %s to %s with sascore %s" % (suspect.id, envelope_sender, envelope_recipient, sascore))
+            self.logger.debug("%s Skipped due to spam score %s" % (suspect.id, sascore))
             return
         
-        if self.config.getint(self.section, 'trustedport') == suspect.get_tag('incomingport'):
-            self.logger.debug("%s mail received on trusted port %s with sascore %s" % (suspect.id, suspect.get_tag('incomingport'), sascore))
-            rediskey = self._gen_key(envelope_recipient, envelope_sender)
-            increment = self.config.getint(self.section, 'increment_trusted')
-        else:
-            self.logger.debug("%s mail received on non-trusted port %s with sascore %s" % (suspect.id, suspect.get_tag('incomingport'), sascore))
-            rediskey = self._gen_key(envelope_sender, envelope_recipient)
-            increment = self.config.getint(self.section, 'increment')
-        
-        attempts = 2
-        while attempts:
-            attempts -= 1
-            try:
-                self._init_backend_redis()
-                if self.backend_redis:
-                    result = self.backend_redis.increase(rediskey, increment)
-                    if result == 0:
-                        self.logger.error(f'{suspect.id} failed to register mail from {envelope_sender} to {envelope_recipient} due to TimeoutError in backend (result==0)')
-                    else:
-                        self.logger.debug(f"{suspect.id} mail from {envelope_sender} to {envelope_recipient} registered with result {result}")
-                attempts = 0
-            except redis.exceptions.TimeoutError as e:
-                # don't retry here...
-                self.logger.error(f'{suspect.id} failed to register mail from {envelope_sender} to {envelope_recipient} due to {str(e)}')
-                attempts = 0
-            except redis.exceptions.ConnectionError as e:
-                # retry possible for connection errors
-                msg = f'{suspect.id} (retry={bool(attempts)})failed to register mail from {envelope_sender} to {envelope_recipient} due to {str(e)}, resetting connection'
-                self.logger.warning(msg) if attempts else self.logger.error(msg)
-                self.backend_redis = None
+        envelope_sender = self._address_normalise(suspect.from_address)
+        for recipient in suspect.recipients:
+            envelope_recipient = self._address_normalise(recipient)
+            
+            if self._envelope_skip(envelope_sender, envelope_recipient):
+                self.logger.debug(f'{suspect.id} Skipped due to envelope sender={envelope_sender} recipient={envelope_recipient}')
+                return
+            
+            if self.config.getint(self.section, 'trustedport') == suspect.get_tag('incomingport'):
+                self.logger.debug("%s mail received on trusted port %s with sascore %s" % (suspect.id, suspect.get_tag('incomingport'), sascore))
+                rediskey = self._gen_key(envelope_recipient, envelope_sender)
+                increment = self.config.getint(self.section, 'increment_trusted')
+            else:
+                self.logger.debug("%s mail received on non-trusted port %s with sascore %s" % (suspect.id, suspect.get_tag('incomingport'), sascore))
+                rediskey = self._gen_key(envelope_sender, envelope_recipient)
+                increment = self.config.getint(self.section, 'increment')
+            
+            attempts = 2
+            while attempts:
+                attempts -= 1
+                try:
+                    self._init_backend_redis()
+                    if self.backend_redis:
+                        result = self.backend_redis.increase(rediskey, increment)
+                        if result == 0:
+                            self.logger.error(f'{suspect.id} failed to register mail from {envelope_sender} to {envelope_recipient} due to TimeoutError in backend (result==0)')
+                        else:
+                            self.logger.debug(f"{suspect.id} mail from {envelope_sender} to {envelope_recipient} registered with result {result}")
+                    attempts = 0
+                except redis.exceptions.TimeoutError as e:
+                    # don't retry here...
+                    self.logger.error(f'{suspect.id} failed to register mail from {envelope_sender} to {envelope_recipient} due to {str(e)}')
+                    attempts = 0
+                except redis.exceptions.ConnectionError as e:
+                    # retry possible for connection errors
+                    msg = f'{suspect.id} (retry={bool(attempts)})failed to register mail from {envelope_sender} to {envelope_recipient} due to {str(e)}, resetting connection'
+                    self.logger.warning(msg) if attempts else self.logger.error(msg)
+                    self.backend_redis = None
     
     
     def lint(self):
