@@ -64,6 +64,7 @@ class PluginSkipper(PrependerPlugin):
             if name in skipplugindict:
                 listcopy.remove(plug)
                 suspect.set_tag(f'{name}.skipreason', skipplugindict[name])
+                self.logger.debug(f'{suspect.id} skipping plugin {name} due to {skipplugindict[name]}')
         return listcopy
     
     
@@ -138,6 +139,7 @@ class AppenderSkipper(PrependerPlugin):
             name = plug.__class__.__name__
             if name in skippluginlist:
                 listcopy.remove(plug)
+                self.logger.debug(f'{suspect.id} skipping appender {plug}')
         return listcopy
     
     
