@@ -915,16 +915,16 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
         else:
             raise ValueError(f"ret type should be tuple(int, str) or int -> but is {type(ret)}")
 
-    async def examine_connect(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], host: str, addr: str) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
+    async def examine_connect(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], host: bytes, addr: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         return self._convert_return2milter(await self.core(suspect=sess, state=CONNECT))
 
-    async def examine_helo(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], helo: str) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
+    async def examine_helo(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], helo: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         return self._convert_return2milter(await self.core(suspect=sess, state=HELO))
 
-    async def examine_mailfrom(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], sender: str) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
+    async def examine_mailfrom(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], sender: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         return self._convert_return2milter(await self.core(suspect=sess, state=MAILFROM))
 
-    async def examine_rcpt(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], recipient: str) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
+    async def examine_rcpt(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], recipient: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         return self._convert_return2milter(await self.core(suspect=sess, state=RCPT))
 
     async def examine_header(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], key: bytes, value: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
