@@ -30,8 +30,10 @@ try:
     import aioredis
     AIOREDIS_AVAILABLE = 1
     aioConnectionError = aioredis.exceptions.ConnectionError
+    aioTimeoutError = aioredis.exceptions.TimeoutError
 except ImportError:
     aioConnectionError = ConnectionError
+    aioTimeoutError = TimeoutError
 
 STRATEGY = 'sliding-window'
 BACKENDS = defaultdict(dict)
@@ -262,16 +264,19 @@ if AIOREDIS_AVAILABLE:
                         ttl = int(ttl.seconds)
                     await r.expire(eventname, ttl)
                     success = True
-                except (ConnectionError, aioConnectionError) as e:
+                    attempts = 0  # no more attempts
+                except (ConnectionError, aioConnectionError, TimeoutError, aioTimeoutError) as e:
                     self._redis = None
                     if attempts:
                         self.logger.warning(f"Connection error in 'add' - retry {str(e)}")
+                        await asyncio.sleep(0.1)
                     else:
                         self.logger.error(f"Connection error in 'add' {str(e)}")
                 except Exception as e:
                     self._redis = None
                     if attempts:
                         self.logger.warning(f"Connection error in 'add' - retry: {str(e)}")
+                        await asyncio.sleep(0.1)
                     else:
                         self.logger.error(f"Connection error in 'add': {str(e)}", exc_info=e)
             return success
@@ -286,16 +291,19 @@ if AIOREDIS_AVAILABLE:
                     r = await self.redis
                     event = await r.hgetall(eventname)
                     success = True
-                except (ConnectionError, aioConnectionError) as e:
+                    attempts = 0  # no more attempts
+                except (ConnectionError, aioConnectionError, TimeoutError, aioTimeoutError) as e:
                     self._redis = None
                     if attempts:
                         self.logger.warning(f"Connection error in 'get_event' - retry {str(e)}")
+                        await asyncio.sleep(0.1)
                     else:
                         self.logger.error(f"Connection error in 'get_event' {str(e)}")
                 except Exception as e:
                     self._redis = None
                     if attempts:
                         self.logger.warning(f"Connection error in 'get_event' - retry: {str(e)}")
+                        await asyncio.sleep(0.1)
                     else:
                         self.logger.error(f"Connection error in 'get_event': {str(e)}", exc_info=e)
             return event, success
@@ -311,16 +319,19 @@ if AIOREDIS_AVAILABLE:
                     except AttributeError:
                         # aioredis >= 2.0
                         await r.hset(eventname, mapping=event_data)
-                except (ConnectionError, aioConnectionError) as e:
+                    attempts = 0   # no more attempts
+                except (ConnectionError, aioConnectionError, TimeoutError, aioTimeoutError) as e:
                     self._redis = None
                     if attempts:
                         self.logger.warning(f"Connection error in 'update' - retry {str(e)}")
+                        await asyncio.sleep(0.1)
                     else:
                         self.logger.error(f"Connection error in 'update' {str(e)}")
                 except Exception as e:
                     self._redis = None
                     if attempts:
                         self.logger.warning(f"Connection error in 'update' - retry: {str(e)}")
+                        await asyncio.sleep(0.1)
                     else:
                         self.logger.error(f"Connection error in 'update': {str(e)}", exc_info=e)
 
@@ -339,16 +350,19 @@ if AIOREDIS_AVAILABLE:
                     except AttributeError:
                         # aioredis >= 2.0
                         await r.hset(eventname, mapping=event_data)
-                except (ConnectionError, aioConnectionError) as e:
+                    attempts = 0  # no more attempts
+                except (ConnectionError, aioConnectionError, TimeoutError, aioTimeoutError) as e:
                     self._redis = None
                     if attempts:
                         self.logger.warning(f"Connection error in 'set_mitigate' - retry {str(e)}")
+                        await asyncio.sleep(0.1)
                     else:
                         self.logger.error(f"Connection error in 'set_mitigate' {str(e)}")
                 except Exception as e:
                     self._redis = None
                     if attempts:
                         self.logger.warning(f"Connection error in 'set_mitigate' - retry: {str(e)}")
+                        await asyncio.sleep(0.1)
                     else:
                         self.logger.error(f"Connection error in 'set_mitigate': {str(e)}", exc_info=e)
 
