@@ -600,7 +600,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
         restrictions_sets = {}
         setupdict = defaultdict(list)
 
-        filename = self.config.get(self.section, 'restrictionfile')
+        filename = self.config.get(self.section, 'restrictionfile', fallback='')
         if not filename or not filename.strip():
             # no definitions -> return empty
             if ignoreerrors:
@@ -899,16 +899,16 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
 
         return sm.CONTINUE
 
-    def examine_connect(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], host: str, addr: str) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
+    def examine_connect(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], host: bytes, addr: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         return self._examine_general(sess=sess, state=asm.CONNECT)
 
-    def examine_helo(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], helo: str) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
+    def examine_helo(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], helo: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         return self._examine_general(sess=sess, state=asm.HELO)
 
-    def examine_mailfrom(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], sender: str) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
+    def examine_mailfrom(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], sender: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         return self._examine_general(sess=sess, state=asm.MAILFROM)
 
-    def examine_rcpt(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], recipient: str) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
+    def examine_rcpt(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], recipient: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         return self._examine_general(sess=sess, state=asm.RCPT)
 
     def examine_header(self, sess: tp.Union[sm.MilterSession, asm.MilterSession], key: bytes, value: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
