@@ -14,8 +14,8 @@
 # limitations under the License.
 #
 #
-from fuglu.shared import ScannerPlugin, DUNNO, Suspect, string_to_actioncode, apply_template
-from fuglu.extensions.sql import DBConfig, get_session, SQL_EXTENSION_ENABLED, DeclarativeBase
+from fuglu.shared import ScannerPlugin, DUNNO, DEFER, Suspect, string_to_actioncode, apply_template, actioncode_to_string
+from fuglu.extensions.sql import DBConfig, get_session, SQL_EXTENSION_ENABLED, DeclarativeBase, RESTAPIError
 from fuglu.stringencode import force_bString, force_uString
 from fuglu.lib.patchedemail import PatchedMessage
 import time
@@ -172,6 +172,12 @@ Tags:
                 'description': "Suspect tags to attach as text part to message for scanning",
             },
 
+            'oversize_attach_suspect_tags': {
+                'default': '',
+                'description': "Suspect tags to attach as text part to message for scanning if message "
+                               "has been stripped due to size",
+            },
+
         }
         self.logger = self._logger()
 
@@ -272,15 +278,20 @@ Tags:
 
     def check_sql_blocklist(self, suspect, runtimeconfig=None):
         """Check this message against the SQL blocklist. returns highspamaction on hit, DUNNO otherwise"""
+        
+        action = DUNNO
+        message = None
         if not SQL_EXTENSION_ENABLED:
             self.logger.error('Cannot check sql blocklist, SQLALCHEMY extension is not available')
-            return DUNNO
+            return action, message
         
         try:
             blocklistings = self._get_sql_blocklist(suspect)
         except Exception as e:
             self.logger.error('%s Could not read UserPrefs: %s' % (suspect.id, str(e)))
-            return DUNNO
+            action = self._problemcode()
+            message = 'Internal Server Error'
+            return action, message
         
         for result in blocklistings:
             self.logger.debug('%s checking blocklist value %s' % (suspect.id, result.value))
@@ -296,18 +307,22 @@ Tags:
                 confcheck = self.config
                 if runtimeconfig is not None:
                     confcheck = runtimeconfig
-                configaction = string_to_actioncode(confcheck.get(self.section, 'highspamaction'), self.config)
+                try:
+                    action = string_to_actioncode(confcheck.get(self.section, 'highspamaction'), self.config)
+                    values = dict(spamscore='n/a')
+                    message = apply_template(self.config.get(self.section, 'rejectmessage'), suspect, values)
+                except RESTAPIError as e:
+                    action = self._problemcode()
+                    self.logger.warning(f'{suspect.id} {actioncode_to_string(action)} due to RESTAPIError: {str(e)}')
+                    message = 'Internal Server Error'
                 highspamlevel = self.config.getfloat(self.section, 'highspamlevel')
                 self._spamreport(suspect, True, True, None, highspamlevel, enginename='SpamAssassin')
                 prependheader = self.config.get( 'main', 'prependaddedheaders')
                 suspect.add_header("%sBlacklisted" % prependheader, result.value) # deprecated
                 suspect.add_header("%sBlocklisted" % prependheader, result.value)
                 suspect.debug('Sender is Blocklisted: %s' % result.value)
-                if configaction is None:
-                    return DUNNO
-                return configaction
         
-        return DUNNO
+        return action, message
 
 
     def _extract_spamstatus(self, msgrep, spamheadername, suspect):
@@ -394,7 +409,7 @@ Tags:
         else:
             self.logger.debug(f'{suspect.id} Return-Path already set as {msgrep["Return-Path"]}')
 
-        extralines = self._text_from_tags(suspect=suspect)
+        extralines = self._text_from_tags(suspect=suspect, stripped=stripped)
         if extralines:
             msgrep = email.message_from_bytes(content, _class=PatchedMessage)
             if msgrep.is_multipart():
@@ -423,7 +438,7 @@ Tags:
                     finallist.extend([t.strip() for t in tag.split(' ') if t.strip()])
         return finallist
 
-    def _text_from_tags(self, suspect):
+    def _text_from_tags(self, suspect, stripped: bool = False):
         """Collect lines to append from given tags"""
         lines = []
         appendtags = self.config.get(self.section, 'attach_suspect_tags')
@@ -432,12 +447,23 @@ Tags:
         except Exception:
             tags = []
 
-        for tag in tags:
-            tag = force_uString(suspect.get_tag(tag, []))
+        oversize_tags = []
+        if stripped:
+            oversize_appendtags = self.config.get(self.section, 'oversize_attach_suspect_tags')
+            try:
+                oversize_tags = Suspect.getlist_space_comma_separated(oversize_appendtags)
+            except Exception:
+                pass
+
+        # make unique list
+        tagnames = list(set(tags + oversize_tags))
+
+        for tagname in tagnames:
+            tag = force_uString(suspect.get_tag(tagname, []))
             if not isinstance(tag, list):
                 tag = [tag]
             lines.extend(tag)
-            self.logger.debug(f"{suspect.id} Got {len(tag)} additional lines from suspect tag {tag}")
+            self.logger.debug(f"{suspect.id} Got {len(tag)} additional lines from suspect tag {tagname}: {tag}")
         return lines
 
     def examine(self, suspect):
@@ -525,29 +551,35 @@ Tags:
         
         action = DUNNO
         message = None
-        
-        if isspam:
-            self.logger.debug('%s Message is spam' % suspect.id)
-            suspect.debug('Message is spam')
+        try:
+            ishighspam = self._is_highspam(runtimeconfig, spamscore)
+            if isspam:
+                values = dict(spamscore=spamscore)
+                message = apply_template(self.config.get(self.section, 'rejectmessage'), suspect, values)
             
-            configaction = string_to_actioncode(runtimeconfig.get(self.section, 'lowspamaction'), self.config)
-            if configaction is not None:
-                action = configaction
-            values = dict(spamscore=spamscore)
-            message = apply_template(self.config.get(self.section, 'rejectmessage'), suspect, values)
-        else:
-            self.logger.debug('%s Message is not spam' % suspect.id)
-            suspect.debug('Message is not spam')
+            if isspam and not ishighspam:
+                self.logger.debug('%s Message is spam' % suspect.id)
+                suspect.debug('Message is spam')
+                configaction = string_to_actioncode(runtimeconfig.get(self.section, 'lowspamaction'), self.config)
+                if configaction is not None:
+                    action = configaction
+            else:
+                self.logger.debug('%s Message is not spam' % suspect.id)
+                suspect.debug('Message is not spam')
+            
+            if spamscore is not None:
+                suspect.tags['SAPlugin.spamscore'] = spamscore # deprecated
+            if ishighspam:
+                configaction = string_to_actioncode(runtimeconfig.get(self.section, 'highspamaction'), self.config)
+                if configaction is not None:
+                    action = configaction
+                    
+            self._spamreport(suspect, isspam, ishighspam, report, spamscore, enginename='SpamAssassin')
+        except RESTAPIError as e:
+            self.logger.warning(f'{suspect.id} failed to query REST API: {str(e)}')
+            action = self._problemcode()
+            message = 'Internal Server Error'
         
-        if spamscore is not None:
-            suspect.tags['SAPlugin.spamscore'] = spamscore # deprecated
-        ishighspam = self._is_highspam(runtimeconfig, spamscore)
-        if ishighspam:
-            configaction = string_to_actioncode(runtimeconfig.get(self.section, 'highspamaction'), self.config)
-            if configaction is not None:
-                action = configaction
-        
-        self._spamreport(suspect, isspam, ishighspam, report, spamscore, enginename='SpamAssassin')
         return action, message
     
     
