@@ -249,11 +249,7 @@ def fuglu_process_worker(queue, config, shared_state, child_to_server_messages,
     # Setup address compliance checker
     # -> Due to default linux forking behavior this should already
     #    have the correct setup but it's better not to rely on this
-    try:
-        address_check = config.get('main','address_compliance_checker')
-    except Exception as e:
-        # might happen for some tests which do not propagate defaults
-        address_check = "Default"
+    address_check = config.get('main','address_compliance_checker', fallback='Default')
     Addrcheck().set(address_check)
 
 
