@@ -20,12 +20,11 @@ import traceback
 import tempfile
 import time
 import os
-import datetime
 from functools import reduce
 import typing as tp
 
 from fuglu.stats import Statskeeper, StatDelta
-from fuglu.shared import DUNNO, ACCEPT, REJECT, DEFER, DELETE
+from fuglu.shared import DUNNO, ACCEPT, REJECT, DEFER, DELETE, utcnow
 from fuglu.connectors.check import HealthCheckSuspect
 from fuglu.debug import CrashStore
 from fuglu.stringencode import force_uString
@@ -182,7 +181,7 @@ class TrackTimings(object):
 
         """
         if not self.timingshash:
-            self.timings_logger.debug('no timings to report')
+            self.timings_logger.debug(f'{suspectid} no timings to report')
             return
 
         if end:
@@ -263,12 +262,7 @@ class SessionHandler(TrackTimings):
         #            for several messages
         self.action = None
         self.message = None
-
-        try:
-            self.enabletimetracker = config.getboolean('main', 'scantimelogger')
-        except Exception:
-            self.enabletimetracker = False
-
+        self.enabletimetracker = config.getboolean('main', 'scantimelogger', fallback=False)
         self.logger.debug("enabletimetracker = %s" % self.enabletimetracker)
 
     def resetvars(self):
@@ -282,6 +276,11 @@ class SessionHandler(TrackTimings):
     def set_workerstate(self, status):
         if self.worker is not None:
             self.worker.workerstate = status
+            
+    def _gen_retmessage(self, suspect, msg):
+        if self.config.getboolean('main', 'suspectidreturn') and suspect.id not in msg:
+            msg = f'{msg} ({suspect.id})'
+        return msg
 
     def handlesession(self, worker=None):
 
@@ -395,29 +394,21 @@ class SessionHandler(TrackTimings):
                         self._defer()
 
                 elif result == DELETE:
-                    self.logger.info("MESSAGE DELETED: %s" % suspect.id)
-                    retmesg = 'OK: (%s)' % suspect.id
-                    if self.message is not None:
-                        retmesg = self.message
-                    if suspect.id not in retmesg:
-                        # make sure fugluid is in return message
-                        retmesg = f"{retmesg} ({suspect.id})"
+                    self.logger.info(f'MESSAGE DELETED: {suspect.id}')
+                    retmesg = self.message or 'OK'
+                    retmesg = self._gen_retmessage(suspect, retmesg)
                     self.protohandler.discard(retmesg)
                 elif result == REJECT:
-                    retmesg = "Rejected by content scanner"
-                    if self.message is not None:
-                        retmesg = self.message
-                    if suspect.id not in retmesg:
-                        # make sure fugluid is in return message
-                        retmesg = f"{retmesg} ({suspect.id})"
-                    retmesg = "%s (%s)" % (retmesg, suspect.id)
+                    retmesg = self.message or 'Rejected by content scanner'
+                    retmesg = self._gen_retmessage(suspect, retmesg)
                     self.protohandler.reject(retmesg)
                 elif result == DEFER:
                     message_is_deferred = True
-                    self._defer(self.message)
+                    retmesg = self.message or 'Try again later'
+                    retmesg = self._gen_retmessage(suspect, retmesg)
+                    self._defer(retmesg)
                 else:
-                    self.logger.error(
-                        'Invalid Message action Code: %s. Using DEFER' % result)
+                    self.logger.error(f'{suspect.id} Invalid Message action Code: {result}. Using DEFER')
                     message_is_deferred = True
                     self._defer()
 
@@ -427,7 +418,7 @@ class SessionHandler(TrackTimings):
                     self.tracktime("Increase-Counters")
                     self.run_appenders(suspect, result, applist)
                 else:
-                    self.logger.warning("DEFERRED %s" % suspect.id)
+                    self.logger.warning(f'DEFERRED {suspect.id}')
 
                 # clean up
                 if suspect.tempfilename() != "(buffer-only)":
@@ -447,15 +438,8 @@ class SessionHandler(TrackTimings):
                 except Exception:
                     pass
 
-                try:
-                    address_compliance_fail_action = self.config.get('main','address_compliance_fail_action').lower()
-                except Exception:
-                    address_compliance_fail_action = "defer"
-
-                try:
-                    message = self.config.get('main','address_compliance_fail_message')
-                except Exception:
-                    message = "invalid sender or recipient address"
+                address_compliance_fail_action = self.config.get('main','address_compliance_fail_action', fallback='defer').lower()
+                message = self.config.get('main','address_compliance_fail_message', fallback='invalid sender or recipient address')
 
                 if address_compliance_fail_action   == "defer":
                     self._defer(message)
@@ -476,11 +460,7 @@ class SessionHandler(TrackTimings):
             finally:
                 # finally is also executed if there's a return statement somewhere in try-except
 
-                try:
-                    remove_tmpfiles_on_error = self.config.getboolean('main', 'remove_tmpfiles_on_error')
-                except Exception:
-                    remove_tmpfiles_on_error = True
-
+                remove_tmpfiles_on_error = self.config.getboolean('main', 'remove_tmpfiles_on_error', fallback=True)
                 if suspect is None:
                     # if there was an error creating the suspect, check if the filename can be
                     # extracted from the protohandler
@@ -490,28 +470,28 @@ class SessionHandler(TrackTimings):
 
                     if remove_tmpfiles_on_error:
                         if tmpfilename:
-                            self.logger.debug(message_prefix+u'Remove tmpfile: %s for failed message' % tmpfilename)
+                            self.logger.debug(message_prefix+'Remove tmpfile: %s for failed message' % tmpfilename)
                         self.protohandler.remove_tmpfile()
                     else:
                         if tmpfilename:
-                            self.logger.warning(message_prefix+u'Keep tmpfile: %s for failed message' % tmpfilename)
+                            self.logger.warning(message_prefix+'Keep tmpfile: %s for failed message' % tmpfilename)
                         else:
-                            self.logger.warning(message_prefix+u'No tmpfile to keep for failed message')
+                            self.logger.warning(message_prefix+'No tmpfile to keep for failed message')
 
                 elif suspect.tempfilename() != "(buffer-only)":
                     # suspect was created but not stopped cleanly
                     if remove_tmpfiles_on_error:
                         try:
                             os.remove(suspect.tempfilename())
-                            self.logger.debug(message_prefix+u'Removed tempfile %s' % suspect.tempfilename())
+                            self.logger.debug(message_prefix+'Removed tempfile %s' % suspect.tempfilename())
                         except OSError:
-                            self.logger.warning(message_prefix+u'Could not remove tempfile %s' % suspect.tempfilename())
+                            self.logger.warning(message_prefix+'Could not remove tempfile %s' % suspect.tempfilename())
                     else:
-                        self.logger.warning(message_prefix+u'Keep tempfile %s for failed message' % suspect.tempfilename())
+                        self.logger.warning(message_prefix+'Keep tempfile %s for failed message' % suspect.tempfilename())
 
                 # try to remove the suspect
                 try:
-                    self.logger.debug(message_prefix+u'Remove suspect (current refs): %u' % sys.getrefcount(suspect))
+                    self.logger.debug(message_prefix+'Remove suspect (current refs): %u' % sys.getrefcount(suspect))
                     suspectid = suspect.id
                     del suspect
                 except Exception:
@@ -529,10 +509,10 @@ class SessionHandler(TrackTimings):
                     except AttributeError:
                         keep_connection = False
 
-                self.logger.debug(message_prefix+u'Session finished')
+                self.logger.debug(message_prefix+'Session finished')
 
         if isuspect > 2:
-            self.logger.info(message_prefix+u'Session finished for %u sessions in same connection' % (isuspect-1))
+            self.logger.info(message_prefix+'Session finished for %u sessions in same connection' % (isuspect-1))
 
 
     def _discard(self, message=None):
@@ -579,10 +559,9 @@ class SessionHandler(TrackTimings):
             try:
                 os.makedirs(trashdir)
             except OSError:
-                self.logger.error(
-                    "Trashdir %s does not exist and could not be created" % trashdir)
+                self.logger.error(f"{suspect.id } Trashdir {trashdir} does not exist and could not be created")
                 return
-            self.logger.info('Created trashdir %s' % trashdir)
+            self.logger.info(f'{suspect.id }Created trashdir {trashdir}')
 
         trashfilename = ''
         try:
@@ -590,22 +569,21 @@ class SessionHandler(TrackTimings):
                 prefix=suspect.id, dir=self.config.get('main', 'trashdir'))
             with os.fdopen(handle, 'w+b') as trashfile:
                 trashfile.write(suspect.get_source())
-            self.logger.debug('Message stored to trash: %s' % trashfilename)
+            self.logger.debug(f'{suspect.id} Message stored to trash: {trashfilename}')
         except Exception as e:
-            self.logger.error(
-                "could not create file %s: %s" % (trashfilename, e))
+            self.logger.error(f"{suspect.id} could not create file {trashfilename}: {e.__class__.__name__}: {str(e)}")
 
         # TODO: document main.trashlog
         if self.config.has_option('main', 'trashlog') and self.config.getboolean('main', 'trashlog'):
             try:
                 with open('%s/00-fuglutrash.log' % self.config.get('main', 'trashdir'), 'a') as handle:
                     # <date> <time> <from address> <to address> <plugin that said "DELETE"> <filename>
-                    now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+                    now = utcnow().isoformat()
                     handle.write("%s %s %s %s %s" % (
                         now, suspect.from_address, suspect.to_address, killerplugin, trashfilename))
                     handle.write("\n")
             except Exception as e:
-                self.logger.error("Could not update trash log: %s" % e)
+                self.logger.error(f'{suspect.id} Could not update trash log: {e.__class__.__name__}: {str(e)}')
 
     def run_plugins(self, suspect, pluglist):
         """Run scannerplugins on suspect"""
@@ -613,9 +591,8 @@ class SessionHandler(TrackTimings):
         self.tracktime("Before-Plugins")
         for plugin in pluglist:
             try:
-                self.logger.debug('Running plugin %s' % plugin)
-                self.set_workerstate(
-                    "%s : Running Plugin %s" % (suspect, plugin))
+                self.logger.debug(f'{suspect.id} Running plugin {str(plugin)}')
+                self.set_workerstate("%s : Running Plugin %s" % (suspect, plugin))
                 suspect.debug('Running plugin %s' % str(plugin))
                 starttime = time.time()
                 ans = plugin.run_examine(suspect)
@@ -633,47 +610,38 @@ class SessionHandler(TrackTimings):
                 suspect.tags['decisions'].append((plugin.section, result))
 
                 if result == DUNNO:
-                    suspect.debug('Plugin makes no final decision')
+                    suspect.debug(f'Plugin {str(plugin)} makes no final decision')
                 elif result == ACCEPT:
-                    suspect.debug(
-                        'Plugin accepts the message - skipping all further tests')
-                    self.logger.debug(
-                        'Plugin says: ACCEPT. Skipping all other tests')
+                    suspect.debug(f'Plugin {str(plugin)} accepts the message - skipping all further tests')
+                    self.logger.debug(f'{suspect.id} Plugin {str(plugin)} says: ACCEPT. Skipping all other tests')
                     self.action = ACCEPT
                     break
                 elif result == DELETE:
-                    suspect.debug(
-                        'Plugin DELETES this message - no further tests')
-                    self.logger.debug(
-                        'Plugin says: DELETE. Skipping all other tests')
+                    suspect.debug(f'Plugin {str(plugin)} DELETES this message - no further tests')
+                    self.logger.debug(f'{suspect.id} Plugin {str(plugin)} says: DELETE. Skipping all other tests')
                     self.action = DELETE
                     self.message = message
                     self.trash(suspect, str(plugin))
                     break
                 elif result == REJECT:
-                    suspect.debug(
-                        'Plugin REJECTS this message - no further tests')
-                    self.logger.debug(
-                        'Plugin says: REJECT. Skipping all other tests')
+                    suspect.debug(f'Plugin {str(plugin)} REJECTS this message - no further tests')
+                    self.logger.debug(f'{suspect.id} Plugin {str(plugin)} says: REJECT. Skipping all other tests')
                     self.action = REJECT
                     self.message = message
                     break
                 elif result == DEFER:
-                    suspect.debug(
-                        'Plugin DEFERS this message - no further tests')
-                    self.logger.debug(
-                        'Plugin says: DEFER. Skipping all other tests')
+                    suspect.debug(f'Plugin {str(plugin)} DEFERS this message - no further tests')
+                    self.logger.debug(f'{suspect.id} Plugin {str(plugin)} says: DEFER. Skipping all other tests')
                     self.action = DEFER
                     self.message = message
                     break
                 else:
-                    self.logger.error(
-                        'Invalid Message action Code: %s. Using DUNNO' % result)
+                    self.logger.error(f'{suspect.id} Plugin {str(plugin)} says invalid message action code: {result}. Using DUNNO')
 
             except Exception as e:
                 CrashStore.store_exception()
                 exc = traceback.format_exc()
-                self.logger.error('%s Plugin %s failed: %s' % (suspect.id, str(plugin), exc))
+                self.logger.error(f'{suspect.id} Plugin {str(plugin)} failed: {str(exc)}')
                 suspect.debug('Plugin failed : %s . Please check fuglu log for more details' % e)
                 ptag = suspect.get_tag("processingerrors", defaultvalue=[])
                 ptag.append("Plugin %s failed: %s" % (str(plugin), str(e)))
@@ -689,9 +657,8 @@ class SessionHandler(TrackTimings):
         self.tracktime("Before-Prependers")
         for plugin in self.prependers:
             try:
-                self.logger.debug('Running prepender %s' % plugin)
-                self.set_workerstate(
-                    "%s : Running Prepender %s" % (suspect, plugin))
+                self.logger.debug(f'{suspect.id} Running prepender {str(plugin)}')
+                self.set_workerstate("%s : Running Prepender %s" % (suspect, plugin))
                 starttime = time.time()
                 
                 out_plugins = plugin.pluginlist(suspect, plugcopy)
@@ -707,11 +674,9 @@ class SessionHandler(TrackTimings):
                     removed = list(plugcopyset - resultset)
                     added = list(resultset - plugcopyset)
                     if len(removed) > 0:
-                        self.logger.debug(
-                            'Prepender %s removed plugins: %s' % (plugin, list(map(str, removed))))
+                        self.logger.debug(f'{suspect.id} Prepender {str(plugin)} removed plugins: {", ".join(list(map(str, removed)))}')
                     if len(added) > 0:
-                        self.logger.debug(
-                            'Prepender %s added plugins: %s' % (plugin, list(map(str, added))))
+                        self.logger.debug(f'{suspect.id} Prepender {str(plugin)} added plugins: {", ".join(list(map(str, added)))}')
                     plugcopy = out_plugins
                     
                 # Appenders
@@ -721,18 +686,15 @@ class SessionHandler(TrackTimings):
                     removed = list(appcopyset - resultset)
                     added = list(resultset - appcopyset)
                     if len(removed) > 0:
-                        self.logger.debug(
-                            'Prepender %s removed appender: %s' % (plugin, list(map(str, removed))))
+                        self.logger.debug(f'{suspect.id} Prepender {str(plugin)} removed appender: {", ".join(list(map(str, removed)))}')
                     if len(added) > 0:
-                        self.logger.debug(
-                            'Prepender %s added appender: %s' % (plugin, list(map(str, added))))
+                        self.logger.debug(f'{suspect.id} Prepender {str(plugin)} added appender: {", ".join(list(map(str, added)))}')
                     appcopy = out_appenders
 
             except Exception as e:
                 CrashStore.store_exception()
                 exc = traceback.format_exc()
-                self.logger.error(
-                    'Prepender plugin %s failed: %s' % (str(plugin), exc))
+                self.logger.error(f'{suspect.id} Prepender plugin {str(plugin)} failed: {str(exc)}')
                 ptag = suspect.get_tag("processingerrors", defaultvalue=[])
                 ptag.append("Prepender %s failed: %s" % (str(plugin), str(e)))
                 suspect.set_tag("processingerrors", ptag)
@@ -748,10 +710,9 @@ class SessionHandler(TrackTimings):
         self.tracktime("Before-Appenders")
         for plugin in applist:
             try:
-                self.logger.debug('Running appender %s' % plugin)
+                self.logger.debug(f'{suspect.id} Running appender {str(plugin)}')
                 suspect.debug('Running appender %s' % plugin)
-                self.set_workerstate(
-                    "%s : Running appender %s" % (suspect, plugin))
+                self.set_workerstate("%s : Running appender %s" % (suspect, plugin))
                 starttime = time.time()
                 plugin.process(suspect, finaldecision)
                 plugintime = time.time() - starttime
@@ -759,8 +720,7 @@ class SessionHandler(TrackTimings):
             except Exception as e:
                 CrashStore.store_exception()
                 exc = traceback.format_exc()
-                self.logger.error(
-                    'Appender plugin %s failed: %s' % (str(plugin), exc))
+                self.logger.error(f'{suspect.id} Appender plugin {str(plugin)} failed: {str(exc)}')
                 ptag = suspect.get_tag("processingerrors", defaultvalue=[])
                 ptag.append("Appender %s failed: %s" % (str(plugin), str(e)))
                 suspect.set_tag("processingerrors", ptag)
