@@ -59,10 +59,25 @@ def fromfixedfile():
     try:
         with open(fixedversionfile, 'r', encoding='utf-8') as f:
             lines = f.readlines()
-        lines = [l for l in lines if l and l.strip()] if lines else []
+        lines = [l.strip() for l in lines if l and l.strip()] if lines else []
         versionstring = lines[0] if lines else None
     except Exception:
         versionstring = None
+    if versionstring and "-" in versionstring:
+        # if version string has been automatically created by git describe, it is of the form
+        # 1.1.0-ddd-gaaaaaaaa
+        #   -> 1.1.0 : closest tag
+        #   -> ddd : distance to closest tag
+        #   -> gaaaaaaaa: git commit
+        try:
+            public, local = versionstring.split("-", maxsplit=1)
+            local = local.replace("-", ".")
+            versionstring = f"{public}+{local}"
+            # this should result in a versionstring like
+            # 1.1.0+ddd.gaaaaaaaa
+            # which is pep 440 compatible because ddd.gaaaaaaaa is a local version identifier
+        except Exception:
+            pass
     return versionstring
 
 
